package heasarc.util;

/** This class is a simple implementation
 *  of a 'Vector' class for doubles.
 *  However, the only functions supported
 *  are adding an element to the end of the
 *  vector, and transforming the vector into
 *  an array.  This is intended to support applications
 *  where an vector will be grown until it's final
 *  size is known at which point it can be treated
 *  as a standard array.
 */

public class DoubleVector {
    
    /** Number of elements used in the vector. */
    int      used = 0;
    
    /** Vector of doubles. */
    double[] vector;
    
    /** Default size */
    public DoubleVector() {
	this(1024);
    }
    
    /** User specified initial size */
    public DoubleVector(int size) {
	used    = 0;
	vector  = new double[size];
    }
    
    /** Add an element to the vector. */
    public void add(double val) {
	if (used == vector.length) {
	    double[] xvec = new double[2*used];
	    System.arraycopy(vector, 0, xvec, 0, used);
	    vector = xvec;
	    xvec   = null;
	}
	vector[used] = val;
	used += 1;
    }
    
    /** Convert the vector to an array */
    public double[] toArray() {
	double[] xvec = new double[used];
	System.arraycopy(vector, 0, xvec, 0, used);
	return xvec;
    }
}
	 
	
	
