package heasarc.w3browse.plot;

/**
 *           W3Browse Plotting Package                          **
 *                                                              **
 * SOURCE: 
 *                                                              **
 *                                                              **
 *****************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.applet.*;
import heasarc.w3browse.*;
import javax.swing.*;

/** CardPanel class creates the panel which switches between plot
 *  options and the actual plot
 */
public class PlotOptions extends JPanel implements ActionListener, 
						   MouseListener    {

  /////////////////////////////////////////////////////////////////
  ///                   class variables                         ///
  /** The Panel Layout
   */
  protected BorderLayout layout;

  /** The panel that contains the plot */
  protected PlotPane   pPane;

  /** Plot specifications */
  public PlotSpecs plotspecs=null;

  /** Error Dialog */
  public ErrorDialog errDialog;

  /** Indicates if the plotting is an applet */
  public boolean isApplet;

  /** Context of applet */
  public AppletContext appletC;

  /** Parameter name for ra in table */
  private String ra;

  /** Parameter name for dec in table */
  private String dec;

  //private JButton plot; 
  private JButton submitB; 

  /** URL query to get data */
  private String url;

  /** sub-Panel for plot type selections                         */
  private JPanel plotTypeSelect;
  /** sub-Panel for plot button                                  */
  private JPanel submitP;
  /** sub-Panel for x and y plot axis options                    */
  private JPanel axisPanel;
  /** sub-Panel for x and y Log options                          */
  private JPanel secXFlow;
  private JPanel secYFlow;
  /** sub-Panel that displays mouseover help                     */
  private JPanel helpText;
  /** sub-Panel that needs to be disabled with some plot options */
  public JPanel sampleP;  


  /** buttons for plot type selection                            */
  private JRadioButton scatter,histogram, cumulative, aitoff, lineplot;
  /** group to process of plot type selections                   */
  private ButtonGroup  bg;
 
  /** label and text are for histogram binning                   */
  private JLabel       numberofBinsL;
  private JTextField   numberofBinsText;
  /** various labels used in interface                           */
  private JLabel       logL,minL,maxL,paramL,xL,yL;
  private JLabel       yOption2L,yOption3L;

  /** selection list for the x and y axises and sampling         */
  private JComboBox    xaxis, yaxis;
  private JComboBox    yaxis2,yaxis3;  // 2nd and 3rd set of y fields to plot
  private JComboBox    cfields;

  /** text fields for x and y axis max and min values             */
  private JTextField   minXField,maxXField,minYField,maxYField;
  private JTextField   minYField2,maxYField2; // second set of y fields to plot
  private JTextField   minYField3,maxYField3; // third set of y fields to plot

  /** Checkbox indicating if sampling should be done              */
  protected JCheckBox isSample;

  /** Checkbox for selecting log y and x axis                     */
  private JCheckBox    isLogx,isLogy;

  /** Checkbox for adding more y axis plotting                    */
  private JCheckBox    yOption2,yOption3;

  /** Checkbox for adding errorbars                               */
  private JCheckBox    showErrorB;
  private JPanel       showErrorP; 

  
  private String       fieldnames[];
  
  /** area that displays mouseover help                           */
  private JTextArea    helpArea;
  private HeasarcQuery hquery;

  /** Sampling settings */
  protected TextField sampleText;


  // Class to handle functionality of checkboxes  
  ItemListener RBhandler = new RadioButtonHandler();              
  ItemListener CBhandler = new CheckBoxHandler();              

  /////////////////////////////////////////////////////////////////
  ///                      constructor                          ///
  /** constructor to create Plot Options Panel                   */
  public PlotOptions(HeasarcQuery hquery, PlotSpecs plotspecs,boolean isApplet, 
                   String ra, String dec, String url, AppletContext appletC) {
      
    this.plotspecs = plotspecs;
    this.isApplet = isApplet;
    this.ra  = ra;
    this.dec = dec;
    this.url = url;
    this.appletC = appletC;
    this.hquery=hquery;

    //--- Create some arrays of fields returned from query
    String[] nn = null;
    String[] names = null;
    try {
      nn = hquery.getNumericNames();
      names = hquery.getNames(false);
    } catch (Exception e) {
      errDialog.display("Could not get numeric column names." +
	" Try changing your query.");   
    }


    //---  Create a window for errors
    errDialog = new ErrorDialog(isApplet);
    errDialog.setVisible(false);

    //------------------------------------------------------------
    //--- Build Plot Options Panel -------------------------------

    setLayout(new BorderLayout());
    setBackground(Color.white); 


    //---- Build Plot Button ---
    buildSubmit();

    //--- Build Plot Type Buttons ---
    //--- Initialize Plot type selection checkboxes with titles
    scatter    = new JRadioButton("Scatter");
    scatter.setSelected(true);
    lineplot   = new JRadioButton("Line");
    histogram  = new JRadioButton("Histogram");
    cumulative = new JRadioButton("Cumulative Histogram");
    aitoff     = new JRadioButton("Aitoff");

    buildPlotType();

    //--- Build Plot Axis Options ---
    buildAxisChoice(nn);

    //--- Build Plot Sampling Options ---
    buildSampleChoice(names);

    //--- Put axis and sampling options together
    JPanel axisSampling = new JPanel(new GridLayout(2,1));
    axisSampling.add(axisPanel);
    axisSampling.add(sampleP);

    axisSampling.setBorder(BorderFactory.createTitledBorder("Plot Parameters"));

    //--- Build Area for mouseover help ---
    buildHelp();

    add(submitP, BorderLayout.NORTH);

    add(plotTypeSelect, BorderLayout.WEST);
    add(axisSampling, BorderLayout.CENTER);
    add(helpText, BorderLayout.SOUTH);
  }  //--- end PlotOptions constuctor

  /////////////////////////////////////////////////////////////////
  ///                      methods                              ///
    
  String getRA() {
      return ra;
  }
    
  String getDec() {
      return dec;
  }

private void buildPlotType() {

  plotTypeSelect = new JPanel();
  plotTypeSelect.setLayout(new GridLayout(2,1));
  JPanel plotTypeSelectTop = new JPanel();
  plotTypeSelectTop .setLayout(new GridLayout(5,1));

  //--- create group for plot type buttons
  bg = new ButtonGroup();
  bg.add(scatter);
  bg.add(lineplot);
  bg.add(histogram);
  bg.add(cumulative);
  bg.add(aitoff);

  //--- Scatter button panel
  JPanel scatterP= new JPanel(new FlowLayout(FlowLayout.LEFT));
  scatterP.add(scatter).setBackground(Color.white);
  plotTypeSelectTop.add(scatterP).setBackground(Color.white);

  //--- Line plot button panel
  JPanel lineplotP= new JPanel(new FlowLayout(FlowLayout.LEFT));
  lineplotP.add(lineplot).setBackground(Color.white);;
  plotTypeSelectTop.add(lineplotP).setBackground(Color.white);

  //--- Histogram button and # of bins option
  JPanel binPanel= new JPanel(new FlowLayout(FlowLayout.LEFT));
  binPanel.add(histogram).setBackground(Color.white);
  numberofBinsL    = new JLabel("# of Bins:");
  numberofBinsL.addMouseListener(this);
  numberofBinsText = new JTextField(5);
  binPanel.add(numberofBinsL);
  binPanel.add(numberofBinsText);
  plotTypeSelectTop.add(binPanel).setBackground(Color.white);

  //--- Cumulative histogram plot button panel
  JPanel cumulativeP= new JPanel(new FlowLayout(FlowLayout.LEFT));
  cumulativeP.add(cumulative).setBackground(Color.white);
  plotTypeSelectTop.add(cumulativeP).setBackground(Color.white);

  //--- Aitoff plot button panel
  JPanel aitoffP= new JPanel(new FlowLayout(FlowLayout.LEFT));
  aitoffP.add(aitoff).setBackground(Color.white);
  plotTypeSelectTop.add(aitoffP).setBackground(Color.white);

  plotTypeSelect.add(plotTypeSelectTop);
  plotTypeSelect.setBorder(BorderFactory.createTitledBorder("Plot Type"));
 
  //--- Assign event and mouseover handlers
  scatter.addItemListener(RBhandler);
  scatter.addMouseListener(this);
  scatter.addActionListener(this);
  lineplot.addItemListener(RBhandler);
  lineplot.addMouseListener(this);
  lineplot.addActionListener(this);
  histogram.addItemListener(RBhandler);
  histogram.addMouseListener(this);
  histogram.addActionListener(this);
  cumulative.addItemListener(RBhandler);
  cumulative.addMouseListener(this);
  cumulative.addActionListener(this);
  aitoff.addItemListener(RBhandler);
  aitoff.addMouseListener(this);
  aitoff.addActionListener(this);
}  //--- end buildPlotType

private void  buildSubmit() {
  submitP = new JPanel();
  submitB = new JButton("Plot");
  submitP.setLayout(new FlowLayout());
  submitB.addMouseListener(this);
  submitB.addActionListener(this);
  submitP.add(submitB);
}

private void  buildHelp() {
  helpText = new JPanel();
  helpArea = new JTextArea("Move the cursor over buttons and text for more information");
  helpText.add(helpArea);
}


/** This routine adds the panel which displays axis information */
  private void buildAxisChoice(String fields[]) {
    String dftra= "ra";
    String dftdec= "dec";
    axisPanel = new JPanel();
    axisPanel.setLayout(new GridLayout(5,4));

    //----------------------------------------------------------
    //--- Plot Parameters headers
    logL = new JLabel("Log?");
    logL.setFont(new Font("Courier", Font.BOLD, 12));
    logL.addMouseListener(this);

    paramL = new JLabel("Parameters");
    paramL.setFont(new Font("Courier", Font.BOLD, 12));
    paramL.addMouseListener(this);

    minL = new JLabel("Min (Scatter or Line)");
    minL.setFont(new Font("Courier", Font.BOLD, 12));
    minL.addMouseListener(this);

    maxL = new JLabel("Max (Scatter or Line)");
    maxL.setFont(new Font("Courier", Font.BOLD, 12));
    maxL.addMouseListener(this);

    axisPanel.add(logL);
    axisPanel.add(paramL);
    axisPanel.add(minL);
    axisPanel.add(maxL);
    //----------------------------------------------------------
    //--- X Log checkbox, X axis fields, X axis min, X axis max

    secXFlow = new JPanel();
    isLogx = new JCheckBox();
    secXFlow.setLayout(new FlowLayout(FlowLayout.LEFT));

    xL = new JLabel("X: ");
    xL.addMouseListener(this);
    secXFlow.add(xL);

    secXFlow.add(isLogx);

    JPanel xaxisP= new JPanel();
    xaxis = fieldChoice(fields,ra);
    xaxis.addActionListener(this);
    xaxisP.add(xaxis);
    xaxisP.setLayout(new FlowLayout(FlowLayout.LEFT));

    Column col = null;   //-- used to get min and max values for plot fields

    JPanel minXP= new JPanel();
    minXP.setLayout(new FlowLayout(FlowLayout.LEFT));
    minXField = new JTextField("",8);
    minXP.add(minXField);

    JPanel maxXP= new JPanel();
    maxXP.setLayout(new FlowLayout(FlowLayout.LEFT));
    maxXField = new JTextField("",8);
    maxXP.add(maxXField);

    try {
       col = hquery.getColumn(getXAxis());
    } catch  (Exception e) {
       errDialog.display("Unable to get min value for column " + xaxis + 
		" from query: " + e);
    }
    if (col !=null) {
       minXField.setText(String.valueOf(col.getMin()));
       maxXField.setText(String.valueOf(col.getMax()));
    }  

    axisPanel.add(secXFlow);
    axisPanel.add(xaxisP);
    axisPanel.add(minXP);
    axisPanel.add(maxXP);

    //----------------------------------------------------------
    //--- Y Log checkbox, Y axis fields, Y axis min, Y axis max
    secYFlow = new JPanel();
    isLogy = new JCheckBox();
    secYFlow.setLayout(new FlowLayout(FlowLayout.LEFT));
    yL = new JLabel("Y: ");
    yL.addMouseListener(this);
    secYFlow.add(yL);
    secYFlow.add(isLogy);

    JPanel yaxisP= new JPanel();
    yaxisP.setLayout(new FlowLayout(FlowLayout.LEFT));
    yaxis = fieldChoice(fields,dec);
    yaxis.addActionListener(this);
    yaxisP.add(yaxis);


    JPanel minYP= new JPanel();
    minYP.setLayout(new FlowLayout(FlowLayout.LEFT));
    minYField = new JTextField("",8);
    minYP.add(minYField);

    JPanel maxYP= new JPanel();
    maxYP.setLayout(new FlowLayout(FlowLayout.LEFT));
    maxYField = new JTextField("",8);
    maxYP.add(maxYField);

    col=null;
    try {
       col = hquery.getColumn(getYAxis());
    } catch  (Exception e) {
       errDialog.display("Unable to get min value for y axis column "  + 
		" from query: " + e);
    }
    if (col !=null) {
       minYField.setText(String.valueOf(col.getMin()));
       maxYField.setText(String.valueOf(col.getMax()));
    }  

    axisPanel.add(secYFlow);
    axisPanel.add(yaxisP);
    axisPanel.add(minYP);
    axisPanel.add(maxYP);

    //----------------------------------------------------------
    //--- 2nd Set Y Log checkbox, Y axis fields, Y axis min, Y axis max
    if (fields.length > 1) {

       //-- label
       JPanel yOption2P= new JPanel();
       yOption2L = new JLabel("Optional 2nd Dataset ");
       yOption2L.addMouseListener(this);
       yOption2 = new JCheckBox();
       yOption2.addItemListener(CBhandler);
       yOption2P.add(yOption2L);
       yOption2P.add(yOption2);
       yOption2.setEnabled(true);

       //-- checkbox
       JPanel yaxis2P= new JPanel();
       yaxis2P.setLayout(new FlowLayout(FlowLayout.LEFT));
       yaxis2 = fieldChoice(fields,"");
       yaxis2.addActionListener(this);
       yaxis2.setEnabled(false);
       yaxis2.addMouseListener(this);
       yaxis2P.add(yaxis2);
       yaxis.setEnabled(true);


       //--- editable parameter min and max fields
       JPanel minY2P= new JPanel();
       minY2P.setLayout(new FlowLayout(FlowLayout.LEFT));
       minYField2 = new JTextField("",8);
       minY2P.add(minYField2);
       minYField2.setEnabled(true);

       JPanel maxY2P= new JPanel();
       maxY2P.setLayout(new FlowLayout(FlowLayout.LEFT));
       maxYField2 = new JTextField("",8);
       maxY2P.add(maxYField2);
       maxYField2.setEnabled(true);

       axisPanel.add(yOption2P);  
       axisPanel.add(yaxis2P);
       axisPanel.add(minY2P);
       axisPanel.add(maxY2P);
       
    }
    //----------------------------------------------------------
    //--- 3nd Set Y Log checkbox, Y axis fields, Y axis min, Y axis max
    if (fields.length > 2) {
       //-- label
       JPanel yOption3P= new JPanel();
       yOption3L = new JLabel("Optional 3nd Dataset ");
       yOption3L.addMouseListener(this);

       //-- checkbox
       yOption3 = new JCheckBox();
       yOption3.addItemListener(CBhandler);
       yOption3P.add(yOption3L);
       yOption3P.add(yOption3);

       //--- parameter selection
       JPanel yaxis3P= new JPanel();
       yaxis3P.setLayout(new FlowLayout(FlowLayout.LEFT));
       yaxis3 = fieldChoice(fields,"");
       yaxis3.addActionListener(this);
       yaxis3.addMouseListener(this);
       yaxis3P.add(yaxis3);
       yaxis3.setEnabled(false);

       //--- editable parameter min and max fields
       JPanel minY3P= new JPanel();
       minY3P.setLayout(new FlowLayout(FlowLayout.LEFT));
       minYField3 = new JTextField("",8);
       minY3P.add(minYField3);

       JPanel maxY3P= new JPanel();
       maxY3P.setLayout(new FlowLayout(FlowLayout.LEFT));
       maxYField3 = new JTextField("",8);
       maxYField3.setEnabled(false);
       maxY3P.add(maxYField3);

       axisPanel.add(yOption3P);  // blank grid column
       axisPanel.add(yaxis3P);
       axisPanel.add(minY3P);
       axisPanel.add(maxY3P);
    }
  }

/** Uses the field_name[] array to create a choice component which
   * contains a list of all field names
   * @return Choice component containing a list of field name
   */
  private JComboBox fieldChoice(String[] fieldnames, String dft) {
      
    boolean hasDft = false;

    JComboBox ch = new JComboBox();
      
    for(int i=0;i<fieldnames.length;i++) {
	if (fieldnames[i].equals(dft)) {
	    hasDft = true;
	}
	if (fieldnames[i].length() < 3 || 
		!fieldnames[i].substring(0,3).equals("w3_")) {
            ch.addItem(fieldnames[i]);
	}
    }
      
    ch.setSelectedItem(dft);
      
    this.fieldnames = fieldnames;
    return(ch);
  } //--- end fieldChoice()


/** Changes the help text when the user moves the mouse over a label */
  public void mouseEntered(MouseEvent e) {
    if(e.getComponent() ==scatter) {
      helpArea.setText("  Creates a scatter plot.");
    } else if(e.getComponent() ==lineplot) {
      helpArea.setText("  Creates a line plot.");
    } else if (e.getComponent() == cumulative) {
      helpArea.setText("  Creates a cumulative histogram.");
    } else if (e.getComponent() == numberofBinsL) {
      helpArea.setText("  Sets the number of bin containers for a " +
	"histogram plot.");
    } else if (e.getComponent() == histogram) {
      helpArea.setText("  Creates a histogram plot.");
    } else if (e.getComponent() == aitoff) {
      helpArea.setText("  Creates an aitoff projection.");
    } else if (e.getComponent() == logL) {
      helpArea.setText("  Click on the below check boxes if you want " +
	"to plot the log values for x or y axis.");
    } else if (e.getComponent() == minL) {
      helpArea.setText("  User can set the minimum value for plot bounds.");
    } else if (e.getComponent() == maxL) {
      helpArea.setText("  User can set the maximum value for plot bounds.");
    } else if (e.getComponent() == paramL) {
      helpArea.setText("  List of parameters that the user can plot. " +
	"Select one each for X and Y axis.");
    } else if (e.getComponent() == xL) {
      helpArea.setText("  X Axis plottable information.");
    } else if (e.getComponent() == yL) {
      helpArea.setText("  Y Axis plottable information.");
    } else if (e.getComponent() == showErrorP) {
      helpArea.setText("  Show Error Bars for plotted data sets");
    } else if (e.getComponent() == yOption2L) {
      helpArea.setText("  Add additional set of points to plot");
    } else if (e.getComponent() == yOption3L) {
      helpArea.setText("  Add additional set of points to plot");
    } else if (e.getComponent() == submitB) {
      helpArea.setText("  Plot using current selections");
    } else if (e.getComponent() == isSample || e.getComponent() == cfields || 
               e.getComponent() == sampleText ) {
      helpArea.setText("  Sample data using a parameter value or range");
    }
  }
  /** Required to support MouseListener */
  public void mousePressed(MouseEvent e) {}
  /** Required to support MouseListener */
  public void mouseReleased(MouseEvent e) {}
  /** Required to support MouseListener */
  public void mouseClicked(MouseEvent e) {}
  /** Required to support MouseListener */
  public void mouseExited(MouseEvent e) {}


public void actionPerformed(java.awt.event.ActionEvent e) {
   if (e.getActionCommand() == "Plot") {
      //--- Gather plot options and create a new plot
      try {
       plotspecs= new PlotSpecs(this,ra,dec,url); 
       pPane.plot(plotspecs);
      } catch(Exception ee) {
        errDialog.display("A plotting error has occured::" +ee.getMessage());
      }
    } else  if (e.getActionCommand() == "Aitoff" ) {
       //--- Disable most options if Aitoff selected
       isLogx.setSelected(false);
       isLogy.setSelected(false);
       isLogx.setEnabled(false);
       isLogy.setEnabled(false);
       showErrorB.setSelected(false);
       showErrorB.setEnabled(false);
       yOption2.setSelected(false);
       yOption2.setEnabled(false);
       yaxis2.setEnabled(false);
       minYField2.setEnabled(false);
       maxYField2.setEnabled(false);
       yOption3.setSelected(false);
       yOption3.setEnabled(false);
       yaxis3.setEnabled(false);
       minYField3.setEnabled(false);
       maxYField3.setEnabled(false);
    } else  if (e.getActionCommand() == "Histogram" ||
	        e.getActionCommand() == "Cumulative Histogram" ) {
       //--- Disable most options if Histogram selected
       isLogx.setEnabled(true);
       isLogy.setSelected(false);
       isLogy.setEnabled(false);
       isSample.setSelected(false);
       isSample.setEnabled(false);
       showErrorB.setSelected(false);
       showErrorB.setEnabled(false);
       yOption2.setSelected(false);
       yOption2.setEnabled(false);
       yaxis2.setEnabled(false);
       minYField2.setEnabled(false);
       maxYField2.setEnabled(false);
       yOption3.setSelected(false);
       yOption3.setEnabled(false);
       yaxis3.setEnabled(false);
       minYField3.setEnabled(false);
       maxYField3.setEnabled(false);
    } else  if (e.getActionCommand() == "Scatter" ||
                e.getActionCommand() == "Line") {
       //--- Enable  options if Scatter or Line selected
       isLogy.setEnabled(true);
       isLogx.setEnabled(true);
       isSample.setEnabled(true);
       showErrorB.setEnabled(true);
       yOption2.setEnabled(true);
       yOption3.setEnabled(true);
    } else  if ("comboBoxChanged".equals(e.getActionCommand())) {
       //--- Display and fill min and max values if optional y 
       //--- parameters are selected
       JComboBox source = (JComboBox)e.getSource();

       if (source == xaxis) {

          Column col=null;
          try {
             col = hquery.getColumn(getXAxis());
          } catch  (Exception ee) {
             errDialog.display("Unable to get values for x axis column "  +
		" from query: " + ee);
          }
          if (col !=null) {
             minXField.setText(String.valueOf(col.getMin()));
             maxXField.setText(String.valueOf(col.getMax()));
          }  
          
       } else if (source == yaxis) {
          Column col=null;
          try {
             col = hquery.getColumn(getYAxis());
          } catch  (Exception ee) {
             errDialog.display("Unable to get values for y axis column "  +
		" from query: " + ee);
          }
          if (col !=null) {
             minYField.setText(String.valueOf(col.getMin()));
             maxYField.setText(String.valueOf(col.getMax()));
          }  
          
       } else if (source == yaxis2) {
          Column col=null;
          try {
             col = hquery.getColumn(getY2Axis());
          } catch  (Exception ee) {
             errDialog.display("Unable to get values for 2nd y axis column "  +
		" from query: " + ee);
          }
          if (col !=null) {
             minYField2.setText(String.valueOf(col.getMin()));
             maxYField2.setText(String.valueOf(col.getMax()));
          }  
          
       } else if (source == yaxis3) {
          Column col=null;
          try {
             col = hquery.getColumn(getY3Axis());
          } catch  (Exception ee) {
             errDialog.display("Unable to get values for 3rd y axis column  " +
		" from query: " + ee);
          }
          if (col !=null) {
             minYField3.setText(String.valueOf(col.getMin()));
             maxYField3.setText(String.valueOf(col.getMax()));
          }  
          
       }
    }
}

 // Inner class that handles functionality for the check boxes.
 private class CheckBoxHandler implements ItemListener 
 { // Opens inner class
       
      public void itemStateChanged (ItemEvent e) { 
         // Retrieves object that fires an event, casts it as a 
         // JCheckbox object and assigns it to source.
         JCheckBox source = (JCheckBox) e.getSource();
         // If the object is selected . . .
         if (source == yOption2) {
            if ( e.getStateChange() == ItemEvent.SELECTED ) {
               //--- if optional y parameter is selected enable all
               //--- related components and display max and min values
               yaxis2.setEnabled(true);
               minYField2.setEnabled(true);
               maxYField2.setEnabled(true);
               Column col=null;
               try {
                  col = hquery.getColumn(getY2Axis());
               } catch  (Exception ee) {
                  errDialog.display("Unable to get values for 2nd y axis "  +
		           "column  from query: " + ee);
               }
               if (col !=null) {
                  minYField2.setText(String.valueOf(col.getMin()));
                  maxYField2.setText(String.valueOf(col.getMax()));
               }  

            } else {
               yaxis2.setEnabled(false);
               minYField2.setEnabled(false);
               maxYField2.setEnabled(false);
            }
         } else if (source == yOption3) {
            if ( e.getStateChange() == ItemEvent.SELECTED ) {
               //--- if optional y parameter is selected enable all
               //--- related components and display max and min values
               yaxis3.setEnabled(true);
               minYField3.setEnabled(true);
               maxYField3.setEnabled(true);
               Column col=null;
               try {
                  col = hquery.getColumn(getY3Axis());
               } catch  (Exception ee) {
                  errDialog.display("Unable to get values for 3rd y" +
		  " axis column from query: " + ee);
               }
               if (col !=null) {
                  minYField3.setText(String.valueOf(col.getMin()));
                  maxYField3.setText(String.valueOf(col.getMax()));
               }  
            } else {
               yaxis3.setEnabled(false);
               minYField3.setEnabled(false);
               maxYField3.setEnabled(false);
            }
         } else if (source == isSample) {
            if ( e.getStateChange() == ItemEvent.SELECTED ) {
               //--- if optional y parameter is selected enable all
               //--- related components and display max and min values
               //System.out.println("enabling text");
               sampleText.setEnabled(true);
               cfields.setEnabled(true);
            } else {
               sampleText.setEnabled(false);
               cfields.setEnabled(false);
            }
         } else if (source == yOption3) {
            if ( e.getStateChange() == ItemEvent.SELECTED ) {
            }
         }
     } // Closes ItemStateChanged method
 } // Closes class CheckBoxHandler

 // Inner class that handles functionality for the radio buttons.
 private class RadioButtonHandler implements ItemListener 
 { // Opens inner class
       
      public void itemStateChanged (ItemEvent e)
        { 
               // Retrieves object that fires an event, casts it as a 
               // JRadioButton object and assigns it to source.
               JRadioButton source = (JRadioButton) e.getSource();
               // If the object is selected . . .
               if ( e.getStateChange() == ItemEvent.SELECTED )
                 // change the foreground to blue
                 source.setForeground(Color.blue);
                 // otherwise set the foreground to black.
               else
                 source.setForeground(Color.black);
              
             } // Closes ItemStateChanged method
      } // Closes class radioButtonHandler


  /** Get yaxis index value */
  public String getXAxis() {
    return (String) xaxis.getSelectedItem();
  }

  /** Get yaxis index value */
  public String getYAxis() {
     return (String) yaxis.getSelectedItem();
  }

  /** Check if Log has been clicked */
  public boolean isXLog() {
    //return(isLogx.getState());
    return(isLogx.isSelected());
  }

  /** Check if Log has been clicked */
  public boolean isYLog() {
    //return(isLogy.getState());
    return(isLogy.isSelected());
  }
  /** returns the string equivalant of entered bin number 
   */
  public String getBin() {
    return numberofBinsText.getText();
  }
  /** returns string value in minXField 
   */
  public String getXmin() throws InputException{
    return minXField.getText();
  }
  /** returns string value in maxXField 
   */
  public String getXmax() throws InputException{
    return maxXField.getText();
  }
  /** returns string value in minYField 
   */
  public String getYmin() throws InputException{
    return minYField.getText();
  }
  /** returns string value in maxYField 
   */
  public String getYmax() throws InputException{
    return maxYField.getText();
  }

  /** Get optional yaxis index number */
  public String getY2Axis() {
     return (String) yaxis2.getSelectedItem();
  }

  /** Check if error bars box has been clicked */
  public boolean showErrorBars() {
    return(showErrorB.isSelected());
  }

  /** Check if 2nd Y axis option has been clicked */
  public boolean isYOption2() {
    return(yOption2.isSelected());
  }

  /** returns string value in optional 2nd maxYField */
  public String getY2max() throws InputException{
    return maxYField2.getText();
  }

  /** returns string value in optional 2nd minYField */
  public String getY2min() throws InputException{
    return minYField2.getText();
  }

  /** Get optional yaxis index number */
  public String getY3Axis() {
     return (String) yaxis3.getSelectedItem();
  }

  /** Check if 3nd Y axis option has been clicked */
  public boolean isYOption3() {
    return(yOption3.isSelected());
  }

  /** returns string value in optional 3nd maxYField */
  public String getY3max() throws InputException{
    return maxYField3.getText();
  }

  /** returns string value in optional 3nd minYField */
  public String getY3min() throws InputException{
    return minYField3.getText();
  }

  /** Returns a string to indicate the plot type */
  public HeasarcQuery getQuery() {
     return hquery;
  }

  /** Returns a string to indicate the plot type
   */
  public String getPlotType() {
    String type = new String();
    JRadioButton cq = getSelection(bg);

    if (cq.equals(scatter)) {
       type = new String("scatter");
    }
    if (cq.equals(lineplot)) {
       type = new String("lineplot");
    }
    else if (cq.equals(histogram)) {
       type = new String("histogram");
    }
    else if (cq.equals(cumulative)) {
       type = new String("cumulative");
    }
    else if (cq.equals(aitoff)) {
       type = new String("aitoff");
    }
    return(type);
  }
  
  public void setPlotPane (PlotPane  pPane) {
     this.pPane = pPane;
  }

// This method returns the selected radio button in a button group
    public static JRadioButton getSelection(ButtonGroup group) {
        for (Enumeration e=group.getElements(); e.hasMoreElements(); ) {
            JRadioButton b = (JRadioButton)e.nextElement();
            if (b.getModel() == group.getSelection()) {
                return b;
            }
        }
        return null;
    }
public void buildSampleChoice(String[] fields){

    sampleP = new JPanel();
    sampleP.setLayout(new FlowLayout(FlowLayout.LEFT));
    //sampleP.setLayout(new GridLayout(2,7));

    cfields  = sampleFieldNames(fields);
    cfields.addMouseListener(this);
    isSample = new JCheckBox();
    isSample.addMouseListener(this);
    sampleP.add(isSample);
    sampleP.add(new Label("Sample data by parameter "));
    sampleP.add(cfields);
    sampleP.add(new Label(" using intervals/value "));
    isSample.addItemListener(CBhandler);
    cfields.setEnabled(false);

    sampleText = new TextField(15);
    sampleText.addMouseListener(this);
    sampleP.add(sampleText);
    sampleText.setEnabled(false);
    showErrorP = new JPanel();
    showErrorB = new JCheckBox();
    showErrorP.add(showErrorB);
    showErrorP.add(new Label("Display error bars (where available)"));
    showErrorP.addMouseListener(this);
    sampleP.add(showErrorP);
    //sampleP.setBackground(new Color(220,200,200));
    sampleP.setBackground(Color.LIGHT_GRAY);

  } //thirdPanel constructor

  /////////////////////////////////////////////////////////////////
  ///                      methods                              ///
  /** Uses an array of fieldnames to create a choice component which
   * contains a list of all field names
   * @return Choice component containing a list of field name
   */
  private JComboBox sampleFieldNames(String [] fields) {

    JComboBox ch = new JComboBox();
    for (int i=0;i<fieldnames.length;i++) {
        if (!fieldnames[i].startsWith("w3_")) {
            ch.addItem(fieldnames[i]);
	}
    }
    return ch;
  }

  /** Sets the help area on mouseover events */
  /*
  public void setText(String help){
     helpText.setColumns(40);
     helpText.setText(help);
     helpText.setColumns(40);
  }
*/

  /** Returns isSample value */
  public boolean getIsSample() {
     return isSample.isSelected();
  }

  /** returns sample text contents */
  public String getSampleText() {
    return sampleText.getText();
  }

  /** returns selected parameter index */
  public String getParam () {
    return (String) cfields.getSelectedItem();
  }

  
}
