package heasarc.w3browse.plot;

/**
 *           W3Browse Plotting Package                          **
 *                                                              **
 * SOURCE: 
 *                                                              **
 *                                                              **
 *****************************************************************/

import java.util.*;
import java.applet.*;
import heasarc.w3browse.*;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.net.URL;
import java.net.URLEncoder;
import java.net.URLConnection;
import java.text.DecimalFormat;


/** CardPanel class creates the panel which switches between plot
 *  options and the actual plot
 */
public class PlotValues {

  /////////////////////////////////////////////////////////////////
  ///                   class variables                         ///
  private AppletContext appletC;
  private String URLparameter="";
  private String columnNames;
  private String host;
  private boolean isHistogram=false;
  /** Error Dialog */
  public ErrorDialog errDialog;

  /** constructor  */
  public PlotValues (AppletContext appletC, String url) {

     if (appletC != null) {errDialog = new ErrorDialog(true);}
     this.appletC = appletC;
     if (url.startsWith("http") ) {
        int first= url.indexOf('/');
        int startstr=first+2;
        String hostlong=url.substring(first+2, url.length());
        int next= hostlong.indexOf('/');
        this.host = hostlong.substring(0,next);
     }
     
  }
  /////////////////////////////////////////////////////////////////
  ///                      methods                              ///

  /** sets histogram flag   */
  public void setHistogram (boolean hist) { 
     this.isHistogram=hist;
  }
  /** Displays plot values  */
  public void updateValues (Column xcol, HashMap allYdata, 
	HashMap inRangeData,double [] sampledata, Column scol) {

     int numberOfDatasets = allYdata.size();

     String [] sampvals = null;
     double [] xdata = xcol.getData();      //--- x axis data 
     String xaxis    = xcol.getField();     //--- name of x axis field
     String xType    = xcol.getType();      //--- x value number type
     String sType;                          //--- sample value number type


     //--- retrieve sampling parameter values for each row
     if (sampledata != null) {
        sampvals = new String [sampledata.length];
        if (scol == null) {
           System.err.println("Warning: Sample data values array is null");
        } else {
           //--- Get number format of sampling field
           sType    = scol.getType();      

           if (scol.isNumeric) { 

              //--- Read data in as double array and convert to String
              double [] sd =  scol.getData();
              if (sampvals.length != sd.length) {
                  System.err.println("Warning: sampling array does not match " +
			"column data");
              }
              for (int i=0;i<sd.length;++i) {
                 sampvals[i] = convertNumber(sd[i], sType);
              }
           } else {

              //--- Read data in as string array 
              String [] sd = (String []) scol.data;
              if (sampvals.length != sd.length) {
                  System.err.println("Warning: sampling array does not match " +
			"column " + "data");
              }
              for (int i=0;i<sd.length;++i) {
                 sampvals[i] = sd[i];
              }
           } 
        }
     }

     //--- Start output table header string with x axis field
     columnNames=xaxis; 

     //--- Extract y axis, y error, daya types and display flag from 
     //--- hashmap structure
     double [][] ydatas= new double [numberOfDatasets][];
     double [][] errors= new double [numberOfDatasets][];
     String   [] yTypes= new String [numberOfDatasets];
     String   [] errorTypes= new String [numberOfDatasets];
     double [][] include= new double [numberOfDatasets][];

     //--- Prepare to extract data from hash
     Set keys=allYdata.keySet();
     Iterator it = keys.iterator();
     int count =0;

     //--- Go through y axis data and save column names, values, data types and
     //--- any error values and display flag
     while (it.hasNext()) {
        String key = (String) it.next();
        Column [] alldata = (Column []) allYdata.get(key);
        
        //--- flags indicating data values are in selected min/max range
        double [] inRangePoints = (double []) inRangeData.get(key);

        if (isHistogram) {
          columnNames +=","+"bin total";  //--- header for histogram values
        } else {
          columnNames +=","+key;  //--- save y axis colunm name for output
        }

        Column c=alldata[0];
        ydatas[count]  = c.getData();   //-- y data points
        yTypes[count]  = c.getType();   //-- y data points

        errors[count]  = null;   //-- error values, may be null
        c=alldata[1];
        if (c !=null ) {
           errors[count]     = c.getData();   //-- error values, may be null
           errorTypes[count] = c.getType();
        }

        include[count] = inRangePoints;   //-- y min/max range flags

        //--- if error data exists save it column name 
        if (errors[count] != null) {columnNames += "," +key+"_error";}

        ++count;

     }

     //--- add Sampling parameter name to table header after x and 
     //--- y axis parameter names
     if (sampledata != null && !isHistogram) {
            columnNames += "," +scol.getField()+ " (sample)";
     }
       
     int inRange;
     String sampString = null;

     //-- this will remain false if all y values are either not in range
     //-- or not in sample criteria
     boolean  includeRow=false;

     String rowValues=null;  //--- string of row x, y, error, sampling values

     for (int i=0;i < xdata.length;++i) {

        //--- Skip if NaN
        if (Double.isNaN(xdata[i])) {
           continue;
        }

        //--- include sampling value if sampling selected
        if (sampledata != null) {
           inRange = (int) sampledata[i] - 1;
           sampString = sampvals[i];
	   if (inRange < 0) {
	            continue;
	   }
        }

        //--- Add formatted x axis value to row string 
        rowValues = convertNumber(xdata[i],xType);


        //--- print values for y axis and y axis errors
        for (int j=0;j < count;j++) {

           //--- blank space if NaN
           if (Double.isNaN(ydatas[j][i])) {
              rowValues +=  ","+" ";   //-- value NaN so omit
              continue;
           }

           //-- blank space if not in  min/max range 
           if (include[j][i] < 0.) {
              rowValues +=  ","+" ";   
              continue;
           }
 
           //--- Add formatted y axis value to row values string
           rowValues +=  ","+convertNumber(ydatas[j][i],yTypes[j]); 
           includeRow = true;  //-- there is at least one value y value

           //--- Add y axis value error if it exists to row values string
           if (errors[j] != null) {

             //--- blank space if NaN
             if (Double.isNaN(errors[j][i])) {
                rowValues +=  ","+" ";
                continue;
             }

             //--- Add formatted error value to row values string
             rowValues +=  ","+convertNumber(errors[j][i],errorTypes[j]); 
           }
          
        }
        //--- Add sampling parameter value to row values string
        if (sampledata != null) {
            rowValues +=  ","+ sampString;
        }

        rowValues +=  "|"; //--- add delimiter

        //--- save string of values to pass on to web page
        if (includeRow) {
           URLparameter +=  rowValues;
        }

        includeRow = false;    //--- reset for next row of values
     }
  }

  /** action for save values button click */
  public void actionPerformed(java.awt.event.ActionEvent e) {
     if (e.getActionCommand() == "Save") {
       displayValuesHTML();
     }
  }


  public void displayValuesHTML() {
     String query = "";
     try {
        //--- query parameters
        query += "pvalues" + "=" + URLEncoder.encode( URLparameter,"UTF-8"); 
     } catch (Exception e) {
        
       errDialog.display("Error encoding plot values for query: " + e);
     }
     //--- more query parameters
     query += "&submit=Submit&columnNames="+columnNames;

     // Create HTML page and the display it
     try {
        URL u = new URL(
          "http://"+host+"/cgi-bin/W3Browse/w3plotvalues.pl");
        URLConnection urlc = u.openConnection();
        urlc.setDoOutput(true);
        urlc.setDoInput(true);
        urlc.setAllowUserInteraction(false);

        DataOutputStream server = 
          new DataOutputStream(urlc.getOutputStream());

        // Send the data
        server.writeBytes(query);
        server.close();
      
        BufferedReader in
          = new BufferedReader(new InputStreamReader(urlc.getInputStream()));
 
        String s;
        String url="";
        while((s = in.readLine()) != null) {
          if (s.compareTo("http") >= 0) {url=s;}
        }
        in.close();

        if (appletC != null) {
           appletC.showDocument(new URL(url),"plotvalues");
        }

        //-------------------------------------------------------------------
        //--- run cgi script that removes the html file we just created
        try {
           Thread.sleep(2000); /* call Java's sleep method */
        } catch (InterruptedException e)    {

        }
        //--- extract file name from full url
        int first= url.indexOf('/');
        int startstr=first+2;  //-- skip over http://
        String hostlong=url.substring(first+2, url.length());
        int next= hostlong.indexOf('/');
        String file = hostlong.substring(next,hostlong.length());

        //--- Call script with file name as parameter
        URL uf = new URL(
          "http://"+host+"/cgi-bin/W3Browse/w3plotfile.pl");
        URLConnection urlcf = uf.openConnection();
        urlcf.setDoOutput(true);
        urlcf.setDoInput(true);
        DataOutputStream serverf = 
          new DataOutputStream(urlcf.getOutputStream());

        // Send the data
        serverf.writeBytes("plotfile="+file);
        serverf.close();

        //--- check for any returned message
        BufferedReader inf = new BufferedReader(
		new InputStreamReader(urlcf.getInputStream()));
 
        //--- read return value in case there was a problem
        String ss;
        while((ss = inf.readLine()) != null) {
        }
        inf.close();
      } catch (Exception ee) {
         errDialog.display("Error displaying plot values: " + ee);
        
      }
    }
    
    /** Convert number to string in format specified by Browse data type */
    private String convertNumber(double number, String type) {
       DecimalFormat df;
       
       Double dd=new Double(number);
       String dstring= dd.toString();
       //System.err.println("PlotValues: dstring="+dstring + " type=" + type);

       if (type.equals("7") ) { 
	  df=  new DecimalFormat ( "00.000000000000000E00" );
          return df.format(number);   
       } else if (type.equals("6") || type.equals("Double")) {
          //--- Look for exponential format
          if (dstring.indexOf("e") > 0 || dstring.indexOf("E") > 0) {
	     df=  new DecimalFormat ( "00.000000000000000E00" );
             return df.format(number);   
          } 
	  df=  new DecimalFormat ( "#######.#########" );
          return df.format(number);   
       } else if (type.equals("1") || type.equals("4") || type.equals("5")) {
          Double d=new Double(number);
          return Integer.toString(d.intValue());   
       }
       return  String.valueOf(number);
        
    }

}

