package heasarc.w3browse.plot;

import java.util.*;


/** Create a sample from within an existing query.
 */
public class Sampler {
    
    Column    dataColumn;
    Checker[] checks;
    String [] crits;
    Object  sampdata;
    
    private abstract class Checker {
	abstract boolean check(int index);
    }
    
    private class EqualityChecker extends Checker {
	
	boolean isNumeric;
	
	String sval;
	String[] sdata;
	
	double dval;
	double[] ddata;
	
	EqualityChecker(String criterion) {
	    isNumeric = Sampler.this.dataColumn.isNumeric;
	    sampdata = dataColumn.data;
	    if (isNumeric) {
		dval  = new Double(criterion).doubleValue();
		ddata = (double[]) dataColumn.data;
	    } else { 
		sval = criterion;
		sdata = (String[]) dataColumn.data;
	    }
	}
	
        boolean check(int i) {
	    if (isNumeric) {
		return ddata[i] == dval;
	    } else {
		return sdata[i].equalsIgnoreCase(sval);
	    }
	}
	
    }
    
    private class RangeChecker extends Checker {
	
	boolean isNumeric;
	
	String sv1, sv2;
	String[] sdata;
	
        double dv1, dv2;
	double[] ddata;
	
	RangeChecker(String criterion, int index) {
	    
	    String v1 = criterion.substring(0,index);
	    String v2 = criterion.substring(index+2);
	    v1 = v1.trim();
	    v2 = v2.trim();
	    
	    isNumeric = Sampler.this.dataColumn.isNumeric;
	    String nam = Sampler.this.dataColumn.fieldname;
			      
	    
	    if (isNumeric) {
		dv1 = new Double(v1).doubleValue();
		dv2 = new Double(v2).doubleValue();
		//System.err.println("dv1,dv2:"+dv1+" "+dv2);
		ddata = (double[]) Sampler.this.dataColumn.data;
	    } else {
		sv1 = v1.toUpperCase();
		sv2 = v2.toUpperCase();
		sdata = (String[]) Sampler.this.dataColumn.data;
	    }
	}
	
	boolean check(int i) {
	    if (isNumeric) {
		return dv1 <= ddata[i] && ddata[i] <= dv2;
	    } else {
		String s = sdata[i];
		return sv1.compareTo(s.toUpperCase()) >= 0 &&
		       sv2.compareTo(s.toUpperCase()) <= 0;
	    }
	}
    }
	    
    Sampler (Column col, String criteria) {
	
	dataColumn = col;
	
        StringTokenizer stcriteria= new StringTokenizer(criteria,",");
	    
	//System.err.println("Sampler:  crit="+criteria);
        int count = stcriteria.countTokens();
	
	checks = new Checker[count];
	    
        crits = new String [count];

        for (int i=0;i<count;i++) {
	    String crit = stcriteria.nextToken();
	    crit = crit.trim();
            crits[i] = crit;
	    checks[i] = getChecker(crit);
	}
    }
    
    public double[] sample() {
	
	int nr = dataColumn.getNrows();
	int nc = checks.length;
	
	double[] samp = new double[nr];
	
	for (int i=0; i<nr; i += 1) {
	    samp[i] = 0;
	    for (int j=0; j<nc; j += 1) {
		//System.err.println("Checking row "+i+" for sample "+j ); 
		if (checks[j].check(i)) {
		   //System.err.println("Sampler samp: row="+i+" check= "+
			//(j) + " adding " + (j+1) + " to "+ samp[i] );
		    samp[i] = j+1;
		    break;
		}
	    }
	}
	return samp;
    }
    
    private Checker getChecker(String criterion) {
	int index = criterion.indexOf("..");
	if (index > 0) {
	    //System.err.println("Index, crit, range:"+index+" "+criterion);
	    return new RangeChecker(criterion, index);
	} else {
	    //System.err.println("Index, crit, eq:"+index+" "+criterion);
	    return new EqualityChecker(criterion);
	}
    }
    public String [] getCriteria () {
        return crits;
    }
    public Object  getSampData () {
        return sampdata;
    }
}
