# $Id: BatchDisplay.pm,v 1.8 2007/10/09 14:10:50 lmm Exp $
# $Log: BatchDisplay.pm,v $
# Revision 1.8  2007/10/09 14:10:50  lmm
# Added empty methods beforeNewTable() and afterTable() .
#
# Revision 1.7  2005/11/22 16:10:41  lmm
# Added empty resultsSummary method
#
# Revision 1.6  2003/07/31 19:10:38  cwh
# Overriding legend subroutine.
#
# Revision 1.5  2002/05/14 18:06:42  cwh
# Added empty YourSearchWas, since that is now a Display method.
# Needed a 'use'.
#
# Revision 1.4  2002/03/15 23:28:45  cwh
# Moved printing of BatchEnd to pageFooter.
#
# Revision 1.3  2001/12/05 23:16:49  cwh
# Changed to case-insensitive match.
#
# Revision 1.2  2001/11/07 23:34:15  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.1  2001/10/25 20:12:14  cwh
# Separated these from the Display.pm module.
#

# This class displays the results in a non-HTML text stream.
# Rows in the results are matched so that each column takes the
# same amount of space in each row.

use strict;

package Browse::BatchDisplay;

use vars qw( @ISA );
@ISA = qw( Browse::TextDisplay );
use Browse::TextDisplay;

# Print just the HTTP header and an indication that the batch data follows.
sub pageHeader {
    print "Content-type: text/plain\n\nBatchStart\n";
}

# No directions
sub Directions {
}

sub pageFooter {
    print "BatchEnd\n";
}

# No search criteria echoed
sub showText {
}

# Just save the metadata.
sub tableHeader {
    my ($self) = @_;
    
    my $textdisplay = $$self{"text"};
    my ($meta) = $$self{"meta"};
    
    $$self{"meta"} = $meta;
    $$self{"rowcount"} = 0;
}

sub legend {}

# This is similar to the processing for text display but
# we don't want any HTML at all, not even "<pre>".

sub closeTable {

    my ($self) = @_;
    
    if ($$self{"intable"}) {
	$$self{"intable"} = 0;
	
	my $array = $$self{"dataarray"};
	my $meta  = $$self{"meta"};
	my $names = $$self{"names"};
	my $types = $$self{"types"};
	
	my $nskip = 0;
	while ($nskip <= $#$names) {
	    if (substr($$names[$nskip], 0, 3) !~ /w3_/i) {
	        last;
	    }
	    $nskip += 1;
	}
	
	Browse::TableFormat::neatenTable($array, $names, $types, undef);
	
	splice(@$names, 0, $nskip);
	
	my $header =  "|".join("|", @$names)."|";
	print $header,"\n";
	$header =~ s/[^\|]/-/g;
	$header =~ s/[\|]/+/g;
	print $header,"\n";
	
	my $line;
	foreach $line(@$array) {
	    splice(@$line, 0, $nskip);
	    print "|",join("|", @$line),"|\n";
	}
	
	$$self{rowcount} = 0;
    }
}

# Don't worry about empty tables or errors.
sub showEmpties {
}
sub resultsSummary {}

sub YourSearchWas {
}
sub beforeNewTable {}
sub afterTable {}

## Successful load
1;
