# $Id: HProdDisplay.pm,v 1.17 2008/06/02 17:50:05 lmm Exp $
# See the pod for ProdMain.pm for documentation of inherited methods.
# See the end of this file for cvs logs and pod.


use strict;

use Browse::ProdMain;
use Browse::Log;
use LWP::Simple;

package Browse::HProdDisplay;
# Inheritance.
use vars qw( @ISA );
@ISA = qw( Browse::ProdMain );


#----------------------------------------------------------------------------
sub doTable {
    my ($self,$table,$tableInfo) = @_;
    $self->tableHeader($table, $tableInfo);
}

sub doKey {
    my ($self,$table,$tableInfo,$key) = @_;
    $self->rowHeader($table,$tableInfo,$key,$self->transInfo);
}

sub doSet {
    my ($self,$setdesc,$vals) = @_;
    $self->setLine($setdesc,$vals);
}

sub doProd {
    my ($self,$prod) = @_;
    $self->productLine($prod);
}


#--- set flag to indicate there are data products for current row
#--- Also stores html for Select All checkbox for display if flag is
#--- ever set
sub prodsForThisRow {
    my ($self,$flag,$string) = @_;
    $$self{"prodsExist"} = $flag if defined $flag;
    $$self{"selectallString"} = $string if defined $string;
    return $$self{'prodsExist'};
}

sub pageHeader {
    my ($self,$in) = @_;

    #  assign title and color for title box for display page 
    my($colorstyle)="dprodtable";
    $$self{"colorstyle"}=$colorstyle;
    Browse::Decor::Header($$in{"popupFrom"},0,0,0,$$self{"colorstyle"});


    print "<script type=\"text/javascript\" language=\"Javascript\">\n";
    Browse::ValidateInput::tickTable();
    print "</script>\n";
    my $formname="w3hdprods";
    Browse::ValidateInput::checkDPList($formname);
    print "<form name=\"$formname\" action=\"w3tar.pl\" method=\"POST\">\n";

    my $navtrail = $$in{"navtrail"};
    unless ($navtrail) {
      my $referrer = ($ENV{"HTTP_REFERER"} or ("/".$::cgiurl."/w3browse.pl"));
      $navtrail = "<a class='navpast' href='$referrer'> Choose Tables</a>";
    }
    print "$navtrail &gt; <span class='navcurrent'>Choose Data Products</span> <span class='navfuture'>&gt; Retrieve Data Products </span><br><p>";
    $navtrail .= " &gt; Choose Data Products";
    $navtrail =~ s/\</&lt;/g;
    $navtrail =~ s/\>/&gt;/g;
    my($nextPopupFrom)= "Retrieve Data Products";
    print "<input type='hidden' name='navtrail' value=\"$navtrail\">";
    print "<input type='hidden' name='popupFrom' value=\"$nextPopupFrom\">";

}   

sub instructions {
    print "<hr><p><span class='large'><ul><li>Do you want to view a data product? Click on its hyperlinked data format. <li>Do you want to retrieve data products in a tarfile? Check the boxes beside each product and click one of the buttons at the bottom of the page.</ul></span>";
    print "<script type=\"text/javascript\" language=\"JavaScript1.2\"><!--\ndocument.write(\"<input type=\\\"checkbox\\\" name=\\\"allfiles\\\" id=\\\"allfiles\\\" onClick=\\\"var a=this.form.elements;for (var i=0; i< a.length; i++){ if (a[i].name.indexOf('File') ==0) { a[i].checked=this.checked }}\\\"> <label for=\\\"allfiles\\\">Select all products for all rows</label>\")\n// -->\n</script>";
}

sub tableHeader {
   my ($self,$table,$tableInfo) = @_;
   (my $ptable = $table) =~ s/$::local_origin//ig;

   my $obsname = $$tableInfo{'observatory_name'};
   my($obs_link) = getFTOOLSlink($obsname);
   print <<EOT;
     <p><strong><a target=helpwin href='$$tableInfo{table_document_url}'>$$tableInfo{table_description} ($ptable)</a></strong>&nbsp;&nbsp;
<a target=helpwin href="/ftools/ftools_subpacks.html">FTOOLS</a>&nbsp;&nbsp;
$obs_link
EOT
}

sub rowHeader {
  my ($self,$table,$tableInfo,$key, $transInfo) = @_;
    my $rowid=$self->rowid(1);
    my ($coordsys, $equinox) = @{$self->coosys()};
    my ($row, $names, $transInfo) = Browse::RowDisplay::getSingleRow($table, 
        $key, undef, $tableInfo, $transInfo, $coordsys, $equinox);
    return 0 unless $row;
    my $i;
    for ($i=0; $i <= $#$row; $i += 1) {
	last if $$names[$i] !~ /^w3_/;
    }
    splice(@$row,   0, $i);
    splice(@$names, 0, $i);
    my($cstyle)=$$self{"colorstyle"};
    print "<br><table class='othertable'>\n".
        "<tr class='$cstyle'><th scope='col' class='$cstyle'>".
	join("</th><th scope='col' class='$cstyle'>",@$names)."</th></tr>\n".
        "<tr class='othertable'><td class='othertable'>".
        join("</td><td class='othertable'>",@$row)."</td></tr>\n".
        "</table><table>\n";
    # Store Select All checkbox code and clear data products flag for this row
    $self->prodsForThisRow(0,"<tr><td colspan=5>\n" .
	"<script type=\"text/javascript\" language=\"JavaScript1.2\">\n" .
	"<!--\ndocument.write(\"<input type=checkbox name=checkall$rowid" .
	" id=checkall$rowid" .
	" onClick=tickTable(this.form.File$rowid,this.checked,'')>" .
	" <label for=\\\"checkall$rowid\\\">" .
	"Select all products in this row</label>\")\n// -->\n" .
	"</script></td></tr>");
	    
    return $transInfo;
} 

sub setLine {
   my ($self,$desc, $vals) = @_;
   if ($vals) {
      #--- if this is the first listing for this row display 
      #--- Select All checkbox and set flag
      if (!$self->prodsForThisRow()) {
         print  $self->{selectallString};
         $self->prodsForThisRow(1);
      }
      print "<tr><td colspan=5 align=left><strong>$desc</strong> " ;
   }
   print "</td></tr>\n";
}

sub closeRow {
    my ($self)=@_;
    print "</table>";
    unless ($self->prodsForThisRow()) {
      print "<p><strong> No products found for this row </strong><br> Please consult the table documentation for information on data availability <p>";
    }
}

sub pageFooter {
    my ($self) = @_;

    if ($self->havelocaldata()) {
      print "<P><input type='submit' value='TAR selected products' onClick='return pageCheck() '><input type='submit' name='Create Download Script' value='Create Download Script' onClick='return pageCheck()'><input type='reset'><p>";
      print " <input type=submit name='Save to Hera' value='Save to Hera'> ";
      print "<a target='helpwin' href='$::heasarc/hera'>".
          "What is Hera?</a>\n";
    } elsif ($self->numprods()) {
      print "<p><b>No local data. Use links to access remote data.";
    }

    print "<input type='hidden' name='savetables' value='".join(',',@{$self->Tables()})."'>\n";
    print "</form>\n";
    Browse::Decor::EmailFooter();
    Browse::Decor::EndTags();

}

sub productLine {

    my ($self, $info) = @_;
    my $rowid=$self->rowid();
    
    my $tag=$$info{'dp_tag'};
    my $type=$$info{'dp_type'};
    my $format=$$info{'dp_format'};
    my $level=$$info{'dp_level'};
    my $url=$$info{'dp_url'};
    if ($type =~ /\W/) {
    }
    
    my ($check, $name, $form, $size, $gif, $file,$id);
   
    if ($format=~/TAR/) {
        $id = $url;
    } elsif ($url !~ /^ftp\:\/\/(o)?legacy/) {
        # External URLs
        $check = $size = "";
        $id = $url;
	$form = "External Link";
        # external gifs:
        if (!$format and $url=~/(gif|jpg|jpeg|png)$/i) { $format=$1;}
        ##if ($url=~/euvepreview/i) { $format="GIF";}  #no longer need hack 5/08
        $type = "Remote: ".$type;
        $file = $url;
    } else { 
        #Local URLs
    
        $url =~  /ftp\:\/\/[^\/]*(\/.*)/;
        $file = "/FTP$1";
        $url = $file;
        $id = $url;
    
    }
    $check = "<input type='checkbox' name='File$rowid' id='$id' value='$id'>";
    $url = "startfv.pl?file=$file" if $format =~ /FITS/;
    $url =~ s/([^:])\/\//$1\//;  # change '//' to '/', except ie 'ftp://'
    $name  = "$type";
    
    
    if ($self->checkSize()) {
        $size = $self->fileSize($file) if $file;
	if ($size) {
           $size=int(($size+512)/1024)." kB"; # just do them all in kB
        }
    }
    $form  = $format;
    if ($self->gifstogo() && ($format=~/JPE?G|GIF|PNG/i) && ($url !~ /z$/i)) {
        $self->gifstogo(1);
        #--- removed hack - no longer needed. lmm-05/2008
        #if ($name =~ /EUVE.*Preview/i) {
           #my ($gifname)=$url=~/dataid=(.*)$/;
           #my $gifurl = "http://archive.stsci.edu/browse1/browsers/euve/gif/";
           #print "EUVE: $gifname<br>\n" if $::debug;
           ## Normally $gifname.gif works. But if there is more than one 
           ## image available for an obs, the first images is named
           ## $gifname_1.gif and there is no $gifname.gif.
           #foreach my $try ("$gifurl/$gifname.gif","$gifurl/${gifname}_1.gif") {
             #if (LWP::Simple::head($try)) {
                #$gif = "<a target='datawin' href='$url'><img border='0' ".
                       #"width='100' height='100' alt='preview image' ". 
                       #"src='$try'></a>";
                #$form="HTML";
                #last;
             #}
           #}
        #} else {
       
        print "HProd::image: name=$name format=$format $url<br>\n" 
		if $::debug;
        $gif = "<a target='datawin' href='$url'><img border='0' " .
		"width='100' height='100' alt='preview image' src='$url'></a>";
        #}
    }
    $size = "" if $type =~ /Remote/;
  
     my $basename = reverse $url;
    ($basename) = $basename =~ /([^\/]+)/;
    $basename = "(" . reverse ($basename) . ")";
    $basename='' if $form =~ /HTML|HTTP/;
    my $link  = "<a target='datawin' href='$url'>$form</a>";
 
    print "<tr>"; 
    print "<td>$check</td>\n";
    print "<td align='left'><label for='$id'> $name $basename</label></td>\n";
    #print "<td>$form</td>\n";
    print "<td>$link</td>\n";
    print "<td align='right'>$size</td>\n" if $self->checkSize;
    print "<td>$gif</td>\n" if $self->showGIFs;
    print "</tr>\n";
 
    $self->havelocaldata(1) if $check =~ /checkbox/;
    $self->numprods(1);
    #$self->numInSet(1);
  
}    
	    

sub getFTOOLSlink{
  my($observatory) = @_;

  $observatory = lc($observatory);
  $observatory =~ s/\s+//g;

  print "<br>Observatory: $observatory" if ($::debug);

  my($link);
  if ($observatory) {
     $observatory = "gro" if $observatory =~ /^cgro$/;
     $observatory = "xte" if $observatory =~ /^rxte$/;
     if(-e "$::root/ftools/$observatory.html") {
       $link = "<a target=helpwin href=\"/ftools/$observatory.html\">\U$observatory\E FTOOLS</a>";
     }
  }
  return($link);

} #eosub--getFTOOLSlink

	    
1;

__END__

# $Log: HProdDisplay.pm,v $
# Revision 1.17  2008/06/02 17:50:05  lmm
# Removed EUVE specific code since it no longer works.  Links now generated
#   via the zzdp table.
#
# Revision 1.16  2007/03/26 19:16:12  lmm
# Enhanced "No Data Products found" message
#
# Revision 1.15  2006/10/24 17:18:48  lmm
# Cosmetic changes only.
#
# Revision 1.14  2005/07/28 18:29:33  lmm
# Corrected link to Hera web page
#
# Revision 1.13  2005/02/23 16:18:44  lmm
# Link to individual files moved to "type" (eg. DIRECTORY) to avoid
#   confusion when HTML label tag toggles checkbox
#
# Revision 1.12  2004/12/16 18:58:56  lmm
# Further improvements for "No data products" messages
#
# Revision 1.11  2004/12/16 14:01:16  lmm
# Added a bit more information to instructions
# Added text to indicate when no data products are available for
#   data product sets
#
# Revision 1.10  2004/10/04 21:06:37  lmm
# Added option to Create Download Script
#
# Revision 1.9  2003/08/28 21:33:20  lmm
# Define text and style for page header and remove HEASARC footer
#
# Revision 1.8  2003/04/28 02:20:47  cwh
# Added new table style.
#
# Revision 1.7  2003/02/25 21:31:05  cwh
# Added savetables as a hidden param to be passed along, so we can log table
# names when files are chosen from the product display page.
#
# Revision 1.6  2002/12/16 18:11:54  cwh
# Added quotes where needed.
# Added labels and ids for section 508 compliance.
# No longer encoding filename/url.
# Consolidated checkbox generation for different data types.
#
# Revision 1.5  2002/11/07 16:46:26  cwh
# Change double-slashes to songle slashes in urls, except for protocol strings
# like 'ftp://'.
#
# Revision 1.4  2002/10/28 17:28:08  cwh
# Encoding the file name of remote files. (Had trouble with urls to scripts
# being truncated if not encoded.)
#
# Revision 1.3  2002/10/25 22:09:48  cwh
# Now showing checkbox for remote products, since we can now retrieve them.
#
# Revision 1.2  2002/10/17 19:17:18  cwh
# Only set format for recognized suffixes if format doesn't exist.
# Oops - url wasn't being set for chandra external products.
# Set size undef for remote prods.
# Report a base filename for each product, since many prod lines look the same.
#
# Revision 1.1  2002/09/05 18:26:56  cwh
# ProdMain is a class which creates objects for displaying data products and
# for generating data product packages for retrieval.  HProdDisplay and
# HProdGen are provided as examples of display and generation classes which
# inherit from ProdMain.  The expectation is that other sites using this code
# will modify the HProd* modules for their needs.
#

=head1 NAME

HProdDisplay - Methods for displaying available data products to the user. Inherits from ProdMain.  See L<Browse::ProdMain> for description of inherited methods.

=head1 SYNOPSIS

 use Browse::ProdMain;

 my($in) = Browse::CGI::ReadParse();
 my $dp = new Browse::ProdMain($in);
 $dp->doProducts($in);

=head1 DESCRIPTION

The HProdDisplay class is created by and inherits from the ProdMain class.

The doProducts() method is a complete product processor, but if you want
more control you can directly call the methods used in doProducts().

=head1 OPTIONAL: USE METHODS DIRECTLY

See L<Browse::ProdMain> for the description of inherited methods.
These are the additional methods in HProdDisplay that you can use if you 
choose not to use doProducts():


=head2 PRINT A TABLE HEADER

 $dp->tableHeader($table,$tableInfo);

Prints the table name, a link to the table documentation, and a link to the FTOOLS.

=head2 PRINT A ROW FROM THE QUERY RESULTS
 
 $dp->rowHeader($table,$tableInfo,$key,$transInfo);

Prints the current row from the query results. $table is the name of the 
table, $tableInfo is the result of Browse::HDB::TableInfo($table), $key is the 
unique key for this row, and $transInfo is returned from rowHeader and kept 
for the next time rowHeader is called.
This method begins an html table in which the data set names and products are displayed.

=head2 PRINT A DATA SET HEADER
 
 $dp->setLine($set);

Prints the set name in an existing html table.

=head2 PRINT A DATA PRODUCT
 
 $dp->productLine($prod);

Prints a line with various ways of accessing an individual data product.
$prod is a hash ref containing information about the product.
Displayed as a row in an html table.

=head1 AUTHOR INFORMATION

High Energy Astrophysics Science Archive Research Center (HEASARC),
NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.

You are free to use this code as you wish, but unless you change
it significantly, please leave the above name & address intact.

=head1 BUGS

=head1 SEE ALSO

L<Browse::ProdMain>, L<Browse::HProdGen>

=cut


=cut
