# $Id: HeasarcQuery.pm,v 1.47 2007/10/16 15:32:14 lmm Exp $
# $Log: HeasarcQuery.pm,v $
# Revision 1.47  2007/10/16 15:32:14  lmm
# Added zzdp table specific code to make queries selecting dp_tag
#   case-sensitive to decrease run time.
#
# Revision 1.46  2007/03/26 19:19:03  lmm
# Quotes are now not added to parameter query if user has entered SQL
#   (indicated by beginning and ending parens)
#
# Revision 1.45  2006/06/23 14:53:37  lmm
# Added changes to propogate precision information to TableFormat.pm
#   (code provided by T. McGlynn)
#
# Revision 1.44  2005/09/02 19:40:04  lmm
# Fixed bug that occurred when cross corr query contained time field
#   qualifiers
#
# Revision 1.43  2005/09/02 14:15:31  lmm
# Further refinement of link dependancy processing.
#
# Revision 1.42  2005/08/12 21:23:48  lmm
# Changed escape for single quote in SQL select statement from backslash
#   to another single quote
#
# Revision 1.41  2005/07/20 16:47:30  lmm
# More improvements to allow additional types of table link dependencies
#    (changes contributed by E. Sabol)
#
# Revision 1.40  2005/07/18 20:45:47  lmm
# Improvements to allow additional types of table link dependencies
#   (changes contributed by E. Sabol)
#
# Revision 1.39  2005/06/29 14:58:16  lmm
# Fixed bug in the code that checks link criteria and dependency for
#   non-default paramaters
#
# Revision 1.38  2005/06/28 20:13:04  lmm
# Filtered out operation characters from query link name for related
#   links
#
# Revision 1.37  2005/06/22 15:32:48  lmm
# Added link dependency and link criterion functionality for parameters
#   not included as table defaults
# Eliminated the automatic placement of quotes around value in sql if field
#   is string format (value may be a select statement)
#
# Revision 1.36  2005/02/24 15:28:46  lmm
# Fixed bug that appeared during cross-correlations of queries made
#   using a time search (from tam)
#
# Revision 1.35  2005/02/15 15:54:23  lmm
# Added zzlink searches for parameters that are not in default display set
#   but are needed for table and bibliography links.  Included in table
#   sql with _link_ prefix.
#
# Revision 1.34  2004/11/09 20:06:41  lmm
# Added removal of leading and trailing spaces for qualifier field in
#   makeQuals().  This was a problem when this field was used as a hash key.
#
# Revision 1.33  2003/12/02 19:56:33  lmm
# Change to code that makes queries case insensitive (This change was made
#   a while ago but never checked in)
#
# Revision 1.32  2003/04/28 02:29:28  cwh
# Allow null as a special value for parameter.
# Don't require '=' in the operator.
# Fixed bug where search offset format was set to int.
#
# Revision 1.31  2002/10/17 22:07:50  cwh
# Added a kludge to handle when the first table in a request has an end_time
# defined as well as a start time, but a subsequent table only has a start time.
# Just sets the end_time col name to the start_time col name.
#
# Revision 1.30  2002/09/05 18:18:42  cwh
# Now using DPQuery instead of DPL.
#
# Revision 1.29  2002/05/16 16:46:20  cwh
# Added '<=' and ">=" as allowed operators.
#
# Revision 1.28  2002/03/17 18:20:06  cwh
# Added use Astro::Utils::SLAConvert;
#
# Revision 1.27  2001/12/17 16:35:55  cwh
# Parameter format wasn't being set for regenerated queries in cross-
#   correlations. (was looking for format for 'field', not 'a_field'.
#
# Revision 1.26  2001/12/06 16:48:42  cwh
# Call setOffsetVar, getOffsetVar to isolate offset var name.
# Call getPrefix instead of looking at $$self{prefix}.
#
# Revision 1.25  2001/11/30 20:01:42  cwh
# Removed distance() sub, now uses distance() in Utils module.
# Now gets key from the keyinfo property of self.
# Fixed problem with default exposure sort field where the field wasn't being
#   set properly.
#
# Revision 1.24  2001/11/08 20:19:22  cwh
# Added Browse:: to sub calls.
# Changed code that removed \w_ from table name to remove $prefix_ if $prefix.
#
# Revision 1.23  2001/10/25 22:20:28  cwh
# Added 'in' to the list of allowable operators (for the class parameter.).
#
# Revision 1.22  2001/10/25 19:27:55  cwh
# Changed _Offset to _Search_Offset for clarity.
#
# Revision 1.21  2001/10/04 22:11:12  cwh
# Users group suggested a default sort: by descending exposure if datasets,
#   otherwise by _Offset if positions defined, else no sorting.
#
# Revision 1.20  2001/09/15 13:42:41  cwh
# Resolve metaparams in makeFields.
# Ra, dec field names are no longer passed on the position string, they are
#   read in from the tableinfo.
# makeQuals can now handle multiple qualifiers for the same parameter.
# Changed condition for return undeframeter format is now set correctly for
#   metaparams.
#
# Revision 1.19  2001/08/08 21:10:54  cwh
# Added check that only betweeen,!betweeen,=,!= operators are entered.
# Added better null handling:
#  for character fields:
#     op        min        max      sql where clause
#    ----------------------------------------------------------------------
#    range | null/space | value || field <= 'value' or field is null
#    !range| null/space | value || field > 'value'
#     =    | null/space | undef || field is null or field = ' '
#     !=   | null/space | undef || not field is null and not field = ' '
#    range | value      |null/spc| field >= 'value'
#    !range| value      |null/spc| field < 'value' or field is null
#    !=    | value      | undef || not field = 'value' or field is null
#    !range| value1     | value2|| not field between 'value1' and 'value2' or
#                                        field is null
#
#  for numeric fields:
#     op        min        max      sql where clause
#    ----------------------------------------------------------------------
#    range | null/space | value || field <= value or field is null
#    !range| null/space | value || field > value
#    =     | null/space | undef || field is null
#    !=    | null/space | undef || not field is null
#    range | value      |null/spc| field >= value
#    !range| value      |null/spc| field < value or field is null
#    !=    | value      | undef || not field = value or field is null
#    !range| value1     | value2|| not field between value1 and value2 or
#                                        field is null
#
# Revision 1.18  2001/07/23 16:49:06  cwh
# Added $entry and $resolver to list of things split from positions line.
#
# Revision 1.17  2001/07/23 16:15:42  cwh
# Changed distance formula from Law of Cosines to Haversine.
#
# Revision 1.16  2001/07/19 19:20:09  cwh
# Changed distance formula to Haversine.
#
# Revision 1.15  2001/07/18 19:43:03  cwh
# Needed a 'use Log'.
#
# Revision 1.14  2001/07/17 18:39:40  cwh
# Added call to Log::SqlLogger.
#
# Revision 1.13  2001/07/09 21:06:19  cwh
# qclose wasn't being called when it should have been: once when getDataLine
# returned null (query done) and when the query returned an error.
#
# Revision 1.12  2001/07/01 01:54:25  cwh
# Put quotes around _Offset - Oracle can't handle leading underscores
# without quotes.
#
# Revision 1.11  2001/06/20 22:57:32  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.10  2001/06/14 21:28:26  tam
# Added support for '==' for exact character matches.
#
# Revision 1.9  2001/06/12 18:07:26  tam
# Fixed a bug in the cone search code which only
# manifested itself when the search area crossed the
# anti-prime meridian.
#
# Revision 1.8  2001/06/12 16:12:39  tam
# Handle qualifiers where the user has an = or !=
# for a numeric field without specifying a value as
# search for nulls on that field.
#
# Revision 1.7  2001/06/08 19:46:37  cwh
# Now passing $::writeHeader to ::Error calls, so we don't get repeating headers.
#
# Revision 1.6  2001/05/30 21:22:48  cwh
# Added use lib's so that the Astro lib in /www/htdocs/cgi-bin/lib/heasarc/ will
#   be found.  Had to re-add the './lib' and '.' libs to keep the order right.
#
# Revision 1.5  2001/05/22 17:52:03  cwh
# Added Id and Log to files without them.
#

use strict;


use Browse::QueryRequest;
use Browse::Log;
use Astro;
use Astro::Utils::SLAConvert;

# This package supports queries of the HEASARC style databse.
package Browse::HeasarcQuery;

# Pi (not used, just here for the fun of it!)
my($PI)   = 3.1415926535897932384626433832795028841971693993751;
# degrees to radians constant (Pi/180)
my($DTOR) = 0.017453292519943295769236907684886127134428718885417;
# arcminutes to radians constant (Pi/(180*60))
my($MTOR) = $DTOR/60;



# Inheritance.
use vars qw( @ISA );
@ISA = qw( Browse::QueryRequest );


# See if this is a local query -- i.e., processed
# by the default database.
sub localQuery {
    my ($self) = @_;
    
    my $tinfo = $$self{tableinfo};
    if ($tinfo) {
        my ($dft) = Browse::DBQ::Registry::getDefault();
	my $loc = $$tinfo{"table_location"};
	return $dft eq $loc;
    } else {
        return 1;
    }
}


sub getData {

    my ($self) = @_;
    

    my $table = $self->getTable();
    if (!$table) {
        return undef;
    }
    
    my $status = $$self{"status"};
    
    if ($status eq "initiate") {
    
    
        if (!$$self{"paraminfo"}) {
	    $self->getMeta($table);
	}
	$$self{"status"} = "data";
	$$self{"currentTable"} = $table;
	$self->makeQuery();
	$self->startQuery();
	
	return $self->putMeta();
	
    } elsif ($status eq "done") {
        return undef;
	
    } else {
        my $line =  $self->getDataLine();
	if (!defined($line)) {
	    $$self{"status"} = "done";
            $self->qclose();
	}
	return $line;
    }
}

sub getMeta {

    my ($self, $table) = @_;
    
    
    my %tableInfo = Browse::HDB::TableInfo($table);
    my $tableInfo = \%tableInfo;
    my $paramInfo = Browse::HDB::ParamInfo($table);
    $paramInfo = $$paramInfo{$table};
    my $dpquery = new Browse::DPQuery($table);
    my ($dpsets, $keyinfo) = $dpquery->getKeysAndDPSets();
    
    if ($self->getPrefix()) {
    
        my $prefix = $self->getPrefix;
	my $xtable = $$tableInfo{"table_name"};
	
	# First check to see if the table name already has a
	# space.  If so we've probably already put in the prefix.
	if ($xtable !~ / /) {
	    $table = "$table $prefix";
	    $$tableInfo{"table_name"} = $table;
	
	    my @ok = keys(%$paramInfo);
	    
            foreach (@ok) {
	
	        if (substr($_,0,1) eq "#") {
	            next;
	        }
	        my $newk = $prefix."_".$_;
	        $$paramInfo{$newk} = $$paramInfo{$_};
	        delete($$paramInfo{$_});
	    }
	
	    @ok = keys(%$paramInfo);
	    foreach (keys(%$tableInfo)) {
	        my $val = $$tableInfo{$_};
	        if (substr($val, 0, 1) eq "@") {
	            substr($val, 0, 1) = "\@$prefix\_";
		    $$tableInfo{$_} = $val;
	        } else {
	            if ($_ eq "right_ascension"  ||
		        $_ eq "declination"      ||
		        $_ eq "start_time"       ||
			$_ eq "target_name") {
		        $val = $prefix."_".$val;
		    }
		    $$tableInfo{$_} = $val;
	        }
	    }
	}
    }
	    
    $$self{"table"}     = $table;
    my ($xtable) = split(" ", $table);
    $$self{"tableinfo"} = $tableInfo;
    $$self{"paraminfo"} = $paramInfo;
    $$self{"dpsets"}    = $dpsets;
    $$self{"keyinfo"}   = $keyinfo;
}

sub putMeta {

    my ($self) = @_;
    my $pos;
    if ($$self{"allpositions"}) {
        $pos = $$self{"allpositions"};
    } else {
        $pos = $$self{"positions"};
    }
    
    return {
       "table"       => $$self{"table"},
       "tableinfo"   => $$self{"tableinfo"},
       "paraminfo"   => $$self{"paraminfo"},
       "dpsets"      => $$self{"dpsets"},
       "keyinfo"     => $$self{"keyinfo"},
       "fields"      => $$self{"fields"},
       "positions"   => $pos,
       "sorts"       => $$self{"sorts"},
       "dirs"        => $$self{"dirs"},
       "quals"       => $$self{"quals"},
       "sortable"    => "up|down",
       "serialization" => $$self{"serialization"},
       "type"        => "meta"
     };
}
       

sub makeQuery {

    my ($self) = @_;
    
    if (!defined($$self{tableinfo})) {
        $self->getMeta($$self{table});
    }
    
    my $prefix = $self->getPrefix();
    
    if ($prefix) {
        $prefix = " $prefix";
    }
    
    if ($$self{"sql"}) {
        # We have an old query about.
	# Just need to redo the cone search stuff.
	$self->makeCone();
	
    } else {
    
        $self->setOffsetVar();
        $self->makeFields();
	$self->makeSort();
	$self->makeQual();
	$self->makeCone(1);
	
    }
    
    my $sql = "select ";
    $sql .= $self->getFields();
    $sql .= " from ";
    
    $sql .= $self->getSource();
    $sql .= $self->getWhere();
    $sql .= $self->getOrder();
    
    $$self{"sql"} = $sql;
    
    my $conn = $$self{"tableinfo"}{"table_location"};
    my ($db) = Browse::DBQ::Registry::getConnection($conn);
    $$self{"DBQ"} = $db;
    print "SQL is: <pre>$sql</pre><br>\n" if $::debug;
    Browse::Log::SqlLogger($$self{"sql"},$conn);
}

sub getFields {

    my ($self) = @_;
    
    my $sql;
    my $comma;
    
    if ($$self{"fieldString"}) {
        $sql .= $$self{"fieldString"};
	$comma = ",";
    }
    
    if ($$self{"coneParam"}) {
        $sql .= $comma.$$self{"coneParam"};
        $comma = ",";
    }
    if ($comma ne ",") {
        die("No fields");
    }
    return $sql;
}

sub getSource {

    my ($self) = @_;
    return "$$self{table}";
}

sub getWhere {
    my ($self) = @_;

    
    my $sql;
    my $and = " ";
    
    my $where ;
    if ($$self{"coneWhere"}) {
        $where = $$self{"coneWhere"};
        $and = " and ";
    }
    
    if ($$self{"qualWhere"}) {
        $where .= $and.$$self{"qualWhere"};
	$and = " and ";
    }
    
    if ($where) {
        $sql = " where $where ";
    }
    return $sql;
}


sub getOrder {
    my ($self) = @_;

    
    if ($$self{"sortString"}) {
        return " order by ".$$self{"sortString"};
    } else {
        return undef;
    }
}

sub startQuery {

    my ($self) = @_;
    my $db = $$self{"DBQ"};
    my $maxrows = $$self{"maxrows"};
    
    if (defined($maxrows)) {
        if ($maxrows =~ /^\s*\d*\s*$/) {
	    $db->oexecute("set rowcount $maxrows");
	} else {
	    $$self{maxrows} = undef;
	}
    }
    $self->{'active'} = 1;
    $db->query_by_row($$self{"sql"});
    if ($DBI::errstr) {
        my $errstr = $DBI::errstr;
        $self->qclose();
        Browse::Utils::Error("Error initiating query: $errstr. " .
		" SQL is: $$self{sql}");
    }
}

sub qclose {

    my ($self) = @_;
    if ($self->{"active"}) {
        
        my $maxrows = $$self{"maxrows"};
        my $db      = $$self{"DBQ"};
        if (defined($maxrows)) {
        	$db->oexecute("set rowcount 0");
        }
        $$self{"status"} = "initiate";
        delete $$self{"active"};

   }
}

sub getDataLine {

    
    my  ($self) = @_;
    
    
    my $db   = $$self{"DBQ"};
    my $line = $db->nextRow();
    if (!defined($line)) {
        return undef;
    }
    my $st = {};
    $$st{"data"}  = $line;
    $$st{"type"}  = "data";
    $$st{"types"} = $$db{"statement"}{"TYPE"};
    $$st{"names"} = $$db{"statement"}{"NAME"};
    $$st{"precisions"} = $$db{"statement"}{"PRECISION"}; #long strings tam

    
    return $st;
}


sub makeFields {

    my ($self) = @_;
    
    # We need to be a little more cautious
    # about not calling this routine multiple
    # times, since it modifies one of the variables
    # used as input (i.e, $$self{fields}).
    
    if ($$self{"fieldString"}) {
        return;
    }
    
    
    my $fields =    $$self{"fields"};
    my $paraminfo = $$self{"paraminfo"};
    my $tableinfo = $$self{"tableinfo"};
    my $prefix    = $self->getPrefix();

    if (!ref($fields)) {
	my (@pkeys) = keys(%$paraminfo);
	my (@fkeys);
	    
	my $key;
	foreach $key(@pkeys) {
	    if (substr($key, 0, 1) eq "#") {
		next;
	    }
	    if ($fields =~ /all/i || $$paraminfo{$key}{"parameter_default"}) {
	        push(@fkeys, $key);
	    }
	
	    @fkeys = sort  
	        {
	            my $ad = $$paraminfo{$a}{"parameter_default"};
	            my $bd = $$paraminfo{$b}{"parameter_default"};
		    if ($ad == 0 && $bd == 0) {
		        return $a cmp $b;
		    } elsif ($ad eq 0) {
		        return 1;
		    } elsif ($bd eq 0) {
		        return -1;
		    } else {
		        return ($ad <=> $bd);
		    }
	        }
	          @fkeys;
		  
	}
  
	$fields= \@fkeys;
	
    } else {

        # resolve metaparams
	foreach (@$fields) {
          $_ = $tableinfo->{$1} if  /\_metaparam\_(\w+)/;
          s/^$prefix\_// if $prefix; # take off leading prefix if any
        }
        # Make sure we put the prefix on explicitly named fields.
	# Check to make sure we haven't done this already...
        my $f0 = $$fields[0];
	if ($prefix && $f0  && ($f0 !~ /^$prefix\_/  || $f0 !~ /^$prefix\./)) {
	    foreach (@$fields) {
	        $_ =~ s/^/$prefix\_/;
	    }
	}
    }
    
    if ($prefix) {
        my $key;
	foreach $key (@$fields) {
	    my $dkey = chip($key);
	    $key = "$dkey as $key";
	}
    }
   
    (my $key = $$self{"keyinfo"}) =~ s/:[^,]*//g;
    
    if (defined($key)) {
        my @ukeys = split(",", $key);
	foreach $key(reverse(@ukeys)) {
	    if ($prefix) {
	        unshift(@$fields, "$prefix.$key as w3_$prefix"."_$key");
	    } else {
	        unshift(@$fields, "$key as w3_$key");
	    }
	}
    }

    #--- Check if parameters specified in link criteria and dependency are 
    #--- default parameters.  If not, add it as a "special" parameter
    my ($existsAsParam, $existsAsMap); 
    my $results;
    my ($zzfield, $op, $tvalue);
    my $zzlinksql="select link_criterion, link_dependency from zzlink " .
	"where table_name='" .  $tableinfo->{table_name} . 
	"' and (link_dependency is not NULL or link_criterion is not NULL)";

    ($results) = Browse::DBQ::query($zzlinksql);
    @$results=grep(/\w/,@$results);

    if (scalar (@$results)) {
       foreach my $x  (@$results) {
          my $zzparam = $x->[0];  #--- check link_criterion parameters
          if ($zzparam =~ /\w/) {  #--- make sure field is not just blanks
             print "zzparam=$zzparam<br>\n" if $::debug;

             #--- Search for tablename.field pattern
             my $mastertable = $tableinfo->{table_name} ;
             foreach my $zzfield ($zzparam =~ /$mastertable\.([\w]+)/ig) {

                $existsAsParam = grep {/\b$zzfield\b/} @$fields;
                $existsAsMap = grep {/\b"_link_" .${zzfield}\b/} @$fields;

                #--- if it is a table field but not displayed, then add it
                if ($paraminfo->{$zzfield}
		    && !$existsAsParam && !$existsAsMap) {
                   #--- add to sql
                   print "Adding _link_$zzfield<br>\n" if $::debug;
                   push(@$fields, "$zzfield as _link_$zzfield");
                }
             }
          }

          $zzparam = $x->[1];  #--- check link_dependency parameter
          if ($zzparam =~ /^(.*)([\=\>\<]|\s+in\s+|\s+not\s+in\s+)/i) {
             my $zzfield = $1;
             $zzfield =~ s/^ *//;
             $zzfield =~ s/ *$//;
             print "--checking $zzfield in @$fields<br>\n" if $::debug;
 
             #--- check that query parameter does not already exist
             $existsAsParam = grep {/\b$zzfield\b/} @$fields;
             $existsAsMap = grep {/\b"_link_" .${zzfield}\b/} @$fields;
 
             #--- if it is a table field but not displayed, then add it
             if ($paraminfo->{$zzfield}
                     && !$existsAsParam && !$existsAsMap) {
                #--- add to sql
                print "Adding _link_$zzfield<br>\n" if $::debug;
                push(@$fields, "$zzfield as _link_$zzfield");
             }

          }
       }
    }

    #--- Check if bibcode links are default parameters.  If not,
    #--- add it as a "special" parameter
    my $zzbibsql="select distinct id_fields from zzbib where table_name='" .
	$tableinfo->{table_name} . "' and id_fields is not NULL";
    ($results) = Browse::DBQ::query($zzbibsql);
    @$results=grep(/\w/,@$results);

    if (scalar (@$results)) {
       foreach my $x  (@$results) {
          $zzfield = @$x[0];
             print "--checking $zzfield <br>\n" if $::debug;
          if ($zzfield =~ /\w/) {  # make sure field is not all blankspaces
             $existsAsParam = grep {$_ eq $zzfield} @$fields;
             $existsAsMap = grep {$_ eq "_link_${zzfield}"} @$fields;
             if (!$$paraminfo{$zzfield}{"parameter_default"} && !$existsAsParam
		   && !$existsAsMap) {
                push(@$fields, "$zzfield as _link_$zzfield");
             }
          }
       }
    }

    $$self{"fieldString"} = join(",", @$fields);
    $$self{"fields"} = $fields;
    return;
}

sub makeSort {

    my ($self) = @_;
    
    
    #if (!$$self{"sorts"}) {
    #    return undef;
    #}
    
    my $prefix = $self->getPrefix();
    if ($prefix) {
        $prefix .= ".";
    }
    
    my $comma;
    
    my $i;
    
    my $sorts = $$self{"sorts"};
    my $dirs  = $$self{"dirs"};
    my $str;
    
    for ($i=0; $i <= $#$sorts; $i += 1) {
    
        print "HQ::makesort::sort $i =" . $$sorts[$i]  . "<br>" if $::debug;
        $str .= "$comma$prefix$$sorts[$i] ";
	
	if ($$dirs[$i]) {
	    $str .= " desc ";
	}
	$comma = ",";
    }

    # default sort was suggested by group: by descending exposure if datasets,
    #  otherwise by Offset if positions defined, else no sorting.
    if ($str !~ /\w/) {
       if ( $$self{"dpsets"} and $$self{"fieldString"} =~ /exposure/i) {
         my ($sortfield) = $$self{"fieldString"} =~ /([^, ]*exposure[^, ]*)/i;
         $str = "$sortfield desc" if $sortfield;
       } else {
         $str = $self->getOffsetVar() if $$self{"positions"}[0] =~ /\d/;
       }
    }

    $$self{"sortString"} = $str;
    return;
}
        
sub makeCone {

    my ($self, $addParam) = @_;
    

    my $coneField= $$self{"positions"}; 
    
    my ($prefix) = $self->getPrefix();
    my $descfix;
    
    if ($prefix) {
	$descfix = "_".$prefix;
        $prefix .= ".";
    }
	
    my $cone = $$coneField[0];
    return undef unless $cone =~ /\d/;
    
    my ($ra, $dec, $r, $entry, $resolver) = split('\|', $cone);

    my $raf  = $$self{tableinfo}{"right_ascension"};
    my $decf = $$self{tableinfo}{"declination"};

    if (substr($raf, 0, 1) eq "@") {
       $raf = substr($raf, 1);
    }
    if (substr($decf, 0, 1) eq "@") {
       $decf = substr($decf, 1);
    }
    if (substr($raf, 0, 2) =~ /\w_/) {
       $raf = substr($raf, 2);
    }
    if (substr($decf, 0, 2) =~ /\w_/) {
       $decf = substr($decf, 2);
    }
    
    $raf  = "$prefix$raf";
    $decf = "$prefix$decf";
    
    if ($r =~ /Default/i) {
        $r = $$self{"radius"}/60;
    }

    my ($ramin, $ramax,  $decmin, $decmax) =
       Astro::Utils::sphrect($r, $ra, $dec);
       
    my $qual = Browse::Utils::distance($raf, $decf, $ra, $dec);
    
    my $oname = $descfix.$self->getOffsetVar();
    
    $$self{"coneParam"} = "$qual/$MTOR as \"$oname\"";
    if ($addParam) {
        my $fields = $$self{"fields"};
        push(@$fields, $oname);
	my $hash = {};
	my $tableinfo = $$self{"tableinfo"};
	my $paraminfo = $$self{"paraminfo"};
	$$hash{"table_name"}       = $$tableinfo{"table_name"};
	$$hash{"param_name"}       = $oname;
	$$hash{"parameter_format"} = "float4:.3f";
	$$hash{"parameter_unit"}   = "'";
	$$paraminfo{$oname}= $hash;
    }
	
    $qual =~ s/ *$//;  # For consistency with USNO query processing.
    $qual = "($qual < " . $r*$DTOR . ")";
    
    my $box;
    
    if ($ramin <= $ramax) {
        $box = "(($raf >= $ramin and $raf <= $ramax) and ".
	       "  ($decf >= $decmin and $decf <= $decmax))";
    } else {
        $box = "(($raf >= $ramin or $raf <= $ramax) and ".
	       " ($decf >= $decmin and $decf <= $decmax))";
    }
    
    $$self{"coneWhere"} = "($box and  $qual)";

    return;
}


sub makeQual {

    my ($self) = @_;
    
    my $and  = "";
    
    my $qual;
    my $qualifiers = $$self{"quals"};
    
    if (!defined($qualifiers)) {
        return undef;
    }
    
    my $prefix = $self->getPrefix();
    
    my $table = $self->getTable();
    
    my $param = $$self{"paraminfo"};
    print "HeasarcQuery: quals:<br>" if $::debug;
    Browse::Utils::DumpRef($qualifiers) if $::debug;

    my $tableinfo = $$self{"tableinfo"};

    my %quals;
    foreach $qual(@$qualifiers) {
        print "HeasarcQuery::qual=$qual<br>" if $::debug;
        my ($field, $rest) = split('\|', $qual,2);
        $quals{$field} .= '||' if $quals{$field};
        $quals{$field} .= $qual;
    }
    
    my $sql;
    my $fld;
    foreach $fld(keys %quals) {
    
        print "HQ: qual = $quals{$fld}<br>\n" if $::debug;
        my $qual;
        my $qualgrpop ='';
        $sql .= " $and (";
        foreach $qual (split('\|\|',$quals{$fld})) {

          print "qual: $qual<br>\n" if $::debug;
          $sql .= $qualgrpop;
          my ($field, $op, $p1, $p2) = split('\|', $qual);

          #--- remove leading and trailing spaces  
          $field  =~ s/^\s+//;
          $field  =~ s/\s+$//;


          unless ($op =~ /^!?between$|^!?[<>]?=?|in/) {
            Browse::Utils::Error("Invalid operator: $op is not allowed.");
          }
	  my $sfield = $field;
          #$sfield = $tableinfo->{$1} if $field=~ /\_metaparam\_(\w+)/;
          ##-- change per tam
          if ($field=~ /\_metaparam\_(\w+)/) {
	      $sfield = $$tableinfo{$1};

              #if ($prefix) {
              if ($field =~ /^$prefix\./) {
	          # Already prepended the prefix to the name.
	          $sfield = substr($sfield, length($prefix)+1);
	      }
	  }

          # kludge for tables without end_times but with metaparam_end_time
          if ((!$sfield) and ($1 =~/end_time/)) {
             $sfield = $$tableinfo{"start_time"};
          }
	  my $ptype = $$param{$sfield}{"parameter_format"};
          print "HQ:field:$field  op is $op p1=$p1 type=$ptype p2=$p2<br>\n" if $::debug;
        
	  if ($prefix) {
              $ptype = $$param{$prefix."_".$sfield}{"parameter_format"};
              #--- remove the a_ for cross cor
              $sfield =~ /^$prefix\_(.*)$/;
              $sfield = $1 if $1;
	      $sfield = "$prefix.$sfield";
  	  }
	
	  my $not;
	
	  # Handle not's by negating the following qualifier.
	
	  if ($op =~ /^\!/) {
	    $op = substr($op, 1);
	    $not = " not ";
	  }
	
	
	  my $quote;
	  my $charlen;

	  $p1 = dequote($p1, $quote);
	  $p2 = dequote($p2, $quote);

          # nulls: if p1 is just space, change to null.
          $p1 = '' if $p1 !~ /\S/;

	  if ($ptype=~ /char/i) {
	
	    $quote      = "'";
	    $charlen = $';   #'
	    
            # Make all character queries case-insensitive except dp_tag/zzdp
	    # queries (zzdp changes added by lmm 10/2007 because of excessive 
            # time it takes these queries to run)
	    # This is a little Sybase specific since
	    # Sybase supports characters only up to char255.
	    # Text variables are used for longer strings,
	    # but these do not support uppercase operations.
	    #
	    
            
	    if ($op ne '==') {
	        if ($charlen <= 255) {
	            $sfield = "upper($sfield)" unless ($sfield eq "dp_tag"
			&& $tableinfo->{table_name} eq "zzdp");
	            $p1 =~ tr/a-z/A-Z/ 
			unless ($p1 =~ /\(/ || ($sfield eq "dp_tag" && 
			$tableinfo->{table_name} eq "zzdp"));
	            $p2 =~ tr/a-z/A-Z/ 
			unless ($p2 =~ /\(/ || ($sfield eq "dp_tag" && 
			$tableinfo->{table_name} eq "zzdp"));
	        } else {
	            if ($p1 =~ /^\'(.*)\'$/) {
		        $p1 = "\'\%".$1."\%\'";
		    } else {
	                $p1 = "\%$p1\%";
		    }
	        }
	    }
	    
	    if ($op eq '=') {
	        if ($p1 =~ /[\%\*]/) {
	            $p1 =~ tr/\*/\%/;
		    #$p1 =~ s/(\')?$/%$1/;
		    $p1 =~ s/(\')+$/%$1/; # changed to prevent *001->*001*
		    $op = " like ";
		}
	    } 
          }
	    if ($op eq '==') {
	        $op = '=';
	    }

	    # Handle searching for nulls.
            if (!length($p1) or $p1 =~ /^null$/i) {
                $op = "is" if !length($p2);
                $p1 = "null";
            }

            my $pe1 = $p1;  # identical for non-character
            my $pe2 = $p2;
            if ($ptype=~ /char/i) {
              print "p1=$p1<br>" if $::debug;
              #--- check for query or a table parameter rather than a 
              #--- constraint string before adding quotes to field
	      #if ($p1 !~ /^(.*select.*)$/) {
	      #if ($p1 !~ /^(.*)$/) {
	      if ($p1 !~ /^\(.*\)$/) {
                 $pe1 = enquote($p1, $quote) unless $p1 eq "null";
              }
	      #if ($p2 !~ /^(.*select.*)$/) {
              print "p2=$p2<br>" if $::debug;
	      if ($p2 !~ /^\(.*\)$/) {
                 $pe2 = enquote($p2, $quote) unless $p2 eq "null";
              }
            }

	    if ($op =~ /between/) {
	        if ($p1 eq "null") { 
                   if (!$not) {
                     $sql .= "(($sfield <= $pe2) or ($sfield is null))";
                   } else {
                     $sql .= "($sfield > $pe2)";
                   }
                } elsif (!length($p2)) {
                   if (!$not) {
                     $sql .= "($not $sfield >= $pe1) ";
                   } else {
                     $sql .= "(($sfield < $pe1) or ($sfield is null)) ";
                   }
                } else {
	           $sql .= "(($not $sfield $op $pe1 and $pe2) ";
                   $sql .= " or ($sfield is null) " if $not;
                   $sql .= ")";
                }
	    } else {  # = or !=
	        $sql .= "(($not $sfield $op $pe1 )";
                if ($p1 eq "null") {
                   if ($ptype=~ /char/i) {
                      $sql .= ($not ? " and " : " or ");
                      $sql .= "($not $sfield = ' ')";
                   }
                } elsif ($not) { 
                   $sql .= " or ($sfield is null)";
                } 
                $sql .= ")";
	    }

        $qualgrpop ='or';
       }
	  $and = "and";
       $sql .= ")";
    }
    $$self{"qualWhere"} = $sql;
    return;
}

sub dequote {
    my ($str) = @_;
    if (!defined($str)) {
        return undef;
    }
    # Don't get rid of quotes in the middle - 
    # I suppose they could be valid.
    $str =~ s/^['"]//;
    $str =~ s/['"]$//;
    return $str;
}

sub enquote {

    my ($str) = @_;
    if (!defined($str)) {
        return undef;
    }
    
    if ($str =~ /^\s*\'.*\'\s*$/ or $str =~ /^\s*\(\'.*\'\)\s*$/) {
        return $str;
    } else {
        #$str =~ s/\'/\\\'/g;
        $str =~ s/\'/\'\'/g;
	$str = "'$str'";
	return $str;
    }
}

# This converts a parameter from the form a_name to a.name
sub chip {

    my ($str) = @_;
    
    $str =~ s/^\@//;
    $str =~ s/^(.)\_/$1\./;
    return $str;
    
}

1;
