# $Id: Parse.pm,v 1.44 2008/12/30 17:05:06 lmm Exp $
# $Log: Parse.pm,v $
# Revision 1.44  2008/12/30 17:05:06  lmm
# Fixed coordinate system parsing problem for file upload function
#   parseInputFile().
#
# Revision 1.43  2006/12/07 15:46:34  lmm
# Fixed bug that occurs with parameter page URL queries that include a
#   table that is associated with more than one mission but no mission
#   name is specified in the URL.
#
# Revision 1.42  2006/10/25 13:12:29  lmm
# Corrected regular expression match for parameter search variable
#
# Revision 1.41  2006/01/12 17:27:48  lmm
# Changes made to sort tables for Browse Keyword Interface (code
#   contributed by tam)
#
# Revision 1.40  2005/12/16 15:06:46  lmm
# Fixed bugs that prevented correct parsing of previously-saved target
#   files
#
# Revision 1.39  2005/12/14 15:48:05  lmm
# Changes made to include meta data table searches (by tam)
#
# Revision 1.38  2004/12/15 20:23:38  lmm
# Changed check for case of no tables found in get_table_list()
# Did a little code cleanup
#
# Revision 1.37  2004/01/15 20:56:23  lmm
# Added code to eliminate control characters read in from uploading
#   objects file
#
# Revision 1.36  2003/12/05 21:10:23  lmm
# Added error checking for parsing uploaded targte files with no
#    or incorrect or insufficient data.
#
# Revision 1.35  2003/12/02 20:13:00  lmm
# Added subroutine parseInputFile() that parses a target file and returns
#   a list of objects/coordinate pairs.  File can be in Pure Text File
#   format or a semi-colon/newline delimited list of objects/coords.
#
# Revision 1.34  2003/07/31 14:05:17  cwh
# Added defaults for Coordinates and Equinox.
#
# Revision 1.33  2003/06/18 15:15:57  cwh
# Map old tables (or table aliases) to new tables.
#
# Revision 1.32  2003/05/20 15:56:18  cwh
# Brought back the decoding of tableheads.
# Added a default unit for search radii.
#
# Revision 1.31  2002/12/16 18:19:51  cwh
# No longer decoding missions, tableheads, $stuff, tabletypes, radius.
#
# Revision 1.30  2002/10/15 21:00:05  cwh
# Now allowing 'deg' and 'sec' as radius units.
#
# Revision 1.29  2002/09/05 18:17:54  cwh
# Set nolog flag from input.
#
# Revision 1.28  2002/07/10 19:20:50  cwh
# Now allows a Radius_unit param.  Radius and unit can now be specified
# in two separate form fields, allowing arbitrary radius values.
#
# Revision 1.27  2002/04/10 22:09:18  cwh
# Added set_defaults() and get_table_list(), which are called by w3table.pl and
#   w3query.pl, since the same code was duplicated in both scripts.
#
# Revision 1.26  2002/03/18 16:39:57  cwh
# Added line which makes all Object type tables searched by default.
#
# Revision 1.25  2001/11/07 23:34:19  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.24  2001/09/15 13:55:26  cwh
# Cleaned up the parsing of the Radius field.
# Fixed problem where non-integer radii (from batch queries) were being truncated.
#
# Revision 1.23  2001/08/28 19:32:10  cwh
# Now keeping batch version in w3config.pl.
# Changed $batch_version to $batchflag.
#
# Revision 1.22  2001/07/01 02:34:53  cwh
# Removed redundant email address.
# Removed HEASARC-specific URL (though may want this to point to our FTP file.)
#
# Revision 1.21  2001/06/20 22:57:33  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.20  2001/05/03 18:59:28  cwh
# Changed heasarc_ to $::local_origin.
#
# Revision 1.19  2001/04/25 17:13:00  cwh
# don't lowercase and add heasarc_ to VizieR tables.
#
# Revision 1.18  2001/04/11 01:40:15  cwh
# Brought back CompoundTable - some things (ie NED) are using the @All_Archive,
# etc. definitions in w3basic.data.
#
# Revision 1.17  2001/04/09 20:31:53  cwh
# Batch strings are case insensitive now.
#
# Revision 1.16  2001/04/03 16:18:07  cwh
# added truncate_priority function.
#
# Revision 1.15  2001/03/12 16:29:46  cwh
# Added debg info.
#
# Revision 1.13  2001/01/24 17:53:22  cwh
# Not using compoundtable anymore.  The basic interface has changed.
#
# Revision 1.12  2000/03/20 21:37:19  saima
# Fixed a problem so that 'heasarc_' recognition is not case
#   sensitive.
#
# Revision 1.11  2000/03/20 15:53:51  saima
# Fixed a problem where the prefix 'heasarc_' should be searched and not
#   just '_' to determine that the 'heasarc_' prefix is not in the table
#   name.
#
# Revision 1.10  2000/03/08 20:11:52  saima
# W3Browse to Browse name change.
#
# Revision 1.9  2000/01/27 18:36:32  saima
# Modified version number to 2.0 again.
#
# Revision 1.7  2000/01/11 18:48:02  saima
# Updated $::addr. Also updated batch retrieval version number to 3.0.
#
# Revision 1.6  1999/09/24 18:58:24  saima
# Fixed bug where default radius specified with '"' was not being
#   converted.
#
# Revision 1.5  1999/06/01 18:00:50  tam
# Fixed bug with radii defined in ".
#   Made references to global functions explicit (&f -> &::f)
#
# Revision 1.3  1999/01/14 18:09:35  saima
# Version update for batch query interface.
#
# Revision 1.2  1999/01/07 20:48:21  saima
# Added CompundTable() for Basic Interface.
#
# Revision 1.1  1998/12/23 16:44:47  saima
# Initial Revision.
#
#=========================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
#  it significantly, please leave the above name & address intact.
#=========================================================================#
# Module Name:    Parse.pm
# Module Author:  Saima Zobair, Raytheon ITSS
# Date:           Nov 1998
# Project:        Browse : HEASARC
#=========================================================================#
# Module Description:
#   Routines that help parse query information in different phases
#   of the Browse query process. 
#
#=========================================================================#

use strict;

package Browse::Parse;
use Browse::Sort;
#--------------------------------------------------------------------------
sub GetTableInfo {      # ... from the information in the query string

  my($tables) = @_;
  my(@table_entries) = @ { $tables };
print "table_entries=@table_entries<br>\n" if $::debug;
print "#table_entries=".($#table_entries+1)."<br>\n" if $::debug;

  my(@tables) = ();
  my($table,@words,$table_string,$batchflag);

  Browse::Utils::Error("Error: Please go back and select one or more tables.")
          if (! scalar(@table_entries));

  foreach (@table_entries) {
     $table_string = $_;
     if($table_string =~ /BATCHRETRIEVALCATALOG/i){
       $batchflag++;
       print "GetTableInfo :: this is a batch query, setting variable ",
             "addr<br>" if($::debug);

       if ($::addr =~ /w3browse/) {  # change only if it is w3browse\@athena
           $::addr  = "tam\@silk.gsfc.nasa.gov,cwh\@lheamail.gsfc.nasa.gov";
       }

       my($batch)         = "BATCHRETRIEVALCATALOG_" . $::batch_version;
       if ($table_string !~ /$batch/i) {
          Browse::Utils::Error("\nBatchStart\n\nBATCH_RETRIEVAL_MSG ERROR: Your version " .
          "of browse_extract.pl is out of date. Please get the latest " .
          "version of this script from: " .
          "$::w3bdocs/w3batchinfo.html.\n");
       }
     }

     # The short table name is the last word of each line.
     @words = split(/\s+/,$table_string);
     $table = pop(@words);

     if ($table !~ "/") { # if not a VizieR table
          $table = lc($table);

          # if table name does not have _ means it's a heasarc table by default
          #if ($table !~ /$::local_origin/i){
          if ($table !~ /($::local_origin|^zz)/i){
            $table =  $::local_origin . $table;
          }
     }

     push(@tables,$table);
     print "GetTableInfo: table: $table<br>\n" if $::debug;
  } 
  print "GetTableInfo: ",scalar(@tables)," tables selected<br>\n" if $::debug;

  return($batchflag,@tables);

} # eosub--GetTableInfo
#------------------------------------------------------------------------
sub TableHead {

  my($in) = @_;
  my($info)= {};
  my(@tableheader,$name,$value,@tables);

  my(@stuff) = split("\0", Browse::CGI::Decode($in->{'tablehead'}));
  #my(@stuff) = split("\0", $in->{'tablehead'});
  if(scalar(@stuff) < 1) { return 0; };

  foreach (@stuff) {
    my($info)= {};
    foreach (split (/&/)) {
      ($name, $value) = split(/=/,$_,2);
       $info->{$name} = $value;
    }
    push(@tableheader,$info);

    if ($info->{'name'} =~ /\|\|/) { #Basic Interface stuff
       my (@comp_table) = split(/\|\|/,$info->{'name'});
       print "<br>this was a compound table:: $info->{'name'} became @comp_table<br>\n" if ($::debug);
       push(@tables,@comp_table);
      
    #} elsif ($info->{'name'} =~ /@/) { #Basic Interface stuff
    #   my(@comp_table) = &CompoundTable($info->{'name'});
    #   push(@tables,@comp_table);
    } else {
       push(@tables,$info->{'name'});
    }
  }
  return( [ @tableheader ] , [ @tables ] );

} # eosub--ParseTables
#------------------------------------------------------------------------
sub CompoundTable {
    # Replace a compound identifier by the tables in it.
    my ($id) = @_;
    my (@tables);

    my($file) = $::basicData;
    if (!open(FILE, $file)) {
        Browse::Utils::Error("Error: Unable to open file $file to resolve compound table ".
               "$id.");
    }
    my(@table_entries) = split("\0",$id);
    my(@mtables) = ();
    my($table,@words);

    foreach (@table_entries) {
       # The short table name is the last word of each line.
        @words = split(/\s+/,$_);
        $table = pop(@words);
        push(@mtables,$table);
    }

    $id = "";
    foreach $id (@mtables){
       if($id =~ /@/){
         my $line;
         $id = substr($id, 1);

         if ($id =~ /AllArchives/i){
            $id = "All Archives";
         }

         while ( $line = <FILE>) {
          if ($line =~ /^\s*\@\s*\|\s*$id\s*\|/) {
            my @vals = split('\|', $line);

            shift(@vals);
            shift(@vals);
            for (@vals) {
                push(@tables,$_);
            }
            last;
          }
         }
      }else{
         push(@tables,$id);
      }
    }
    print "<br>CompoundTable: tables :: @tables<br>\n" if ($::debug);
    close(FILE);
    return (@tables);

} # eosub--CompoundTable
#------------------------------------------------------------------------
sub ParamInfo {
  my($in) = @_;

  #my(@params) =  split("\0", Browse::CGI::Decode($in->{'w3xparam'}));  
  my(@params) =  split("\0", $in->{'w3xparam'});  
  print "\n<pre>ParamInfo::All params: @params</pre>" if($::debug);

  my(@param_info) = ();
  my($param) = {};
  foreach (@params) { 
    $param = {};
    foreach(split(/&/)) {
      my($name, $value) = split(/=/,$_,2);
      #print "$name	 -> $value <br>" if $::debug;
      $param->{$name} = $value;
    }
    push(@param_info, $param);
  }
  return( [@param_info] );

} #eosub--Parse::ParamInfo
#------------------------------------------------------------------------


sub PrepRadius {
  my($in) = @_;

  # FYI, the HTML entity for the degree symbol is '&#176;',
  # and 176 in hex is 0xb0.
  my($degsymbol) = "\xb0";
  #print "Parse::in->{Radius}=",$in->{'Radius'},"<br>\n" if $::debug;

  #$in->{'Radius'} = Browse::CGI::Decode($in->{'Radius'});

  if ($in->{'Radius'} =~ /^All-sky/i) {
      $in->{'Radius'} = 180 * 60;     # 180 degrees * 60 arcminutes/degree
      return ($in);
  }

  # allow old unit specification style
  my ($radius,$unit) = $in->{"Radius"} =~ /([\d.]+)(.*)$/;
  $in->{'Radius_unit'} = $unit if $unit; 

  if ($in->{'Radius'} =~ /Default/i) {$in->{'Radius_unit'} = "arcmin"};
  #convert to arcminutes
  if ($in->{'Radius_unit'} =~ /sec|\"/) {                    # arcseconds
      $in->{'Radius'} = $radius / 60;
  } elsif ($in->{'Radius_unit'} =~ /deg|$degsymbol/) {       # degrees
      $in->{'Radius'} = $radius * 60;
  }

  #print "prepped: in->{Radius}=",$in->{'Radius'},"<br>\n" if $::debug;
  return($in);

} #eosub--PrepRadius
#------------------------------------------------------------------------

sub truncate_priority {
   # This subroutine takes a $tablehead type reference to an array of hashes 
   #   with table info, and an optional maximum table priority, and returns 
   #   a new $tablehead type ref to an array of hashes of table info, where:
   #     if no maxpriority: 
   #          only returns tables where int(priority) = int(lowest priority),
   #          where the lowest priority is determined for each mission/
   #          tabletype.
   #     if maxpriority given:
   #          only returns tables whose int(priority) <= maxpriority, for all
   #          tables.
   my ($tableinfo,$maxpriority) = @_;
   my @newtables=(); 
   my %maxpriority= {};
   my %seen= {};
   # sort tableinfo by priority
   my @newtab = sort { $$a{"priority"} <=> $$b{"priority"}} @$tableinfo; 
   foreach (@newtab) {
       my $blah = $$_{"mission"}.$$_{"tabletype"};

       unless ($seen{$blah}) {
          $maxpriority{$blah} = $$_{"priority"} 
		unless (($$_{"priority"} > $maxpriority{$blah}) 
	        and defined($maxpriority{$blah}));
          $maxpriority{$blah} = $maxpriority if $maxpriority;
          $seen{$blah}++;
       }
   }
  
   #if ($::debug) { 
       #foreach (keys %maxpriority) {
           #print "max priority for $_ : $maxpriority{$_}<br>\n";
       #}
   #}

   foreach (@newtab) {
       my $blah = $$_{"mission"}.$$_{"tabletype"};
       push @newtables, $_ if (int($$_{"priority"}) <= $maxpriority{$blah} 
              or $$_{"tabletype"} =~ /Object/i);
   }

 #      my $first = $newtab[0];
 #      $maxpriority ||= int($$first{"priority"})+0.999999999;
 #      print "maxpriority=$maxpriority<br>\n" if $::debug;
 # 
 #      foreach (@newtab) {
 #        push @newtables, $_ if int($$_{"priority"}) <= $maxpriority;
 #      }
  
   return \@newtables;

}
#------------------------------------------------------------------------

sub set_defaults {
   my ($in) = @_;

   $::nolog = 1 if defined $$in{"nolog"};
   
   print "Parse: setting defaults..." .$in->{displaymode} ."<br>\n" if $::debug;
   # get coordsys and equinox vars separated
   if ($in->{"Coordinates"} =~ /^[JB]\d/ or !defined $in->{"Coordinates"}) {
      ($in->{"Equinox"}) = $in->{"Coordinates"} =~ /([\d]+)/;
      $in->{"Equinox"} ||= "2000";
      $in->{"Coordinates"} = "Equatorial";
   }

   $in->{'maxpriority'} ||= 99;  # all if not defined
   $in->{'maxpriority'} ='' if $in->{'maxpriority'} =~ /def/;

   $in->{"ResultMax"} = 1000 unless (defined $in->{"ResultMax"});
   $in->{"sortby"} = 'priority' unless $in->{"sortby"};

   return $in;
}

#------------------------------------------------------------------------

sub get_table_list {

    my ($in) = @_;
    my $sortTables = 0;

    # get tabletypes
    my (@tabletypes) = ("Observation","Object","Proposal","Physics");
    @tabletypes = split("\0", $in->{'table_type'}) if $in->{'table_type'};

    # get missions and individual tables (multiples can be joined with '||'s)
    my @missions   = ();
    my @moretables = ();
    
    foreach my $key (keys %$in) {
        #print "Parse: key $key ->" . $in->{$key}  ."<br>\n" if $::debug;
        push @missions,   split(/\0|\|\|/, $in->{$key}) 
		if $key=~ /^Observatory/i;
        push @moretables, split(/\0|\|\|/, Browse::CGI::Decode($in->{$key})) 
		if $key=~ /^tablehead/i;
    }

    # eliminate duplicate missions
    my %mseen=();
    @missions = grep {! $mseen{$_}++ } @missions; 

    # get tables from missions
    my $tablehead=[];  # keep this here - for mod_perl!
    if (@missions) {
        $sortTables = 1;
        $tablehead = Browse::HDB::GetQueryTables([ @missions ],[ @tabletypes ]);
        #print "Parse:: tablehead dump<br>" if $::debug;
        #Browse::Utils::DumpRef($tablehead) if $::debug;
    }

    # set up to eliminate duplicate tables
    my %seen=();
    my @orderedTables= ();
    foreach (@{$tablehead}) { 
        $seen{$_->{'name'}}=$_; 
	push(@orderedTables, $_);
    }

    my $allvizier=0;
    foreach (@moretables) {
        s/name=//i;
	
        $_ = $::maptables{$_} if ($::maptables{$_});
        $_ = "name=".$_ unless /name=/;
	
        # handle tables specified with aliases: @ASCA 
        if (/@/) {
	    $sortTables = 1;
            s/name=//;
            s/&.*//;
            my @comptables=();
            @comptables=Browse::Parse::CompoundTable($_) if /@/;
            foreach (@comptables) {
                $_="name=".$_ unless /name=/;
            }
            push @moretables, @comptables;
            next;
        }
    
        my $info = {};
        foreach (split (/&/)) {
            my ($key, $value) = split(/=/,$_,2);
            $info->{$key} = $value;
        }
	
        my $name = $info->{'name'};
	
        if ($name =~ /^Z\/0$/) {
	    $allvizier = 1;
        }
	
        $name = $::local_origin . $name unless $name =~ /$::local_origin|\/|^zz/i;
        next if $seen{$name};

        if ($in->{"Action"} =~ /parameter|options|detailed mission/i) {
            my ($info2) = Browse::HDB::get_table_meta($name);
            foreach (keys %$info2) {
                $$info{$_} = $$info2{$_} unless defined $$info{$_};
		
		# Check if there is a vizier query being done.
                if ($_ =~ /tabletype/i && $$info{$_} =~ /vizier/i) {
                    $in->{'vizier_query'}=1;
                }
            }
            #-- (Parameter page only) Fixing problem where table but 
            #-- not mission is specified and the table is associated 
            #-- with more than one mission.  (The || used to separate 
            #-- missions here is used to determine tables in other code).  
            #-- This code chooses the first mission.   
            if (!scalar(@missions) && $info->{mission} =~ /.*\|\|.*/) { 
                my ($firstmission) = split(/\|\|/,$info->{mission}); 
                $info->{mission} = $firstmission;
            }
	    
        } else {
	
            my @fields = split("\&", $name);
            my $fld;

            foreach $fld(@fields) {
                my ($key, $val) = split("=", $fld, 2);
                $info->{$key} = $val;
            }
        }

        # If you ever want to make the tables sensitive to the table_types
        #     selected by the user, un-comment the lines marked with '# **'
        # ** foreach (@tabletypes) {
        # **    if (/$info->{'table_type'}/) {
        $info->{'name'} = $name;
        $seen{$name}    = $info;
        
	push(@orderedTables, $info);
        # **    }
        # **}
    } # end for moretables

    # get Vizier tables, add them to the seen hash
    if (defined($in->{"vizier_query"}) and !$allvizier) {
    
         Browse::Vizier::getTables($tablehead, \@missions, $$in{"ExpVizier"});
         foreach (@{$tablehead}) { 
             $seen{$_->{'name'}}=$_;
         }
    }

    
    if ($sortTables) {
        # eliminate duplicate tables by just taking tables from values of seen hash
        @$tablehead= values(%seen);
        $tablehead = Browse::Sort::sortTables($tablehead, $in->{"sortby"});
	
    } else {
        $tablehead = \@orderedTables;
    }
    

    # get the default tables based on priority, set the message to let the user
    #  know that not all tables were queried.
    
    my $default_tables  = Browse::Parse::truncate_priority($tablehead,$$in{'maxpriority'});
    
    my $numalltables    = scalar @$tablehead;
    my $numselecttables = scalar @$tablehead;

    if ($numselecttables < $numalltables) {
        my $newquery = $$in{"input"};
        $newquery =~ s/maxpriority=[^&]*/maxpriority=/;
        $$in{"msg"} .= "There are ".($numalltables-$numselecttables) .
	  " additional tables available, but with lower priority, " .
	  "which were not queried.  <a href=w3query.pl?$newquery>" .
	  "Query all $numalltables tables</a>\n";
    }

    # complain if no tables found matching criteria
    unless (  scalar @$tablehead  )  {
        my $tabletypes = "for table type(s) " .join(" or ",@tabletypes);
        Browse::Utils::Error("Error: Sorry, could not find any ".
           " selected tables $tabletypes. </h2>".
           "You may wish to select additional table types or missions/categories.".
           "<p>The four table types are:<ul>".
           "<li>Observation: Lists of Observations actually taken or scheduled.".
           "<li>Object:      Compilations of discrete sources in the sky.".
           "<li>Proposal:    Proposals and proposed observations.".
           "<li>Physics:     Non-astronomical data, e.g., line energies.".
           "</ul>");
    }

    return($tablehead, \@missions, \@tabletypes, $in, $default_tables);

}

#------------------------------------------------------------------------
#  Parse file of saved query results.  Return unique targets.
#  may be list of targets, coords delimited by semicolons or 
#  newlines or "Pure Text" output.  Check that coordinate systems match

sub parseInputFile {
   my ($in)= shift @_; # data string from file
   my ($queryCoordSystem)= shift @_; # coordinate system of query
   #--- Remove leading and trailing spaces for string comparisons
   $queryCoordSystem =~ s/^\s+//;
   $queryCoordSystem =~ s/\s+$//;
   
   my @coordslist;
   my $errorMessage = "";
   my ($finalEntry)="";
 

   my $foundColumns =0;
   my $foundCoords =0;
   
   # $in holds one string with all file data so look for clue that 
   # it is Pure Text
   if ($in =~ /Coordinate system/) {   
      # File is in Pure Text Format.  Spilt input into separate tables
      my (@tables) = split(/Results from/,$in);

      for my $table (@tables) {
         if ($table =~ /\S/) {
            #print " Parse:table: $table<br>" if $::debug;
            # split table on new line
            my @tablelines = split(/\n/,$table);
            my ($label,$cs,$i);

            # Find line with "Coordinate system" and save value
            for ( $i=0; $i < $#tablelines;++$i) {
               if ($tablelines[$i] =~ /Coordinate system/) {
                  ($label,$cs) = split(/:/,$tablelines[$i]); 
                  #--- Remove leading and trailing spaces for string comparisons
                  $cs =~ s/^\s+//;
                  $cs =~ s/\s+$//;
                  #print "System=$cs<br>" if $::debug;
                  last;
               } else {
                  next;
               }
            }

            # from coordinate system determine what string to look for in header
            # Equatorial coords could be J2000, J1950 or Equatorial
            if ($queryCoordSystem eq "J2000" || $queryCoordSystem eq "J1950") {
		$queryCoordSystem = "Equatorial";
   	    }
            if ($cs eq "J2000" || $cs eq "J1950") {
		$cs = "Equatorial";
   	    }
 
            #print "CS $cs---->$queryCoordSystem<br>\n" if $::debug;
            if ($cs !~ /$queryCoordSystem/) {
               $errorMessage= "Error: Coordinate system mismatch.<p>" .
                "The system for upload file coordinates is $cs.  The " .
                 "system selected on the Search Form is $queryCoordSystem." ;
	
            }
            my $lookfor;
            if ($cs =~/Equatorial/) {
               $lookfor="^ra|a_ra|^RA";    # regular or cross_cor or vizier
            } else {$lookfor ="lii"};

            # skip to header line if necessary
            while ($tablelines[$i] !~ /^\|/ && $i < $#tablelines) {
               ++$i;
            }
            
            # read next line that should start with | and look for 
            # correct column to read
            if (($i < $#tablelines) && ($cs =~ /\S/) 
			            && ($tablelines[$i] =~ /^|/)) {
               my @header = split (/\|/,$tablelines[$i]);
               my $j;
               for ( $j=0; $j <= $#header;++$j) {
                  if ($header[$j] =~ /$lookfor/i) {
                     $foundColumns = 1;  # coords were found!
                     last;
                  }
               }
               if ($foundColumns) {
                  my $column=$j;   # column used to get coordinates
            
                  my $coordset;
                  my %seen = ();
                  for ( $i=$i+1; $i <= $#tablelines; ++$i) {
                     @header = split (/\|/,$tablelines[$i]);
                     $coordset = $header[$column] . ", " . $header[$column+1];

                     #print "Parse: coord pair=$coordset<br>\n" if $::debug;

                     if ($coordset =~ /\d/) {
                        $foundCoords=1;
                        #  add only if unique
                        push(@coordslist,$coordset) unless $seen{$coordset}++;
                     }
                  }
               } # columns for coords were found
            }
         } else {next;}
      }     
   } elsif ($in =~ /\S/) {
      # File is not in Pure Text Format. 

      # replace newlines with semicolons 
      $_=$in;
      tr/\n/;/;
      $in=$_;
      #print "parse: $in<br>" if $::debug; 
      my @entry=split(/;/,$in);
      my %seen = ();
      foreach my $x (@entry) {

        if ($x =~ /\S/) {
           $foundCoords=1;
           $_=$x;
           s/\r//;# try to eliminate any control characters to compare
           #print "parse tr:*$_*<br>" if $::debug; 
           push(@coordslist,$x) unless $seen{$_}++;
        }  else {
           next;
        }
      }
   }
   if (!@coordslist || !$foundCoords) {
      $errorMessage= "Parser could not find coordinates or objects in" .
         " the upload file.  Please check file format or Browse Help" .
         " and try again.";
   }
   # Return list of targets/coordinates separated by semicolon
   $finalEntry = join(";", @coordslist);
   return ($finalEntry, $errorMessage);
}

1;
#########
#  EOF  #
#########
