#$Id: ProdHera.pm,v 1.19 2008/01/30 15:23:41 lmm Exp $
#=======================================================================#
# Package Name:   ProdHera.pm
# Package Author: Laura McDonald, SSAI
# Date:           02/05
# Project:        Browse : HEASARC
#=======================================================================#
# Module Description:
# Transmit data products to Hera system for analysis
#
# Handles both Anonymous Hera and Save to Hera options.
# Save to Hera option tranfers files to Hera system after 
# user logs in with registered ID and password.
# Anonymous Hera transfers data to an newly generated account 
# (no registration required) and if browser is configured and FV software
# is installed a hera session will start up automatically.
#=======================================================================#
use strict;

use Browse::ProdMain;
use Browse::Utils;
use LWP::Simple;
use Sys::Hostname;

package Browse::ProdHera;

# Inheritance.
use vars qw( @ISA );
@ISA = qw( Browse::ProdMain );

#----------------------------------------------------------------------------

#--- file type needed by hera to determine directory type
sub filetype {
    my ($self,$type) = @_;
    $$self{'filetype'} = $type if defined $type; 
    return $$self{'filetype'};
}

#--- Build AnonHera entry for Tar log 
sub LogMessage {
    my ($self,$input) = @_;
    if ($$self{"logmessage"}) {
      $$self{"logmessage"} .= ";" . $input if $input;
    } else {
      $$self{"logmessage"}  = $input if $input;
    }
    return $$self{"logmessage"};
}

#--- input values passed along through Hera form
sub Input {
    my ($self,$input) = @_;
    $$self{"input"} = $input if $input;
    return $$self{"input"};
}


#--- query search radius
sub searchRadius {
    my ($self,$radius) = @_;
    $$self{"radius"} = $radius if $radius;
    print "ProdHera:: radius = $$self{radius} <br>" if $::debug;
    return $$self{"radius"};
}

#--- query search coordinates
sub targetCoords {
    my ($self,$coords) = @_;
    $$self{"coords"} = $coords if $coords;
    print "ProdHera:: coords = $$self{coords} <br>" if $::debug;
    return $$self{"coords"};

}

#---------------------------------------------------------------------------
#--- overwritten method applied for all products
#--- creates the file/URL list
sub doProd {
   my ($self,$val) = @_;
   my $url = $$val{'dp_url'};
   my $filelist=();
 
   print "ProdHera:looking at file $url...<br>" if $::debug;
   if ($$val{'dp_format'} =~ /\bDIRECTORY\b/i) {
      #--- current product is a directory so process all its files 
      $filelist = $self->descendtree($url, $filelist);
      foreach my $fileFromDir (@$filelist) {
         if ($self->applyFileFilter($fileFromDir) || 
            	!$self->{controlid} =~/AnonHera/) {
            $self->Request($fileFromDir);
         }
      }
   } elsif ($url =~  /ftp\:\/\/[^\/]*(\/.*)/) {
         #--- one file
         my $file = "/FTP$1";
         $file = $url unless -s $file;
         if ($self->applyFileFilter($file) || 
		!$self->{controlid} =~/AnonHera/)  {

   print "ProdHera:adding at file $url...<br>" if $::debug;
            $self->Request($file);
         }
   } else {
         #--- remote file
         if ($self->applyFileFilter($url)) {
            $self->Request($url);
         }
   }
}

#---------------------------------------------------------------------------
#--- overwritten method applied to all rows
sub closeRow {
   my($self)=@_;

   #--= Send data products for this row to Hera 
   print "ProdHera::closeRow:prossesing row<br>" if $::debug;
   $self->processRow();

   #--- clear out data products before starting next row
   print "clearing out requests<br>" if $::debug;
   undef $self->{request};
}

#---------------------------------------------------------------------------
#--- overwritten method applied to all rows
sub doKey {
    my ($self,$table,$tableInfo,$key) = @_;
    # get column name/value pairs for each row to send to Hera
    $self->{HeraTransmission}="";
    $self->{HeraTransmission} .= $self->rowHeader($table,$tableInfo,$key);
}

#---------------------------------------------------------------------------
#--- Grabs column name and value for current row
sub rowHeader {
    my ($self,$table,$tableInfo,$key) = @_;
    my $in=$self->Input();
    my $target;
    (my $ptable = $table) =~ s/$::local_origin//ig;

    #--- table name
    my $rowTransmission="TABLE=$ptable\n";  #first line
    #$self->LogMessage("table=$ptable");  # info for log file

    #--- query coordinates
    #--- If offest is included in row data use it rather than search object
    #--- for ra and dec
    if ($in->{offsetTarget}) {
       my (@offsets)=split("\0",$in->{offsetTarget});
       print "Prodhera::rowHeadr::offsT = " .$in->{offsetTarget} . 
	  "num=". scalar(@offsets) ." key=$key table=$ptable<br>\n" if $::debug;
       for my $elem (@offsets) {
           $elem =~ /(.*)_unique_(.*)\|\|\|(.*)\|\|/;
           my $offTarget=$1;
           my $offTable=$2;
           my $compkey=$3;
           print "targ=$offTarget, table=$offTable heasarc_$ptable" .
		" ckey=$compkey== $key<br>" if $::debug;

           if ($offTable =~ /\bheasarc_$ptable\b/ && $compkey eq $key) {
              print "calling resolve $ptable $key $offTarget<br>" if $::debug;
              $self->targetCoords($self->resolveCoords($offTarget));
           }
       }
    }
    my $coords=$self->targetCoords();
    
    if ($coords) {
      my ($searchra, $searchdec)=split(/,/,$coords);
      if ($searchra && $searchdec) {
      print "Prodhera::SearchRA,SearchDec = $coords<br>\n" if $::debug;
         $rowTransmission .= "SearchRA=" . 
	     Astro::CoordConv::RA_Decimal2HMS($searchra,10) . "\n";
         $rowTransmission .= "SearchDec=" .
	     Astro::CoordConv::Dec_Decimal2DMS($searchdec,10) . "\n";
      }
    }

    #--- query radius
    #if ($self->searchRadius()) {
    #--- if Save to Hera use radius used for table, else use X radius parameter 
    if ($in->{"radius_$table"}) {
       $rowTransmission .= "SearchRADIUS=" .$in->{"radius_$table"} . "\n";  
    } else {
       $rowTransmission .= "SearchRADIUS=" .$self->searchRadius() . "\n";  
    }

    my $rowid=$self->rowid(1);
    my ($coordsys, $equinox) = @{$self->coosys()};
    my ($row, $names, $transInfo) = Browse::RowDisplay::getSingleRow($table, 
        $key, undef, $tableInfo, $self->transInfo, $coordsys, $equinox);

    return 0 unless $row;
    my $i;

    #--- row column names and values
    for ($i=0; $i <= $#$row; $i += 1) {
       $$row[$i] =~ s/&nbsp;/ /g;    # remove some html
       $rowTransmission .=  "$$names[$i]=$$row[$i]\n"; 
       if ($$names[$i] =~/target/) {
           $target = $$row[$i];
       }
    }
    return $rowTransmission;
} 

#---------------------------------------------------------------------------
#--- get coordinates for target
sub resolveCoords {
   my ($self,$target) = @_;
   my ($coordsys, $equinox) = @{$self->coosys()};
   print "coneQuery: resolving $target <br>" if $::debug;
   my %coneQuery=();
   $coneQuery{Entry}=$target;
   $coneQuery{Coordinates}=$coordsys;
   $coneQuery{Equinox}=$equinox;
   if ($::debug) {
       foreach my $x (keys %coneQuery) {
            print "coneQuery: $x ->$coneQuery{$x} <br>";
       }
    }
   my $parameter = Browse::Parameter->new("Parameter");
   my ($positions,$resmsg)=$parameter->MakeConeQuery(\%coneQuery);
   #my ($positions,$resmsg)=Browse::Parameter::MakeConeQuery(\%coneQuery);

   my $searchPositions=@$positions[0];
   my ($ra,$dec,$search_radius,$entry,$resolver)= 
	split('\|',$searchPositions);
   return $ra ."," . $dec;
}


#---------------------------------------------------------------------------
#--- if AnonHera is used Set MIME type to start up fv after files 
#--- are transfered 
#--- If Save To Hera function has been selected display the number 
#--- of files saved
sub finish {
   my ($self,$in) = @_;

   if ($self->{controlid} =~/AnonHera/) {
      
      Browse::Log::WriteLog("[". DateTime::asctim()."] $::REMOTE_HOST :: " . 
	"AnonymousHera - " . $self->LogMessage(), $::hera_DPLog);

      print "Content-Type: text/x-hera\n" .
          "Content-Disposition: inline; filename=browsehera.xhera\n\n";
      print "USERID=" . $self->{username};

      print "\n% if you are reading this message then you need to perform \n" .
	"% 2 steps to configure your web browser before you can use the \n" .
	"% Hera data analysis service provided by the HEASARC:\n"; 


      print "% 1) install Fv (a small FITS file viewer program) on your " .
	"machine, and\n";

      print "% 2) configure your web browser to use Fv as a helper " .
	"application\n";

      print "% Please follow the simple instructions on the Hera web " .
	"page at \n";

      print "%      http://heasarc.gsfc.nasa.gov/hera/\n";
 

   } else {
     #my $msg = "No data products found.<br><br>Please go back and make" .
	#" sure you have at least one table row and at least one data" .
	#" product type selected.  If so, the reason may be that we do" .
	#" not have any data products of the type selected for the table" .
	#" rows selected - for example, an observation you chose may be" .
	#" scheduled, or performed, but not yet in the archive.";

    if ($self->{localfilecount} ==0 && $self->{remotefilecount} == 0) {
        # Browse::Utils::Error("$msg");
        $self->noProductsMessage();
    }

     Browse::Decor::Header("Save To Hera",0,0,0,"dprodtable" );
    
     #--- Display file transfer counts 
     my $count = "";
     $count .= $self->{localfilecount} . " local files" 
	if $self->{localfilecount};
     $count .= " and " if $self->{localfilecount} and $self->{remotefilecount};
     $count .= "$self->{remotefilecount} remote files" 
	if $self->{remotefilecount};
     print "<h3>Hera -- Astronomical Data Analysis over the Internet</h3>\n";
     print "<p>Success: $count were saved to ". $self->{controlid} .
	"\'s Hera disk space area.<p>\n";
     print "To access these data, you need to connect to " .
	"<a href='/hera/'>Hera</a> using " .
	"the <a href='/docs/software/ftools/fv/'>fv  program</a>.";


    #--- Connect to Hera button
    print "<p>If you have set up fv has a browser helper application " .
	"you can ";

     print "<form action='connecthera.pl'>\n".
      "<input type='submit' value='Connect To Hera' name='connecthera'>" .
	"&nbsp;(the connection may take up to 1 minute to start)<p><p>";

     print "<p>Alternatively, you can start fv on your computer and then " .
	"select the <b>Connect to Hera</b> menu option.  This can be done " .
	"directly by adding the -hera option to the fv command, as in:<p>" .
    	"<pre>fv -hera</pre><p>" .
	"See the <a href='/hera'>Hera web page</a> for more information.<p>\n";

      print "<hr> ".
      "<input type='button' value='Close window' onClick='self.close()'> ".
      "</form>";


   }

}

#---------------------------------------------------------------------------
#--- process input hash and then decide which Hera function to use
sub pageHeader {
   my ($self,$in) = @_;
   my $interface=$in->{'Save to Hera'};
   
   $self->Input($in);

   #--- initialize file counts for later reporting
   $self->{localfilecount}=0;
   $self->{remotefilecount}=0;

   print "ProdHera::pageHeader " . $$in{"controlid"} . " " .
	$in->{Entry} . " " . $in->{Radius} ."<br>" if $::debug;

   # Distinguish between Anonymous and Regular Hera
   $self->{newuser}=0;
   if ($interface=~ /\w/) {
      
      $$in{"controlid"} =~ /([-\w._]+)/; #untaint controlid
      if ($$in{controlid} ne $1) {
	 $self->quit("Your Hera ID contains illegal characters. " .
	 " Please enter a different ID.");
      }
      $self->{controlid} = $1 if $$in{"controlid"};

      $$in{"password"} =~ /([-\w._]+)/; #untaint password
      if ($$in{password} ne $1) {
	 $self->quit("Your Hera password contains illegal " .
	 "characters.  Please enter a different password.");
      }
      $self->{password} = $1 if $$in{"password"};

      #--- if first time through display Hera login form and exit
      $self->{newuser} = 1 if $$in{"newuser"} =~ /on/;
      if (!$self->{controlid} || !$self->{password}) {
         $self->printform();
         Browse::Decor::EmailFooter();
         Browse::Decor::EndTags();
         exit;
      }

      #--- AFTER first time through ----
      #--- For Save to Hera option from Preview and Retrieve page data 
      #--- products have already been gathered.  Process them now.
      my @totallist;
      foreach (keys %$in) {
         #--- keys starting with file should be list of files separated by \0
         if (/^File/) {
            foreach my $file (split("\0",$$in{$_})) {
               #print "checking $file<br>" if $::debug;
               if (-d $file) {
                  #--- Get all files in directory
                  my $filelist=();
                  $filelist = $self->descendtree($file, $filelist);
                  for my $ff (@$filelist) { 
                     $self->Request($ff);
                  }
               } else {
                  $self->Request($file);
               }
            }
         }
      }
     $self->closeRow();

   } else {
       $self->{controlid} = "AnonHera";
       $self->{password}="defaultPassword";
   }
   #--- Read coords and radius (should be in arc minutes) after logging in
   $self->targetCoords($self->resolveCoords($in->{Entry})) 
	unless $self->targetCoords();
   $self->searchRadius($in->{Radius}) unless $self->searchRadius();  
   
}

#---------------------------------------------------------------------------
#--- run special tar script to get file names
sub dotarscript {
    my ($self,$file) = @_;

    my $files="";
    # need to stay on same host so I know where to look for tarfile
    my $host = Sys::Hostname::hostname();
    $file =~ s/heasarc|heasarcdev/$host/;

    #--- use listonly parameter to get list of files
    my $lst=LWP::Simple::get($file."&listonly=1");
    
    foreach my $ff (split(/\n/,$lst)) {
        $files .=$self->dolocalfile($ff);
        ++$self->{localfilecount};
        
    }
    
    return $files;
}


#---------------------------------------------------------------------------
#--- Create data products file list to send to Hera manager
sub processRow {

    #my ($self,$in) = @_;
    my ($self) = @_;

    #-- if no data files skip this row
    unless ($self->Request()) {
       return;
    }
    my @files = keys %{$self->Request()};
    
    unless (scalar(@files)) {
      return;
    }
     

    # determine if file is local, remote or XTE and call appropriate 
    # subroutine
    foreach my $file(@files) {
     
        if ($file =~ /[\!\$\;]/) {
            print "Invalid file: $file<br>\n";
	} elsif ($file =~ /xteTar\.pl/) {
            $self->{HeraTransmission} .= $self->dotarscript($file);
	} elsif ($file =~ /\:|%3A/i) {
            #print " ProdHera: remote file=$file<br> " if $::debug;
	    $self->{HeraTransmission} .=$self->doremotefile($file);

            ++$self->{remotefilecount};
        } else {
	    $self->{HeraTransmission} .= $self->dolocalfile($file);

            ++$self->{localfilecount};
        }
    }

    #print "Content-Type: text/plain\n\n";
    #print  "Transmission::". $self->{HeraTransmission}."<br>";

    $self->transmitToHera();
    
}


#---------------------------------------------------------------------------
#--- create Hera directives for remote data products, some may be output 
#--- from CGI scripts
sub doremotefile {
     my ($self,$file) = @_;
     $file =~ /^([^;\s]*)$/;
     $file = $1;
     my $files="";

     if ($file =~ /ftp/i) {
        $files .=  "DPFILE=$file\n";
     } else {
        $files .=  "DPFILE=$file\n";
     }

     return $files;
}

#---------------------------------------------------------------------------
#--- create Hera directives for local files
sub dolocalfile {
     my ($self, $file) = @_;
     $file =~ /^([^;\s]*)$/;
     $file = $1;

     my $files="";

     if ($file) {
        $files .=  "DPFILE=" . "ftp://" . $::ftphost . "$file\n" ;
     } 
     return $files;
     
}

#---------------------------------------------------------------------------
#--- Tranfer row info and names of data product files to Hera
sub transmitToHera {
   my ($self) = @_;

   # Hera connection variables
   my $heraserver = 'herasrv.gsfc.nasa.gov'; # heraserver machine
   #my $port = 9468; # port on heraserver
   my $port = 60000; # port on heraserver
   my $id = 'HeraBrowser'; # string to identify me to the Hera manager
   my $key = rand(); # session key

   my $ans;
   #--- Write hera string to file to get accurate size?
   my $transmissionfile="/tmp.shared/W3Browse/browse-hera/herafile.$$.txt";
   open OUTPUT, ">$transmissionfile" or 
	Browse::Utils::Error("Can't create Hera file");
   print OUTPUT $self->{HeraTransmission}; 
   close OUTPUT; 
   
   #--- Info for log file.  Put commas between files and parameters
   ( my $logentry=$self->{HeraTransmission})=~ s/[\n\r\0]/, /g; 
   $self->LogMessage( $logentry ); # info for log file

   #--- Establish connection and authenticate user
   my $heramgr = $self->get_socket_conn($heraserver,$port);
   $self->establish_channel($heramgr,$id,$key);
   my $username=$self->authenticate_user($heramgr,$id,$key);

   $self->{username} = $username; #save for later use

   #--- Create a temporary file name
   my $tmpname= "tmp." .rand(); 

   my $filetype=$self->filetype();

   #--- Transmission start 
   print $heramgr 
	"DataTransmissionStart $id $key $username $filetype $tmpname\n";

   print "sent <b>DataTransmissionStart $id $key  $username $filetype " .
	"$tmpname</b><br>\n" if $::debug;   #debug

   #--- get Hera response
   $ans = <$heramgr>;
   print "got <b>$ans</b><br>\n" if $::debug;

   if ($ans !~ /DONE/i) {
        mailalert ("Unable to copy file $tmpname: $ans<br>");
        print "start transmission failed  for file $tmpname: $ans<br>\n" 
		if $::debug;
   } else {
        print "Sent start notice for file $tmpname: $ans<br>\n" if $::debug;
   }
  
   #--- Get and send the file.  Use current host to find file.
   my $ua = new LWP::UserAgent;
   my $request = HTTP::Request->new(GET => "http://" .
	Sys::Hostname::hostname() ."/$transmissionfile");
   $request->header(Accept => 'text/*');
   my $response = $ua->request($request,
      sub {
        #--- send data length
        print $heramgr "DataLength $id $key ".length($_[0]).
                       " $username $tmpname\n";
        #--- send file contents
        print $heramgr $_[0];

        print "Sent DataLength $id $key ".length($_[0]).
                " $username $tmpname\n" if $::debug;

        #--- get Hera response for file transfer
        $ans = <$heramgr>;
        print "got <b>$ans</b><br>\n" if $::debug;

        if ($ans !~ /DONE/i) {
           mailalert("Error: Unable to read Hera data file " .
	      "$transmissionfile  response=$ans");
           $self->LogMessage("datalength=0");  #info for log file
        }  #else {
           #$self->LogMessage("datalength=" . length($_[0])); #info for log file
        #}
     }
   );

   unless ($response->is_success) {
          mailalert("Error: Unable to read Hera data file $transmissionfile");
   }

  
   #--- Transmission end
   print $heramgr "DataTransmissionEnd $id $key $username $tmpname\n";
   print "sent <b>DataTransmissionEnd $id $key $username " .
	" $tmpname</b><br>\n" if $::debug;

   $ans = <$heramgr>;
   print "got <b>$ans</b><br>\n" if $::debug;

   close_channel($heramgr,$id,$key,$username,$self->{password});

  
   print "unlinking <b>$transmissionfile</b><br>\n" if $::debug;
   #unlink ($transmissionfile) or
          #mailalert("Error: Unable to delete file $transmissionfile");

}  #--- end transmitToHera

#---------------------------------------------------------------------------
# Authentication
sub authenticate_user {
  my ($self,$heramgr,$id,$key) = @_;

  #$self->filetype("DATA_FILE");   # default 
  # QUERY_FILE file type now applies to both anon and registered id transfers
  $self->filetype("QUERY_FILE");   
  
  print $heramgr "Authentication $id $key " . $self->{controlid} . 
	" " . $self->{password} . "\n";
  print "sent <b>Authentication $id $key " . $self->{controlid} . " " .
	$self->{password} . "</b>" .  " new user flag=" . $self->{newuser} .
	"<br>\n" if $::debug;   #debug

  my $ans = <$heramgr>;
  print "got <b>$ans</b><br>\n" if $::debug; #debug

  my $user=$self->{controlid};  #Default for case of matching id and password

  if ($ans =~ /NEW_RECONFIRM/ and !$self->{newuser}) {
    #--- Check for Anon Hera account id OR non valid user id
    if ($self->{controlid} =~/AnonHera/) {
        my ($msg, $user) = split (/\s+/, $ans);

        print "Anon user= <b>$user</b><br>\n" if $::debug; #debug
        $self->LogMessage("ID=$user");  # info for log file

        #$self->filetype("QUERY_FILE");
        return $user;
    } else {
       # Non AnonHera and user name not found, but user said they were not new
       print "user name not found, but user said they existed<br>\n" 
	   if $::debug;

       # cancel new user
       print $heramgr "RequestCancelUser $id $key " . $self->{controlid} . "\n";

       print "sent <b>RequestCancelUser $id $key " . $self->{controlid} . 
	   "</b><br>\n" if $::debug;

       $ans = <$heramgr>;
       print "got <b>$ans</b><br>\n" if $::debug;

       unless ($ans =~ /DONE/) {
          mailalert("RequestCancelUser $key " . $self->{controlid} .
	    " failed: $ans");
       }
       $self->quit("User " .$self->{controlid} . " is not an existing user.  Please ".
            "check that the name is entered correctly.<br>\n");
    }

  } elsif ($ans =~ /FAIL/ and $self->{newuser}) {
    #--- Failed so user name selected by new user must already exist
    print "user name found, but user said they were new<br>\n" if $::debug;
    $self->quit("Username " . $self->{controlid} . " is already taken.  Please try ".
         "another username.<br>\n");

  } elsif ($ans =~ /FAIL/ and !$self->{newuser}) {

    #--- User/passwd do not match, user is not new so QUIT
    $self->quit("The username and password you typed do not match our records.  ".
         "Please try again.<br>\n");

  } elsif ($ans =~ /NEW_RECONFIRM/ and $self->{newuser}) {
    #--- Create User Space if new user (NOT AnonHera)
    # user name not found, and user said they were new - create user space

    print "user name not found, and user said they were new - ".
          "create user space<br>\n" if $::debug;

    $user=$self->{controlid};
    print $heramgr "CreateUserSpace $id $key " . $self->{controlid} . "\n";
    print "sent <b>CreateUserSpace $id $key " . $self->{controlid} .
	  "</b><br>\n" if $::debug;
    $ans = <$heramgr>;
    print "got <b>$ans</b><br>\n" if $::debug;
    unless ($ans =~ /DONE/) {
       mailalert("CreateUserSpace $id $key " .$self->{controlid} .
	  " failed: $ans");
       $self->quit("Unable to create your user space. <br>\nMail has been 
		sent to the maintainers.<br>\n");
    }
    print "Created new user space<br>\n" if $::debug;
    return $user;
  }
  print "authentication done<br>\n" if $::debug;
  return $user;
} # end sub authenticate_user

#---------------------------------------------------------------------------
#---  Establish Trust Channel
sub establish_channel {
  my ($self, $heramgr,$id,$key) = @_;
  print $heramgr "EstablishTrustChannel $id $key\n";
  print "sent <b>EstablishTrustChannel $id $key</b><br>\n" if $::debug;
  my $ans = <$heramgr>;

  print "got <b>$ans</b><br>\n" if $::debug;

  unless ($ans =~ /AGREED/) {
     mailalert("EstablishTrustChannel $id $key failed.\n $!\n $ans");
     $self->quit("Can't establish trusted channel with Hera manager: $ans<br>\n".
          "Mail has been sent to the maintainers.<br>\n");
  }
  print "got trusted channel<br>\n" if $::debug;
}

#---------------------------------------------------------------------------
#---  Get socket connection
sub get_socket_conn {
  my ($self, $heraserver,$port) = @_;
  my $heramgr = new IO::Socket::INET (PeerAddr => $heraserver,
                                     PeerPort => $port,
                                     Proto    => 'tcp');

  unless ($heramgr) {
    #mailalert("Unable to contact Hera manager.\n $!");
    $self->quit("Unable to contact Hera manager.  Reason: $!<br>\n".
         "Mail has been sent to the maintainers.<br>\n");
  }
  print "got socket connection to $heraserver<br>\n" if $::debug;
  return $heramgr;
} #end sub get_socket_conn


#---------------------------------------------------------------------------
sub close_channel {
  #my ($heramgr,$id,$key,$lcount,$rcount,$controlid,$password) = @_;
  my ($heramgr,$id,$key,$controlid,$password) = @_;

  print $heramgr "CloseTrustedChannel $id $key\n";
  print "sent <b>CloseTrustedChannel $id $key</b><br>\n" if $::debug;
  my $ans = <$heramgr>;
  print "got <b>$ans</b><br>\n" if $::debug;
  unless ($ans =~ /DONE/) {
     mailalert("CloseTrustedChannel $id $key failed: $ans");           
  }
  close($heramgr);
  print "Closed trusted channel<br>\n" if $::debug;

} # end sub close_channel

#---------------------------------------------------------------------------
sub mailalert {

  # send alert to me and Pan
  #my $addresses = 'cwh@lheamail.gsfc.nasa.gov,chai@lheamail.gsfc.nasa.gov';
  my $addresses = 'lmm@milkyway.gsfc.nasa.gov';

  $ENV{PATH} = "/usr/lib/";
  open(MAIL,"|/usr/lib/sendmail -oi -t");

    print MAIL <<EOT;
To: $addresses
From: Browse-Hera-interface
Reply-To: $addresses
Subject: HeraBrowser error

@_
EOT
    close(MAIL);
}

#---------------------------------------------------------------------------
#--- Display Hera login form
sub printform {
   my ($self, $controlid,$password,$newuser) = @_;

   my $in=$self->Input();
   

   Browse::Decor::Header("Save To Hera",0,0,0,"dprodtable" );
   #print "<p><b>Data product files were found ".
      #"matching your criteria.\n<br>";

   print "<form name='savetohera' method=POST " .
    "action='/$::cgiurl/w3hdprods.pl'>\n".
    "In order to copy these data to the Hera user area, a login for Hera is ".
    "required.</b></p>\n".
    "<p>(If you've never used Hera before, enter the username ".
    "and password you'd like to use, and check the 'new user' box.): </p>\n".
    "<p><table><tr><td><label for='controlid'>Hera user name: </label>" .
    "<input type='text' name='controlid' id='controlid' value='" .
    $self->{controlid} . "'></td><td></td></tr>\n".
    "<tr><td width=\"35%\"><label for='password'>Hera password: </label>" .
    "<input type='password' name='password' id='password' value='" .
    $self->{password} . "'>" .
    "</td><td>New Users: Please use only alphanumeric characters " .
    "(A-Z,a-z,0-9) and special characters .  _  - (period, underscore," .
    " dash)</td></tr></table> \n".
    "<label for='newuser'>Check this box if this a new user name: </label>" .
    "<input type='checkbox' name='newuser' id='newuser' " .
    $self->{newuser} . "></p>\n".
    "<p><input type='submit' name='Save to Hera' value='Save to Hera'>\n".
    " <a target='helpwin' " .
    "href='$::heasarc/hera'>" .
    "Intro to Hera</a>\n";

   foreach my $key (keys %$in) {
      if ($key =~ /^RetrieveSet|^Target|^File|^offsetTarget/) {
          foreach my $set (split("\0",$$in{$key})) {
            print "set $key  ==>  $set<br>" if $::debug;
             print "<input type='hidden' name='$key' value='" . $set . "'>\n";
          }
       } else {
            
            print "$key  ==>  $$in{$key}<br>" if $::debug;
           print "<input type='hidden' name=\"$key\" value='".
        	$in->{$key} ."'>\n" unless $key =~ /controlid|password/;
       }
   }
   print "</form></b></p>\n";

}

#---------------------------------------------------------------------------
#--- create a list of files in a directory
#sub descendtree {

    #my ($url,$filelist) = @_;
    #print "descending $url<br>\n" if $::debug;
     #my $dir;
    ##--- replace system name with local directory path
     #$url=~ /ftp\:\/\/(heasarc|heasarcdev|legacy|olegacy).gsfc.nasa.gov\/(.*)/;
     #$dir = "$2";
     #$dir = $url if $dir !~ /\w/;
     #$dir .= "/" if $dir !~ /\/+$/; 
     #$dir = "/FTP/". $dir unless $dir =~/^\/FTP\//;
#
    #if (opendir(DIR, "$dir")) {
        #my @files = readdir(DIR);
        #closedir(DIR);
	#my $file;
	#foreach $file (@files) {
#
	    #next if $file =~ /^\./; # ignore hidden files
#
	    #if (-d "$dir$file") {
               #print "$dir$file is dir so descend again<br>\n" if $::debug;
	        #$filelist = descendtree("$dir$file/",$filelist);
	    #} else { 
		#push(@$filelist, "$dir$file");
                #print "pushing $dir$file<br>\n" if $::debug;
	    #}
	#}
    #} else {
         #Browse::Utils::Error("Error opening data products directory");
    #}
    #return $filelist;
#} 	# end sub descendtree

sub quit {
  my ($self, $msg) = @_;

  $self->printform(); 
  print "<p><font color=red>$msg</font></p>\n";
  print "quit<br>\n" if $::debug;
  Browse::Decor::EmailFooter();
  Browse::Decor::EndTags();

  exit;
}
   
sub noProductsMessage {
  my ($self) = @_;
    my $msg = 
        "The data associated with the selected row " .
	" are not in the archive due to one of the following reasons:<ul>" .
   	"<li> the observation has not taken place</li>" .
   	"<li> data have not been processed</li>" .
   	"<li> data are still proprietary </li></ul><p>" .
        "Please consult the table documentation for information on " .
	"data availability.";
        Browse::Utils::Error("$msg");

}

sub pageFooter {
#exit;
}


1;
#$Log: ProdHera.pm,v $
#Revision 1.19  2008/01/30 15:23:41  lmm
#Uses row target offset value for SearchRa and SearchDec instead of
#  user-entered source so correct coordinates are used in case of multiple
#  sources. If no offset SearchRa and SearchDec are null.
#
#Revision 1.18  2007/10/09 14:19:06  lmm
#Now calling MakeConeQuery() as a Parameter class method.
#
#Revision 1.17  2006/11/06 21:32:19  lmm
#Fixed bug that was introduced when new "No Products" message method
#  was created.
#
#Revision 1.16  2006/10/24 19:00:36  lmm
#Moved "no products" error message to class method noProductsMessage().
#
#Revision 1.15  2006/06/23 17:11:00  lmm
#Improved Hera instructions on Save To Hera web page
#Added check for offset target (functionalty not complete)
#
#Revision 1.14  2006/03/30 20:02:31  lmm
#Fixed omission of SearchRA, SearchDec and SearchRadius for Save To Hera
#  transfer
#
#Revision 1.13  2006/03/07 21:07:59  lmm
#Corrected file processing for the Save to Hera option from the Preview and
#  Retrieve page.
#
#Revision 1.12  2005/11/21 20:07:11  lmm
#Moved "No data products" message so that it applies to all tables
#  instead of one.
#
#Revision 1.11  2005/10/31 17:20:08  lmm
#Changed port number for Hera server (per Pan Chai)
#
#Revision 1.10  2005/09/19 15:33:42  lmm
#File filter check now applied to Save to Hera option (but not to
#  Anonymous Hera).
#Subroutine descendtree() was moved to ProdMain as a class method.
#
#Revision 1.9  2005/08/16 20:47:55  lmm
#Fixed error in code that adds hidden elements to Save to Hera form
#
#Revision 1.8  2005/08/16 17:00:20  lmm
#Minor changes to log entry
#
#Revision 1.7  2005/08/16 16:22:30  lmm
#Added more information to Anonymous Hera entry in log file.  Entries now
#  go to separate log file instead of tar log.
#
#Revision 1.6  2005/08/04 21:27:42  lmm
#Fixed bug that was causing an infinite loop in descendtree()
#
#Revision 1.5  2005/07/28 18:29:33  lmm
#Corrected link to Hera web page
#
#Revision 1.4  2005/07/12 16:21:57  lmm
#Added entry in tar log file for anonymous hera clicks
#
#Revision 1.3  2005/07/05 20:37:18  lmm
#Added better checking for illegal characters in login ID/password
#  Hera file that has been saved in /tmp.
#
#Revision 1.1  2005/06/24 16:23:44  lmm
#Initial checkin
#
