# $Id: RowDisplay.pm,v 1.28 2008/12/15 19:21:35 lmm Exp $
# $Log: RowDisplay.pm,v $
# Revision 1.28  2008/12/15 19:21:35  lmm
# Made changes to accommodate multiple coordinate pairs in heasarc tables.
#   These new coordinates are traeted the same as standard ra and dec.
#
# Revision 1.27  2007/08/13 15:40:20  lmm
# Line breaks <br> for long strings are now added after HTML encoding
#
# Revision 1.26  2007/03/28 15:30:27  lmm
# Added calls to HTML::Encode() where table column value may contain < or >
#   or other symbol that should not be interpreted as HTML.
#
# Revision 1.25  2006/06/23 18:20:21  lmm
# Added changes to propogate precision information to TableFormat.pm
#   (code provided by T. McGlynn)
#
# Revision 1.24  2005/12/20 21:19:24  lmm
# Added title tag to parameter name link to improve tool tips
#
# Revision 1.23  2005/02/15 15:57:51  lmm
# Skip non default display set parameters (_link_ prefix) that are used
#    for table and bibliography links.
#
# Revision 1.22  2003/09/15 20:42:30  lmm
# Added additional check for field formats with colon (:) separator.  if
#    value after colon is non-numeric (e.g. html) it is not added to
#    reformat array
#
# Revision 1.21  2003/08/29 16:46:07  lmm
# Defined text and style for page header and removed HEASARC footer
#
# Revision 1.20  2003/04/28 02:33:32  cwh
# Added new table style.
#
# Revision 1.19  2002/12/16 18:34:26  cwh
# No longer decoding info or target.
# Print 'no rows found' if no rows, and return.
#
# Revision 1.18  2002/05/26 02:27:22  cwh
# Assignment of names array was happening too late - caused the header line to
# be empty on all but the first single line for each table.
#
# Revision 1.17  2002/03/25 18:17:21  cwh
# Fixed bug that caused coords not to be transformed correctly for non-J2000.
#
# Revision 1.16  2002/03/17 19:03:57  cwh
# Added ra/dec -> lii/bii label conversion for galactic coords
# Don't quit with an error, just return undef
# Pass accuracy to tableformat.
#
# Revision 1.15  2002/02/14 17:55:10  cwh
# HTML cleanup: changed <font> element to <span> element.
#
# Revision 1.14  2001/12/05 23:45:15  cwh
# Now allows unique key to be specified as keyname=keyvalue.
# Stripping local_origin before printing table name before results.
# Uses keystring instead of getting query_key or unique_key.
#
# Revision 1.13  2001/11/07 23:34:19  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.12  2001/09/15 14:06:35  cwh
# Do the conversion of spaces to nbsp's and !! to links to help in the
#   single rows.
#
# Revision 1.11  2001/08/28 19:30:27  cwh
# Add link to help for ra,dec error flags.
#
# Revision 1.10  2001/07/01 01:59:49  cwh
# Forgot to add to log last commit: had to recreate @truncs to send along to
# TableFormat::setTransform, or else the formatting of the fields got lost on
# the single-row display of all fields.
#
# Revision 1.9  2001/07/01 01:58:13  cwh
# Changed cgi-bin to viz-bin for VizieR call.
# When displaying units, change underscores to spaces and 'micro' to '&micro;'.
# When deciding not to display units, match /^mjd/ not == "mjd".
# When deciding not to display units of degrees, only withhold them if param
# name is $table_info{right_ascension} or $table_info{declination}.
#
# Revision 1.8  2001/06/20 22:57:34  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.7  2001/06/18 19:08:51  cwh
# Tom added check for query_key for Vizier tables, since not always 'recno'.
#
# Revision 1.6  2001/06/14 21:29:08  tam
# Use '==' for index searches to get exact character matches.
#
# Revision 1.5  2001/06/14 15:39:44  tam
# Handle cases where a cross-correlation may
# not show multiple tables.
#
# Revision 1.4  2001/06/12 16:13:47  tam
# Handle abstracts (or other long text fields) in row display
# Don't go to VizieR for cross-correlation row displays
#
# Revision 1.3  2001/05/22 17:52:04  cwh
# Added Id and Log to files without them.
#

use strict;
use Browse::QueryRequest;
use HTML;

package Browse::RowDisplay;

# Handle the case where the user has clicked on
# a right arrow.
sub displaySingle {

    my ($key, $in) = @_;
    $key =~ s/\.[xy]$//;
    my $y = $$in{$key.".y"};
    
    my ($dumm, $info) = split('\|', $key, 2);
    #$info = Browse::CGI::Decode($info);
    my ($table,$keyval) = split('\|\|\|', $info, 2);
    $keyval =~ s/\|\|$//;  # trim off ending ||

    if ($y > 7  && $table =~ /\//  && $table !~ / x /i) {
        if ($table =~ / x / and $keyval =~ /\|\|/) {
            ($table) = $table=~ /\s*(\S+)/;
            ($keyval) = $keyval=~ /\|\|([^|]*)/;
        }
 	$keyval =~ s/^\s*//;
        #my $key = Browse::DPL::getKey($table);
        ###need to fix?
	my %tableinfo = Browse::HDB::TableInfo($table);
	my $key       = $tableinfo{"query_key"};
	
	my $url = "$::vizier/viz-bin/VizieR-5?-source=$table&$key=$keyval";
	print "Location:$url\n\n";
	exit();
    }
    my ($tables, $keys) = parseInfo($table, $keyval);
    displayRows($tables, $keys, $in);
    exit();

}

# Parse a single row's worth of data -- this
# may correspond to a cross-correlation.
sub parseInfo {

    my ($table, $keyval) = @_;
    
    # Extra handling to process cross correlations.
    my (@tables)  = split(" +x +", $table);
    my (@xkeys)   = split('\|\|', $keyval);
    if (length($xkeys[$#xkeys]) <= 0 or $xkeys[$#xkeys] =~ /^\|$/) {
        pop(@xkeys);
    }
    foreach (@tables) {
	$_ =~ s/ .*//;
    }
    
    # Cross-correlations may not have information
    # on all tables.
    
    while (length($xkeys[$#xkeys]) == 0) {
        pop(@xkeys);
    }
    while ($#xkeys < $#tables) {
        pop(@tables);
    }
#    if ($#xkeys > $#tables) {
#        splice(@tables, $#xkeys);
#    }
    
    return (\@tables, \@xkeys);
}

# Process a request where a user has asked to
# display many rows worth of data by clicking on the check boxes.
sub displayMulti {

    my ($target, $in) = @_;
 
    print "RowDisplay::displayMulti target=$target<br>" if $::debug;
    
    my @targets = split("\0", $target);
    my @tables;
    my @keyf;
    my $t;
    
    foreach $t(@targets) {
        #$t = Browse::CGI::Decode($t);
	my ($tab, $key) = split('\|\|\|', $t, 2);
	my ($nt, $nk) = parseInfo($tab, $key);
	push(@tables, @$nt);
	push(@keyf, @$nk);
    }
    displayRows(\@tables, \@keyf, $in);
    exit;
}
    
# Query the database and show all the fields for the specified
# set of table/key pairs.
sub displayRows{

    # This subroutine displays all database parameters of selected rows
    my($tables, $keys, $in) = @_;
    
    #Browse::Decor::Header("results");
    # add meaningful title and color the title background on Display page
    my($class) = "datacell1";
    Browse::Decor::Header($$in{"popupFrom"},0,0,0,$class);

    my $coordsys = $$in{"Coordinates"};
    my $equinox  = $$in{"Equinox"};
    print "in RowDisplay::displayRows: coordsys=$coordsys<br>\n" if $::debug;
    if (length($coordsys) <= 0) {
        $coordsys="Equatorial";
    }
    if (length($equinox) <= 0) {
        $equinox=2000;
    }
    

    # Get the metadata info we will need.
    my %alltab;
    
    my $table;
    foreach $table (@$tables) {
        $alltab{$table} = 1;
    }
    
    my $allStr = join(",", keys(%alltab));
    
    # TableInfo handles the single table case specially.
    my ($tableinfo,$paraminfo);
    if ($allStr !~ /\,/) {
	$tableinfo = {};
	$$tableinfo{$allStr} = {Browse::HDB::TableInfo($allStr)};
    } else {
        $tableinfo = Browse::HDB::TableInfo($allStr);
    }
    
    my $lastTable;
    my $divider;
    my ($row,$names,$meta);
    
    my $i;
    my $request;
    my $transInfo;
    for($i=0; $i <= $#$tables; $i += 1) {
    
        $table  = $$tables[$i];
	my $keystr = $$keys[$i];
	
	if ($table ne $lastTable) {
	    $transInfo = undef;
	}
	
	my $tinfo = $$tableinfo{$table};
	
	($row, $names, $transInfo,$meta) = getSingleRow($table, $keystr, 
		"all", $tinfo, $transInfo, $coordsys, $equinox);
	
        unless (ref $row) {
           print "No rows found.<br>\n";
           return;
        } 
	print $divider;
	$divider = "<hr>\n";
        (my $ptable = $table) =~ s/$::local_origin//ig;
	
        print "<span class=\"large\">Catalog: </span>".
                "<a target='helpwin' href='$$tinfo{table_document_url}'>".
                "$$tinfo{table_description} ($ptable)</a><br><br>\n";
        print "<table class='othertable' " .
		"title='All parameters for selected rows'>\n<thead>" .
		"<tr class='othertable'>\n",
               "<th scope='col' align='left' class='othertable'>" .
		"<b>Parameter Name</b></th> ",
               "<th scope='col' align='left' class='othertable'>" .
		"<b>Parameter Value</b></th> ",
               "<th scope='col' align='left' class='othertable'>" .
		"<b>Unit</b></th> ",
               "<th scope='col' align='left' class='othertable'>" .
		"<b>Description</b></th>\n</tr></thead><tbody>\n";
	
        my $ra = $$tinfo{"right_ascension"};
        $ra =~ s/^\@//;
        my $dec = $$tinfo{"declination"};
        $dec =~ s/^\@//;

        #--- find names of other ra/dec columns
        my ($oras, $odecs) = Browse::Display::otherRaDecs($tinfo);

	my $pinfo = $$meta{"paraminfo"};
        my @line = @$row;
	my $name;
        foreach $name (@$names) {
	
            my $value = shift(@line);
            $value=~ s/!!/<a href="$::w3bhelp#coorderror">!!<\/a>/g;
            #-- If format is not HTML encode so value is not interpreted as tag
            if ( $$pinfo{$name}{'parameter_format'} !~ /^char[^:]+:html/) {
              $value = HTML::Encode($value);
              $value =~ s/\s+/\&nbsp\;/g; # don't want to break the line
            }
	    $value =~ s/\0/\<br\>/g;

            #--- adjust names for Ras and Decs
            if (scalar(@$oras) && scalar(@$odecs)) {
               my @check_other_ras = grep /^$_$/, @$oras;
               my @check_other_decs = grep /^$_$/, @$odecs;
               if ($coordsys =~ /galactic/i ) {
                  if (scalar(@check_other_ras)) {
                     $_ =~ s/ra$/lii/;
                  }
                  if (scalar(@check_other_decs)) {
                     $_ =~ s/dec$/bii/;
                  }
               } elsif ($coordsys =~ /ecliptic/i) {
                  if (scalar(@check_other_ras)) {
                     $_ =~ s/ra$/elon/;
                  }
                  if (scalar(@check_other_decs)) {
                     $_ =~ s/dec$/elat/;
                  }
               }
            }
            if ($name eq $ra) {
               if ($coordsys =~ /galactic/i) {
                  $name = "_Lii";
               } elsif ($coordsys =~ /ecliptic/i) {
                  $name = "_Elon";
               }
            } elsif ($name eq $dec ) {
               if ($coordsys =~ /galactic/i) {
                  $name = "_Bii";
               } elsif ($coordsys =~ /ecliptic/i) {
                  $name = "_Elat";
               }
            }
	    
            #--- Do not display all columns
	    if ($name =~ /^w3_|^_link_/i) {
	        next;
	    }
            my $unit = $$pinfo{$name}{'parameter_unit'};
	    undef($unit) if $unit =~ /\bmjd/i;
            
            #--- Remove original unit value for Ras & Decs
	    if ($unit eq "degree" and 
	       (lc($name) eq lc($$tinfo{right_ascension}) or 
                lc($name) eq lc($$tinfo{declination}) or 
                scalar(grep /^$name$/, @$oras)  or 
                scalar(grep /^$name$/, @$odecs) )) { 

	        undef($unit);
	    }
            $unit = "&nbsp;" unless $unit =~ /\S/;
            $unit =~ s/_/ /g;
            $unit =~ s/micro/&micro;/g;
            my $desc = $$pinfo{$name}{'parameter_description'} || "&nbsp";
            print "<tr class='othertable'>\n";
            print "<td class='othertable'>";
            print "<a target='helpwin' href='$$tinfo{table_document_url}",
                  "#$name' title='link to parameter information'> $name </a></td>";
            print "<td class='othertable'>$value</td>";
            print "<td class='othertable'>$unit</td>";
            print "<td class='othertable'>$desc</td></tr>";
        }
        print "</tbody></table>\n";

    }
    print "<hr>" ;
    Browse::Decor::EmailFooter();
    Browse::Decor::EndTags();

}

# Do the database query for a single row.
sub getSingleRow {

    my ($table, $keystr, $fields, $tinfo, $transInfo, $coordsys, $equinox) = @_;

    $keystr =~ s/^ *//;
    $keystr =~ s/ *$//;
	
    my $request = Browse::QueryRequest->new($table);
    print "RowDisplay::keystr=$keystr<br>\n" if $::debug;
  
    my $fieldStr=''; 
    unless ($keystr=~/=/) { 
       $fieldStr = $$tinfo{"unique_key"};
       if (!$fieldStr) {
   	   Browse::Utils::Error("Unable to find unique key for $table");
	   exit();
       }
    }

    my (@fields,@vals);
    if ($fieldStr=~ /\w/) {
       @fields = split(",", $fieldStr);
       @vals = split('\|', $keystr);
    } else {
       foreach (split('\|', $keystr)) {
          my ($field,$val) = split (/\s*=\s*/,$_);
    print "RowDisplay::pushing $field $val <br>\n" if $::debug;
          push (@fields,$field);
          push (@vals,$val);
       }
    }
 
    print "RowDisplay::fields=@fields<br>\n" if $::debug;
    print "RowDisplay::vals=@vals<br>\n" if $::debug;
    
	
    my @quals;
    my ($i);
    for ($i=0; $i <= $#fields; $i += 1) {
        print "RowDisplay:: $fields[$i] $vals[$i] <br>\n" if $::debug;
	push(@quals, "$fields[$i]|==|$vals[$i]") if defined $vals[$i];
    }
	
    $request->qclose();
    
	
    $request->setTable($table);
    $request->setFields($fields) if $fields;
    $request->setQuals(\@quals);

    $request->initiate();
	
    my $meta = $request->getResult();
    my $data = $request->getResult();
    
    if (!$meta || !$data ||
	$$meta{"type"} ne "meta" || $$data{"type"} ne "data") {
        # Browse::Utils::Error("Error querying $table for single row");
	# exit();
        return undef;
    }
    
    my $names = $$data{"names"};
    my $types = $$data{"types"};
    my $precisions = $$data{"precisions"};   #long strings lmm
    my $accuracy;
    if (!$transInfo) {
	my $radecs;

	my $raf  = $$tinfo{"right_ascension"};
	my $decf = $$tinfo{"declination"};
	if (defined($raf) && defined($decf)) {
	    if (substr($raf, 0, 1) eq '@') {
		$raf = substr($raf,1);
	    }
	    if (substr($decf, 0, 1) eq '@') {
		$decf = substr($decf,1);
	    }
	    $radecs = ["$raf,$decf"];
	}
        my ($oras, $odecs) = Browse::Display::otherRaDecs($tinfo);
        if (scalar(@$oras) == scalar(@$odecs)) {
            for (my $i=0; $i < scalar(@$oras); $i++) {
              push (@$radecs, "$$oras[$i],$$odecs[$i]"); 
           }
        }
	    
	my $pinfo = $$meta{"paraminfo"};
        ($accuracy) = $$pinfo{$raf}{"parameter_format"} =~ /:\.(\d+)f/;
        print "RowDisplay::accuracy=$accuracy<br>\n" if $::debug;
	    
	my $mjds;
	my $p;
	    
	foreach $p(keys(%$pinfo)) {
	    if (substr($p, 0, 1) ne '#') {
		if ($$pinfo{$p}{"parameter_unit"} =~ /mjd/i) {
		    push(@$mjds, $p);
		}
	    }
	}
	    
        # this is reproduced from within Display:setTransform - 
        #should be moved out (HDB?)
        my $name;
        my @truncs;

        foreach $name (@$names) {

           my ($form) = $$pinfo{$name}{"parameter_format"};
           print "RowDisplay::name=$name form=$form<br>\n" if $::debug;
           my ($bas, $out) = split('\:', $form);
	   if (defined($out) and $out =~ /\d/) {
               push(@truncs, "$name,\%".$out);
           }
        }
	    
        #long strings tam
        ($transInfo) = Browse::TableFormat::setTransform($names, $types, 
	   $precisions, $radecs, $mjds, \@truncs, $coordsys,$equinox,
	   "ISO","Sexagesimal",$accuracy);

    }
    my $row = $$data{"data"};
    Browse::TableFormat::transformLine($row, $transInfo);
    my ($racol, $deccol) = ($$transInfo{"racols"},$$transInfo{"deccols"});
   
    my $raf  = $$tinfo{"right_ascension"};
    my $decf = $$tinfo{"declination"};
    my ($oras, $odecs) = Browse::Display::otherRaDecs($tinfo);
    foreach my $name (@$names) {
       $name = Browse::Display::renameRaDecs($name, $raf, $decf, $oras, $odecs, 
	$coordsys);
   } 

    # We defer the transformat of spaces into NBSPs to here
    # because this is the point that we know that
    # the data is going to be presented in HTML.
    my $col;
    foreach $col(@$racol,@$deccol) {
        #$$row[$col] =~ s/ /\&nbsp\;/g;
        $$row[$col] =~ s/!!/<a href="$::w3bhelp#coorderror">!!<\/a>/g;
    }

    return ($row, $names, $transInfo,$meta);
}

#sub otherRaDecs {
   #my($tinfo) = @_;
   ##--- find names of other ra/dec columns
   #my $other_radecs = $$tinfo{"other_ra_dec"};
   #my @oras;
   #my @odecs;
   #if ($other_radecs) {
      #my @other_list = split(/\|\|/, $other_radecs); 
      #foreach (@other_list) {
         #my ($ora, $odec) = split(/\s*,\s*/); 
         #if (defined($ora) && defined($odec)) {
            #$odec =~ s/^\@//;
            #$ora =~ s/^\@//;
            #push(@oras, $ora);
            #push(@odecs, $odec);
         #}
      #}
   #}
   #return \@oras, \@odecs;
#}
1;
