#$Id: SearchBasedOn.pm,v 1.5 2007/06/04 15:09:09 lmm Exp $

#----------------------------------------------------------------
#--- methods to display SkyView images and "Search Based on" box
#----------------------------------------------------------------

use strict;

package Browse::SearchBasedOn; 
use Browse::SkyView;
use Browse::TableFormat;

sub new {
   my ($self,$mode) = @_;
   my $obj = {};

   if ($mode) {
       $mode =~ /([\-\w]*)/; $mode=$1;   # untaint $mode

       #dynamically load display module
       $mode = "Browse::$mode";
       eval "package Browse::_firesafe; require $mode";
       bless $obj, $mode;
   } else {
       bless $obj;
   }
   return $obj;
}

sub MaxRows {
    my ($self, $maxrows) = @_;
    $$self{"maxrows"} = $maxrows if defined($maxrows);
    return $$self{"maxrows"};
}

sub Positions {
    my ($self, $positions) = @_;
    $$self{"positions"} = $positions if $positions;

    if ($positions) {
       my ($coord1,$coord2,$rad,$name,$resolver) = split('\|', @$positions[0]);
       $self->SkyViewCoords($coord1,$coord2);
    }
    return $$self{"positions"};
}

sub Qualifiers {
    my ($self, $qualifiers) = @_;
    $$self{"qualifiers"} = $qualifiers if $qualifiers;
    return $$self{"qualifiers"};
}

sub Sorts {
    my ($self, $sorts) = @_;
    $$self{"sorts"} = $sorts if $sorts;
    return $$self{"sorts"};
}

sub Directions {
    my ($self, $dirs) = @_;
    $$self{"dirs"} = $dirs if $dirs;
    return $$self{"dirs"};
}

sub DisplayType {
    my ($self, $displaytype) = @_;
    $$self{"displaytype"} = $displaytype if $displaytype;
    return $$self{"displaytype"};
}

sub Resmsg {
    my ($self, $resmsg) = @_;
    $$self{"resmsg"} = $resmsg if $resmsg;
    return $$self{"resmsg"};
}

sub SkyViewCoords {
    my ($self, $x, $y) = @_;
    $$self{"skyviewX"} = $x if $x;
    $$self{"skyviewY"} = $y if $y;
    return ($$self{"skyviewX"},$$self{"skyviewY"});
}

sub SavedQuery {
    my ($self, $savedquery) = @_;
    $$self{"savedquery"} = $savedquery if $savedquery;
    return ($$self{"savedquery"});
}

sub CoordSys {
    my ($self, $coordsys, $equinox) = @_;
    $$self{"coordsys"} = $coordsys if $coordsys;
    $$self{"equinox"} = $equinox if $equinox;
    return ($$self{"coordsys"},$$self{"equinox"});
}

sub displayImages {
    my ($self)=@_;

    # Display DSS and RASS images (if 1 object queried) 
    my $string = "";
    $string .= " <table>\n<tr>";

    # If one object queried display its SkyView image
       # Size in pixels of Skyview image 
       my $xsize=100;
       my $ysize=100;
       my ($skyviewCoord1,$skyviewCoord2) = $self->SkyViewCoords();
       my $coord_sys = CoordSys();
       my $equinox="2000";
       $string .= " <td align='center'><font size='-1'>\n" .
	    "Images generated by <a href='http://skyview.gsfc.nasa.gov'>" .
	    "<i>SkyView</i></a><br>Click on image to see full <i>SkyView</i>" .
	    " image</font></td>\n</tr><tr>\n";

       my($skyviewDssImage)=Browse::SkyView::prepareImage($skyviewCoord1,
	    $skyviewCoord2,'Digitized+Sky+Survey',$coord_sys,$equinox,
	    $xsize,$ysize);
       my($skyviewRassImage)=Browse::SkyView::prepareImage($skyviewCoord1,
	    $skyviewCoord2,'RASS-Cnt+Broad',$coord_sys,$equinox,$xsize,$ysize);
       my($skyviewDssUrl)=Browse::SkyView::prepareLink($skyviewCoord1,
	    $skyviewCoord2,'Digitized+Sky+Survey',$coord_sys,$equinox);
       my($skyviewRassUrl)=Browse::SkyView::prepareLink($skyviewCoord1,
	    $skyviewCoord2,'RASS-Cnt+Broad',$coord_sys,$equinox);

       $string .= " <td><table>\n<tr><td align='center'>\n" .
	"<a href='$skyviewDssUrl' target='skyview'>" .
	"<img src='$skyviewDssImage' border='1' height='$xsize' " .
	"width='$ysize'></a><br><font size='-1'>" .
	"<a href='http://www-gsss.stsci.edu/SkySurveys/DSS%20Description.htm
'>DSS</a>" .
	" Optical image, 2.83\'</font></td></tr>\n" .
	"<tr><td align='center'><a href='$skyviewRassUrl' target='skyview'>" .
	"<img src='$skyviewRassImage' border='1' height='$xsize' " .
	"width='$ysize'></a><br><font size='-1'>" .
	"<a href='http://www.xray.mpe.mpg.de/cgi-bin/rosat/rosat-survey'>" .
	"RASS</a> X-ray image, 75.0\'</font></td>\n</tr>";

       $string .="<tr>\n<td align='center'><font size='-1'>\n" .
	"Images centered on requested position</td></tr>";

       $string .="</table>\n";
       $string .="</td></tr></table>\n";

    return $string;
}

#--- Display search criteria and qualifiers
sub displaySearch {
    my ($self)=@_;

    my $string="";
    my $maxObjectListLimit=10;  # limit the number of targets to list
    my $maxrows=$self->MaxRows();
    my ($coord_sys,$equinox) = $self->CoordSys();

    #These are wrong if there are more than one sort param - fix
    my ($sortinfo,$sort);
    my $sorts=$self->Sorts();
    foreach $sort (@$sorts) {
        $sortinfo .= $sort;
    }

    my ($dirinfo,$dir);
    my $dirs=$self->Directions();
    foreach $dir (@$dirs) {
        $dirinfo .= $dir;
    }

    my $f = "<span class='small'>";
    my $fe = "</span>";

    my $radius;
    my($num, $obj,$pos,$object_info);
    my $hidden_object_info="";
    my $pos_count;
    my $positions=$self->Positions();

    if (defined(@$positions)) {$pos_count=@$positions;}   # get target count

    foreach $pos (@$positions){
       my ($coord1,$coord2,$rad,$name,$resolver) = split('\|',$pos);
       $radius = $rad;


       my $format = "decimal" if $coord_sys =~ /gal/i;
       ($coord1,$coord2)  = Browse::TableFormat::fromJ2000($coord1,$coord2,
		$coord_sys,$equinox,$format);
#
       # save coords without units for SkyView query
       #if ($pos_count == 1) {$self->SkyViewCoords($coord1, $coord2);}

       $coord1 .= "&#176;" if $format =~ /decimal/;
       $coord2 .= "&#176;" if $format =~ /decimal/;
       $coord1 =~ s/\s+/&nbsp;/g;
       $coord2 =~ s/\s+/&nbsp;/g;

       $object_info .= " </tr> <tr><td>&nbsp;</td>\n" if $num;

       #--- HTML-encode  non-words 
       $name =~ s/\W/"&#".ord($&).";"/ge; 
       my $resv="<input type='text' name='Entry' id='Entry' value='$name'>";
       if ($name =~ /[^\d\.\+\-\s\,]/) {
            $resv .= "<br> resolved by $resolver to " .
		"[&nbsp;$coord1,&nbsp;$coord2&nbsp;]";
       }
       $obj = "<td>$f $resv $fe</td>";
       $object_info .= "    ".$obj . "\n";

       #  Put together hidden input in case there are too many targets
       # to list for requery
          my $hiddenresv=
		"<input type='hidden' name='Entry' id='Entry' value='$name'>";
          $hidden_object_info .= "    ".$hiddenresv . "\n";

       $num++;
    }
       $string .= " <table class='datatable'>" .
	   "<tr><td colspan='2' align='left'><b>Search was based on: </b>" .
	   "</td></tr>\n";

    my $qual_info=$self->doQualifiers();

    my($radiusval) =$radius;  # radius is either "Default" or in degrees, 
                              # convert to arc minutes for requery entry 
                              # box lmm

    if ($radius =~ /\d/ ) {
        $radiusval *= 60;  # radius is in degrees so convert
        my $radunit = "&deg;";
        if ($radius < 1) {
           $radius *= 60;
           $radunit = "\'";
        }

        if ($radius < 1) {
           $radius *= 60;
           $radunit = "\"";
        }
        $radius = sprintf("%.2f", $radius) . " $radunit";
    }
#
    # object/coordinates
    if ($object_info) {
       # If target file has been uploaded there may be many targets.
       # do not list targets on page if there are too many
       if ($pos_count > $maxObjectListLimit) { 
          $string .= "<tr><td>$f Object/Coordinates: $fe</td><td>" .
		"There are too many targets ($pos_count) to list" .
		"$hidden_object_info</td> </tr>";
       } else { 
          $string .= "<tr><td>$f <label for='Entry'>" .
		"Object/Coordinates:</label> $fe</td>$object_info  </tr>\n";
       }
    }
   
    my $resmsg=$self->Resmsg();
    if ($resmsg) {
       $string .= "<tr><td colspan='2'><span class='note'>$resmsg</span></td></tr>\n";
    }

    # coordinate system
    $string .= "<tr><td>$f Coord. System: $fe</td><td>$f $coord_sys";
    if ($coord_sys !~ /Galactic/i) {
         $string .= ", equinox  $equinox";
    }
    #if ($csError =~ /\S/) {
       #$string .= "<br><span class='note'>$csError</span>";
    #}
    $string .= "$fe </td></tr>\n";

    # number of table rows to display
    $maxrows = "(no limit)" unless $maxrows;
    if ($maxrows) {
       $string .= "<tr><td>$f <label for='maxrows'>Maximum Rows:</label>" .
	   " $fe</td><td>$f " .
	   "<input type='text' name='maxrows' id='maxrows' value='$maxrows'>" .
	   " $fe</td></tr>\n";
    }

    if ($qual_info) {
       $string .= "<tr><td>$f Parameter Query: $fe</td><td>$f $qual_info $fe</td></tr>\n";
    }

    # search radius
    if ($object_info) {
       #$string .= "<tr><td>$f Search Radius: $fe</td><td>$f $radius $fe</td></tr>\n";
       # Display as editable text, use arc minutes 
       $string .= "<tr><td>$f <label for='radius'>Search Radius:</label> " .
	  "$fe</td><td>$f <input type='text' name='Radius' id='radius' " .
	  "value='$radiusval'> $fe<input type='hidden' name='Radius_unit' " .
	  "value='arcmin'>$f arc minutes $fe</td> </tr>\n";    #lmm

    }

    $dirinfo = "" if $dirinfo =~ /down/ or !$dirinfo;
    $dirinfo = ", reverse order" if $dirinfo =~ /up/;
    $string .= "<tr><td>$f Sorted by: $fe</td><td>$f ${sortinfo}$dirinfo $fe</td></tr>\n" if $sortinfo;

    $string .= "<tr><td><input type='submit' name='requery' value='Reissue Query' disabled></td>";  #lmm serialize query
    

    #-- Save Query does not work when query has been reissued.  Check for 
    #--- existence of saved query variable
    $string .= "<td align='right'><input type='submit' name='files' " .
	#--#"value='Save Query To File' onClick=\"noNewWindowSubmit(this.form)\">" .
	"value='Save Query To File' onClick='noNewWindowSubmit(this.form)'>" .
	"</td>" unless !$self->SavedQuery();

    $string .="</tr>\n"; #lmm serialize query

    $string .="</table>\n";



    return $string;
}

sub doQualifiers {
    my ($self)=@_;

    my ($num2,$qual,$qual_info);
    my $last_par="";
    my $qualifiers = $self->Qualifiers();
    if ($qualifiers) {
    foreach $qual (sort @$qualifiers){
       my ($field,$op,$min,$max) = split('\|',$qual);
       # need unit info in order to know if we need to convert from MJD
       $min = DateTime::mjd2iso($min) if defined $min and 
	   $field =~ /metaparam_start_time|metaparam_end_time/;
       $max = DateTime::mjd2iso($max) if defined $max and 
	   $field =~ /metaparam_start_time|metaparam_end_time/;
       $field = "<i>$1</i>" if $field =~ /_metaparam_(.*)/;
       $qual_info .= " AND<br>" if $num2 and $last_par !~ /^$field$/;
       $qual_info .= " OR<br>" if $num2 and $last_par =~ /^$field$/;
       if ($op =~ /between/) {
          (my $not, $op) = $op =~ /(!?)(.*)$/;
          $not = "not" if $not;
          if (!$min or $min =~ /'\s?'/) {
             if (!$not) {
                 $qual_info .= "$field <= $max";
             } else {
                 $qual_info .= "$field > $max";
             }
          } elsif (!$max or $max =~ /'\s?'/) {
             if (!$not) {
                 $qual_info .= "$field >= $min";
             } else {
                 $qual_info .= "$field < $min";
             }
          } else {
             $qual_info .= "$field $not $op $min and $max";
          }
       } else {
          $qual_info .= "$field $op $min";
       }

       $qual_info  =~ s/\s+/&nbsp;/g;
       $num2++;
       $last_par = $field;

    }
    }
    return $qual_info;
}

1;

#$Log: SearchBasedOn.pm,v $
#Revision 1.5  2007/06/04 15:09:09  lmm
#Corrected link to DSS page under DSS thumbnail.
#
#Revision 1.4  2006/10/24 19:10:18  lmm
#Requery button is now disabled until page loads.
#
#Revision 1.3  2006/06/23 18:22:08  lmm
#Made some minor changes to quotation marks
#
#Revision 1.2  2005/12/16 15:18:32  lmm
#Cosmetic changes
#
#Revision 1.1  2005/02/23 18:49:50  lmm
#Initial checkin
#
