# $Id: Sort.pm,v 1.10 2005/12/16 15:20:20 lmm Exp $
# $Log: Sort.pm,v $
# Revision 1.10  2005/12/16 15:20:20  lmm
# Debug change.
#
# Revision 1.9  2002/09/13 20:51:46  cwh
# *** empty log message ***
#
# Revision 1.8  2002/04/10 22:05:13  cwh
# Simplified sorting of tables.
#
# Revision 1.7  2001/11/07 23:34:20  cwh
# Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
# namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
# front of every module call and package declaration.  Also any mod::new() had
# to be changed to mod->new().
#
# Revision 1.6  2001/08/10 21:32:07  cwh
# Fixed typo that prevented sorting tables by the table type.
#
# Revision 1.5  2001/05/17 02:54:54  tam
# Unused functions deleted -- should this package
# go somewhere else?
#
# Revision 1.4  2001/04/03 16:21:22  cwh
# Changed the sort arrow assigned to descending from 'up' to 'down'.
# Moved the sortTables subroutine here.
#
# Revision 1.3  2000/03/08 20:12:52  saima
# W3Browse to Browse name change.
#
# Revision 1.2  1999/01/07 20:53:07  saima
# Sorts from the results page are no longer redirects. Removed
#   MakeSortString(), GetPSortParams() deprecated routines. Modified
#   callSort() so that Cone search is called by default.
#
# Revision 1.1  1998/12/23 17:12:23  saima
# Redid the sorts so that when a user clicks on an up/down arrow,
#   a call is made to the appropriate Cone search or Parameter
#   search module. The query information is stored in the Results page.
#   This a much cleaner way of handling the sorts.
#
# Revision 1.2  1998/04/01 17:02:40  saima
# Fixed a minor bug which now sends mjd parameters to the parameter
#   search correctly.
#
# Revision 1.1  1998/03/10 21:48:05  saima
# Initial revision.
#
#=======================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=======================================================================#
# Module Name:    Sort.pm
# Module Author:  Saima Zobair, Raytheon STX
# Date:           February 19, 1998
# Project:        Browse : HEASARC
#-----------------------------------------------------------------------#
# Module Description:
#
# The routines in this module help call search routines to sort by 
# a given parameter name. 
# 
#=======================================================================#

use strict;
package Browse::Sort;

#------------------------------------------------------------------------
sub sortTables {

    my ($tables, $sort) = @_;
    $sort = "priority" if !defined($sort);

    my $desc = 0;    #--- descending order
    if ($sort =~ /^-/) {
        $desc = 1;
        $sort = substr($sort, 1);
    }

    if ($::debug) {
        print "sortTables: n= $#$tables, sort=$sort desc=$desc<br> \n";
        my $v = $$tables[0];
        print "         keys are: ",join(",", keys(%$v) ),"<br>\n";
        print "         values are: ",join(",", values(%$v) ),"<br>\n";
    }
    

    my @newtab;
    if ($sort =~ /radius|priority/) {  # numeric
        @newtab = sort { $$a{$sort} <=> $$b{$sort}} @$tables;
    } else {
        @newtab = sort { $$a{$sort} cmp $$b{$sort}} @$tables;
    }
    if ($::debug) {
        my $va = $newtab[0];
        print "         keys are: ",join(",", values(%$va) ),"<br>\n";
    }
    @newtab = reverse @newtab if $desc;

    return \@newtab;
}
#--------------------------------------------------------------------------
#########
#  EOF  #
#########
1;
