#$Id: VizierQuery.pm,v 1.33 2008/10/28 17:34:30 lmm Exp $
#$Log: VizierQuery.pm,v $
#Revision 1.33  2008/10/28 17:34:30  lmm
#Changes made to accommodate new metadata returned from Vizzier queries.
#
#Revision 1.32  2008/03/06 20:26:43  lmm
#Added check for slash (/) in Vizier name.  Browse bases all Vizier
#  table processing on detection of the slash.  (There is probably one
#  out of 350 Vizier tables that does not contain a slash.)
#
#Revision 1.31  2008/03/06 17:28:02  lmm
#Made changes to accommodate new XML format returned by Vizier.
#
#Revision 1.30  2007/11/20 15:11:22  lmm
#Added default sort for URL and first non-coordinate/non-offset
#  field s now used for parameter display key if a more standard
#  key field (like rec_no) does not exist.
#
#Revision 1.29  2006/10/24 20:51:30  lmm
#Now uses Vizier::convertName() to shorten long and clumsy table names.
#
#Revision 1.28  2006/06/23 18:49:37  lmm
#Added changes to propogate precision information to TableFormat.pm
#  (code provided by T. McGlynn)
#
#Revision 1.27  2005/12/16 15:48:29  lmm
#Made slight change to the URL sent to vizier involving the unique key.
#Offset column name changed from _r to _Search_Offset (per users' request)
#
#Revision 1.26  2003/08/29 16:50:30  lmm
#Changed default radius to 1 arc minute from 10
#
#Revision 1.25  2003/07/23 20:42:50  cwh
#Added offsetvar name.
#
#Revision 1.24  2002/09/05 18:16:59  cwh
#Added more error reporting.
#
#Revision 1.23  2002/03/17 19:23:38  cwh
#Kludge to print error when Vizier doens't allow param queries of big tables
#without a cone limitation.
#For some reason I was putting '|'s in urls.  Commented these out.
#Added 'r' to list of requested columns.
#
#Revision 1.22  2002/01/22 22:34:00  cwh
#Vizier XML output uses a line like:
#     <!-- RowName:  ${GSC2.2} -->
#to identify the unique key if the key is not the standard 'recno'.
#Added code to read this key and use it.
#
#Remove trailing '|' from a qualifier group - was causing a failure for
#  one catalog.
#
#Revision 1.21  2002/01/16 21:46:23  cwh
#Added debug stuff.
#
#Revision 1.20  2002/01/15 16:09:55  cwh
#Added kludge to replace the degree symbol used by VizieR to 'deg'.  The XML
#   parser was choking on the symbol.
#
#Revision 1.19  2001/11/07 23:34:21  cwh
#Moved everything except DBD/ into lib/Browse/, so as not to pollute the module
#namespace with our Sort, Search, CGI, etc.  This required putting Browse:: in
#front of every module call and package declaration.  Also any mod::new() had
#to be changed to mod->new().
#
#Revision 1.18  2001/09/15 14:35:11  cwh
#Position string no longer has ra, dec fields.
#Consolidated hard-coding of 10 arcmin radius.
#Added handling of multiple qualifiers for the same parameter.
#
#Revision 1.17  2001/08/20 20:55:23  cwh
#Fix LINK problem from VizieR before parsing.
#Changed error message and mail message.
#
#Revision 1.16  2001/08/20 16:49:04  cwh
#Added trap for XML parsing errors.
#
#Revision 1.15  2001/07/23 16:49:06  cwh
#Added $entry and $resolver to list of things split from positions line.
#
#Revision 1.14  2001/07/19 17:12:41  cwh
#set fields to null if fields=Standard.
#
#Revision 1.13  2001/07/17 18:38:06  cwh
#Added qclose()
#Added Log::VizieRLogger() call.
#
#Revision 1.12  2001/07/01 01:41:19  cwh
#Changed cgi-bin to viz-bin. On some servers either will work, on others only the latter works.
#
#Revision 1.11  2001/06/19 16:33:40  cwh
#*** empty log message ***
#
#Revision 1.10  2001/06/19 16:32:54  cwh
#Added Id and Log back in.


use strict;

package Browse::VizierQuery ;

use LWP::UserAgent;
use HTTP::Request;
use XML::Parser;

use vars qw( @ISA );
@ISA = qw( Browse::QueryRequest );

sub getData {
    $0 = 'VizierQuery.pm'; # required for MailMessage

    my ($self) = @_;
    
    my $table = $self->getTable();
    
    if (!$table) {
        #print "alert('VQ: no table');" if $::debug;
        return undef;
    }
    
    my $status = $$self{"status"};
        
    if ($status eq "initiate") {
    
        $self->setOffsetVar("_r");
        my $req = $self->makeURL();
	my $ua = new LWP::UserAgent();
        my $res = $ua->request($req);
	
				  
	my $parser = new XML::Parser("Style" => "Tree");
        if ($res->is_error) {
           my $msg = $res->error_as_HTML;
           Browse::Utils::Error("Error connecting to VizieR. $msg<p>\n");
           exit;
        }
  	my $cnt = $res->content;
        if (length($cnt)<=0) {
           Browse::MailMessage::MailMessage("Zero-length content returned from VizierQuery!\n\n Query was: $$self{query_url}\n\n$@",$::addr);
           Browse::Utils::Error("An error occurred while trying to query VizieR. An email has been sent to the maintainers.");
        }

        #Kludge
        if ($cnt =~ /A position is required to query/i) {
           Browse::Utils::Error("Error: A position or target name is required to query the $table catalog.");
        }
	
	# Fix errors in VizieR XML.
	
	# KludgeKludgeKludgeKludgeKludge
	$cnt =~ s/\<LINK \\[^\n]*//gm;
	$cnt =~ s// deg/gm;
	# KludgeKludgeKludgeKludgeKludge

        $$self{"unique_key"} = "recno"; # by default

        if ($cnt =~ /-- RowName:\s*\$\{([^}]+)/) {
           $$self{"unique_key"} = $1; 
        }
        my $xmlTree;
        eval {
	$xmlTree = $parser->parse($cnt);
        };
        if ($@) {
        #Browse::MailMessage::MailMessage("Vizier parsing error!\nVizier Request is: $$self{query_url}\n\n$@",$::addr);
        Browse::Utils::Error("*Error:  Browse cannot parse XML outputs from VizieR query.  If you have done a query of all VizieR tables you may be able to query selected VizieR tables using the Mission selections buttons.<br>Notice has been sent to the maintainers.");
        }

	
	# The XML tree should be an array of XML references.
	# See the perldoc for XML::Parser for information
	# on the format.  We assume the following general
	# structure:
	#   ASTRO
	#     DEFINITIONS          -- definitions of coordinate systems
	#     INFO                 -- parameters used in request (may be repeated)
	#     RESOURCE               -- the overall VizieR resource (may be repeated)
	#       TABLE                -- a table within the resource (may be repeated)
	#         NAME
	#         TITLE
	#         FIELD
	#         DATA
	#           CSV              -- where the data actually resides
	#
	# Many elements of the structure are omitted for clarity.
	# element of the returned data we are at.
	
	$$self{"status"} = "data";
	$$self{"tree"} = $xmlTree;
        $$self{"currentTable"} = 0;
    }
    my $data =  $self->getNextXML();
    if (!defined($data)) {
        $$self{"status"} = "done";
        $self->qclose();
    }
    return $data;
}

sub makeURL {

    my ($self) = @_;

    #if ($::debug) {
       #print "VizierQuery::self dump:<br>";
       #Browse::Utils::DumpRef($self) if $::debug;
    #}

    my $positions = $$self{"positions"};
    my $table = $$self{"table"};
    
    my $url;

    #print "table requested: $table<br>\n" if $::debug;

    if ($table =~ /^Z\/0/) {       # all tables Viz query
        $url = "-source=";
    } elsif ($table !~ /^Z\//) {   # specific table query
        $url = "-source=".Browse::CGI::Encode($table);
    } else {                       # journal table query
        my ($v, $kw, $val) = split('\/', $table);
	$url = "-kw.$kw=$val";
    }
    #print "VizierQuery::url=" . Browse::CGI::Decode($url) ."<br>" if $::debug;
    
    if ($#$positions >= 0) {
    
        my ($ra,$dec,$rad,$entry,$resolver) = split('\|', $$positions[0]);
	
	if ($rad =~ /Default/i) {
	    $rad = $$self{"radius"}/60;
	}
	
	if ($dec !~ /[\+\-]/) {
	    $dec = "+$dec";
	}
	$url .= "&-c=".Browse::CGI::Encode("$ra $dec");
	if (!defined($rad)) {
	    $rad = $$self{"radius"}/60.;
	}
	$url .= "&-c.rm=".Browse::CGI::Encode(60*$rad);
    }
    #print "VizierQuery::url=" . Browse::CGI::Decode($url) ."<br>" if $::debug;
    
    my ($sorts) = $$self{"sorts"};

    if (defined($$sorts[0])) {
        $url .= "&-sort=".Browse::CGI::Encode($$sorts[0]);
    }  else {  
        $url .= "&-sort=".Browse::CGI::Encode($self->getOffsetVar());
    }
    
    my $quals = $$self{"quals"};

    my $qual2;
    my %quals;
    foreach $qual2(@$quals) {
        my ($field, $rest) = split('\|', $qual2,2);
        $quals{$field} .= '||' if $quals{$field};
        $quals{$field} .= $qual2;
    }

#%quals = (ExpTime => "ExpTime|between|100|500||ExpTime|=|1326");
#%quals = (ExpTime => "ExpTime|!=|");

    my $fld;
    #foreach $qual(@$quals) {
    foreach $fld (keys %quals) {
    
        #print "qual vq = $quals{$fld}<br>\n" if $::debug;
        my $qual;
        my $qualgrpop ='';
        $url .= '&'.Browse::CGI::Encode($fld).'=';

        foreach $qual (split('\|\|',$quals{$fld})) {
          my ($field, $op, $p1, $p2) = split('\|', $qual);
	  $p1 =~ s/^\'(.*)\'$/$1/;
	  $p2 =~ s/^\'(.*)\'$/$1/;
 
	  if ($op eq '!=') {
	      $url .= Browse::CGI::Encode('!'.$p1);
	  } elsif ($op =~ 'between') {
	      my $not;
	      if ($op =~ /'!'/) {
	          $not = '!';
	      }
	    
	      $url .= Browse::CGI::Encode("$not$p1 .. $p2");
	  } else {
	      $url .= Browse::CGI::Encode($op.$p1);
  	  }
#          $url .= "|";
        }
    }
#    $url =~ s/\|$//; # trim off trailing | from qual group
    
    my $fields = $$self{"fields"};
    #print "VizierQuery::fields: " . $$self{"fields"} . "<br>" if $::debug;
    if ($fields =~ /^all$/i) {
        $url .= "&-out.all";
	
    } elsif (ref($fields)) {
    
        my $list;
	my $comma;
	my $fld;
	foreach $fld(@$fields) {
	    if ($fld !~ / as /) {
	        $list .= "$comma$fld";
		$comma = ",";
	    }
	}
	$url .= "&-out=".Browse::CGI::Encode($list);
    } else {
        delete $$self{"fields"} if $fields =~ /Standard/i;
    }

    #--- playing with uniq key
    my %tableinfo = Browse::HDB::TableInfo($table);
    my $uniq_key = $tableinfo{"unique_key"}; 
    my $addcols ="r,_RAJ2000,_DEJ2000";
    if ($uniq_key) {$addcols = $uniq_key ."," . $addcols;}
    #print "VizierQuery:: uniq " . $tableinfo{"unique_key"} . 
	#"cols=$addcols<br>" if $::debug;
    $url .= "&-oc=deg&-out.add=".Browse::CGI::Encode("$addcols");
    #--- end playing with uniq key
    
    if (defined($$self{"maxrows"})) {
        my $max = $$self{"maxrows"};
        if ($max !~ /^\s*\d*\s*$/) {
	    $url .= "&-out.max=9999";
	} else {
	    $url .= "&-out.max=$max";
	}
    }
        
	
    $url = "$::vizier/viz-bin/asu-xml?$url";
    #print "url: <a href=$url>$url</a><br>\n" if $::debug;
    $$self{"query_url"} = $url;
    #print "VizierQuery::url=" . Browse::CGI::Decode($url) ."<br>" if $::debug;

    Browse::Log::VizieRLogger($url,$table,join(',',@$quals),join(',',@$positions));

    return new HTTP::Request GET => $url;
    
}

sub getNextXML {

    my ($self) = @_;
    
    if ($$self{"currentdata"}) {
        my $data = $$self{"currentdata"};
	my $row = $$self{"currentrow"};
    
	if ($row > $#$data) {
	    my $data = $self->getNextTable();
	    return $data;
	} else {
	    my $sep = $$self{"colsep"};
	    my @data = split($sep, $$data[$row]);
	    if (defined($$self{"keycolumn"})) {
	        unshift(@data, $data[$$self{"keycolumn"}]);
	    }
	    my $res = {};
	    $$res{"type"}  = "data";
	    $$res{"data"}  = \@data;
	    $$res{"names"} = $$self{"names"};
	    $$res{"types"} = $$self{"types"};
            $$res{"precisions"} = $$self{"precisions"};  #long strings tam
	    $$self{"currentrow"} += 1;
	    return $res;
	}
    }
    return $self->getNextTable();
    
}

sub getNextTable {

    my ($self) = @_;
    
    my $tree  =  $$self{"tree"};
    my $table =  $$self{"table"};
    
    my $jonly = ($table =~ /^Z\/[^0]/);
    
    my $treeindices = $$self{"treeindices"};
    
    if (!defined($treeindices)) {
        #if ($$tree[0] ne "ASTRO") {
        #--- XML returned from Vizier changed 3/2008 
        if ($$tree[0] ne "VOTABLE") {
           # print "alert('VQ: this is $$tree[0], not astro');" if $::debug;
            return undef;
        }
	
	$treeindices = [0,0];
	$$self{"treeindices"} = $treeindices;
	
    }
    my $astro = $$tree[1];
    #print "<br>astro tree= @$astro  <br>" if $::debug;
    
    my $resflds;
    
    my ($res, $tab) = @$treeindices;
    
    if ($res == 0) {
        $res = findField("RESOURCE", $astro, $res);
	if ($res < 0) {
        #print "alert('VQ: no res');" if $::debug;
	    return undef;
	}
	
	$resflds = $$astro[$res+1];
	$self->updateResource($resflds);
	$tab = 0;
    } else {
        $resflds =  $$astro[$res+1];
    }
    
    while (1) {
    
        $tab = findField("TABLE", $resflds, $tab);
	if ($tab < 0) {
	    $res = findField("RESOURCE", $astro, $res);
	    if ($res < 0) {
                #print "alert('VQ: no res in loop');" if $::debug;
	        return undef;
	    } else {
	        $resflds = $$astro[$res+1];
	        $self->updateResource($resflds);
	    }
	    
	} else {
	    my $tabflds = $$resflds[$tab+1];
	    
	    if (findField("DATA", $tabflds, 0) > 0 ) {
	        my $meta = $self->metaInfo($tabflds);
		if (($jonly && $$meta{"table"} !~ /^J\//) || !$meta ) {
		} else {
		    @$treeindices = ($res, $tab);
                    #print "VQ:returning meta <br>" if $::debug;
                    #Browse::Utils::DumpRef($meta) if $::debug;
		    return $meta;
		}
	    }
	}
    }
}

sub updateResource {
    my ($self, $resflds) = @_;
    
    my $ind;
    
    $ind = findField("NAME", $resflds, 1);
    if ($ind > 0) {
	$$self{"resource_name"} = getText($$resflds[$ind+1]);
        #print "VizQ:: name found! $$resflds[$ind+1]" if $::debug;
    }
    $ind = findField("TITLE", $resflds, 1);
    if ($ind > 0) {
	$$self{"resource_title"} = getText($$resflds[$ind+1]);
    }
} 

sub metaInfo {

    my ($self, $table) = @_;
    
    my %paraminfo = ();
    my %tableinfo = ();
    
    my @fields;
    my @types;
    my @precisions;  #long strings tam
    
    my $i;
    my $tname;
    
    print "VQ:-Processing Meta<br>" if $::debug;
    #Browse::Utils::DumpRef($table) if $::debug;
    for ($i=0; $i<= $#$table; $i += 1) {
    
        my $key = $$table[$i];
	my $val = $$table[$i+1];
        #print "VQ: key=$key r=" . ref($key) ." val=$val r=" . 
		#ref($val) ."<br>" if $::debug;

        #if ($key eq "NAME") {
        if (ref($key) =~ /HASH/ && $tableinfo{"table_name"} !~ /\w/) {
	    #$tname = getText($val);
	    #$tableinfo{"table_name"}= $tname;
            my $hash = $key;
            print "VQ: table=$$hash{name} id=$$hash{ID} <br>" if $::debug;

            if ($$hash{"name"} && $$hash{"ID"}) {
               $tname=$$hash{"name"};
               print "VQ: table=$tname<br>" if $::debug;
               if ($tname !~ /\//) {
                  #-Browse wants Vizier names to contain slash(/) so ignore the
                  #-few that don't
                  return undef;
               } else {
	          $tableinfo{"table_name"} = $tname;
               }
            }
	    
	#} elsif ($key eq "TITLE") {
	} elsif ($key eq "DESCRIPTION") {
	    #$tableinfo{"table_description"} = getText($val);
            my $vName=getText($val);
            #--- see if table name used can be improved
            $vName = Browse::Vizier::convertName($vName);
	    $tableinfo{"table_description"} = $vName;
	    
	} elsif ($key eq "FIELD") {
	    $self->processField($val, \%paraminfo, \@fields, \@types, 
		\@precisions);   #long strings tam

	    
	} elsif ($key eq "DATA") {
	    $self->prepareData($val);
	}
	    
    }
    
    #$self->finalCheck(\%paraminfo, \%tableinfo, \@fields, \@types);
    $self->finalCheck(\%paraminfo, \%tableinfo, \@fields, \@types, 
	\@precisions);   #long strings tam

    
    $tableinfo{"table_location"}     = "vizier.u-strasbg.fr";
    
    # Don't have documentation for individual tables, just
    # the overall resource.
    
    my @de = split("/", $tname);
    if ($de[0] eq "J") {
        $tname = "$de[0]/$de[1]/$de[2]/$de[3]";
    } else {
        $tname = "$de[0]/$de[1]";
    }
    my $keystr;
    if ($tableinfo{"query_key"}) {
        $keystr = "$tableinfo{query_key}:int";
    }
    
    my @flds = split('/', $tname);
    
    my $tdoc;
    if ($table =~ /^J/) {
	$tdoc = "$flds[0]/$flds[1]/$flds[2]/$flds[3]";
    } else {
        $tdoc = "$flds[0]/$flds[1]";
    }
	
    $tableinfo{"table_document_url"} = "$::vizier/viz-bin/Cat?$tdoc";
    
    my $pos;
    if ($$self{"allpositions"}) {
        $pos = $$self{"allpositions"};
    } else {
        $pos = $$self{"positions"};
    }
    $$self{"names"} = \@fields;
    $$self{"types"} = \@types;
    $$self{"precisions"} = \@types;   #long strings tam
    my %meta = (
       "table"       => $tableinfo{"table_name"},
       "tableinfo"   => \%tableinfo,
       "paraminfo"   => \%paraminfo,
       "dpsets"      => undef,
       "keyinfo"     => $keystr,
       "fields"      => $$self{"names"},
       "positions"   => $pos,
       "sorts"       => $$self{"sorts"},
       "dirs"        => $$self{"dirs"},
       "quals"       => $$self{"quals"},
       "type"        => "meta",
       "resource_name" => $$self{"resource_name"},
       "resource_title"=> $$self{"resource_title"},
       "sortable"      => "down",
       "query_url"     => $$self{"query_url"},
       "serialization" => $$self{"serialization"}
      );
     
    
    return \%meta;
}
	    
sub getText {
    
    my ($tag) = @_;
    my $i;
    
    for ($i=1; $i < $#$tag; $i += 2) {
        if ($$tag[$i] eq "0") {
	    return $$tag[$i+1];
	}
    }
}

sub prepareData {

    my ($self, $val) = @_;
    
    my $i;
    my $csv;
    
    for ($i=1; $i < $#$val; $i += 1) {
        if ($$val[$i] eq "CSV") {
	    $csv = $$val[$i+1];
	}
    }
    
    if (!$csv) {
        $$self{"currentdata"} = [];
	$$self{"currentrow"} = 0;
	return;
    }
    
    my $hash;
    for ($i=0; $i < $#$val; $i += 1) {
    
        if ($$csv[$i] eq "0") {
	
	    my $datastring = $$csv[$i+1];
	    my @lines = split("\n", $datastring);
	    $hash = $$csv[0];
	    my $j;
	    
	    if ($lines[0] =~ /^\s*$/) {
	        shift(@lines);
	    }
	    
	    for ($j=0; $j < $$hash{"headlines"}; $j += 1) {
	        shift(@lines);
	    }
	    $$self{"currentdata"} = \@lines;
	    $$self{"currentrow"}  = 0;
	    last;
	}
    }
    
    if ($$hash{"colsep"}) {
        $$self{"colsep"} = $$hash{"colsep"};
    } else {
        $$self{"colsep"} = "\t";
    }
    
    if (!$$self{"currentdata"}) {
        $$self{"currentdata"} = [];
	$$self{"currentrow"} = 0;
    }
}
	
sub finalCheck {

    #my ($self, $paraminfo, $tableinfo, $fields, $types) = @_;
    #long strings tam
    my ($self, $paraminfo, $tableinfo, $fields, $types, $precisions) = @_;
    
    my $field;
    my $raf;
    my $decf;
    my $key;
    my $unique_key = $$self{"unique_key"};
    
    my $i;
    delete ($$self{"keycolumn"});
    
    for ($i=0; $i <= $#$fields; $i += 1) {
        $field = $$fields[$i];
	
        if ($field eq "_RAJ2000") {
	    $raf = $field;
	    
	} elsif ($field eq "_DEJ2000") {
	    $decf = $field;
	}
	
	if ($field =~ /(\_)?$unique_key/i) {
	    $key = $field;
	    $$self{"keycolumn"} = $i;
	}
    }
    
    if (!defined($$self{"keycolumn"})) {
        
        for ($i=0; $i <= $#$fields; $i += 1) {
            $field = $$fields[$i];
	    
            #if ($field ne "_RAJ2000" && 
	        #$field ne "_DEJ2000") {
            #--- use first non-coordinate non-search offset field as key 
	    #--- (used for parameter listing)
            if ($field !~ /RAJ2000/ && 
	        $field !~ /DEJ2000/ && 
	        $field !~ /_r/  ) {
	        $key = $field;
	        $$self{"keycolumn"} = $i;
		last;
 	    }
        }
    }
	       
    if (defined($raf) && defined($decf)) {
        #$$tableinfo{"default_search_radius"} = 10;
        $$tableinfo{"default_search_radius"} = 1;
        $$tableinfo{"right_ascension"}       = $raf;
	$$tableinfo{"declination"}           = $decf;
    }
    
    
    $$paraminfo{"#mjds#"}        = [];
    $$paraminfo{"#all#"}         = $fields;
    $$paraminfo{"#default#"}     = $fields;
    $$paraminfo{"#not_default#"} = [];   
    
    if (defined($key)) {
        $$tableinfo{"query_key"} = $key;
	unshift(@$fields, "w3_$key");
	unshift(@$types, 4);
	unshift(@$precisions, 4);   #long strings tam
    } else {
        $$tableinfo{"query_key"} = "_unknown_";
	
    }	    
}

sub findField {

    my ($want, $vals, $pos) = @_;
    
    # Need to search odd values.
    if (! $pos%2) {
        $pos += 1;
    } else {
        $pos += 2;
    }
    
    while ($pos <= $#$vals) {
        #print "pos=$pos val=$$vals[$pos]<br>" if $::debug;
        if ($$vals[$pos] eq $want) {
	    return $pos;
	}
	$pos += 2;
    }
    return -1;
}

sub qclose {

    my ($self) = @_;
    $$self{"currentdata"} = undef;
    $$self{"currentrow"}  = undef;
    $$self{"treeindices"} = undef;
    
    $$self{"status"} = "initiate";
}


sub processField  {

    #my ($self,$vals, $paraminfo, $names, $types) = @_;
    #long strings tam
    my ($self,$vals, $paraminfo, $names, $types, $precisions) = @_;
    
    my $hash = $$vals[0];

    my $key = $$hash{"name"};
    my $param = {};
    $$param{"parameter_name"} = $key;
    
    $$param{"parameter_unit"} = $$hash{"unit"};
    
    
    #--- data returned from Vizier queries change from time to time
    #--- recent change - 10/2008
    my $type = $$hash{"datatype"};
    my $prec = (defined $$hash{"precision"}) ? ":.". 
	$$hash{"precision"}. "f" : ""; 
    my $len =  (defined $$hash{"arraysize"}) ? $$hash{"arraysize"} : 
	$$hash{"width"};
    #print "VQ: type=$type len= $len prec=$prec<br>" if $::debug;
    
    if ($type eq "A" || $type eq "char") {
    
        $$param{"parameter_format"} = "char$len";
        #long strings tam
	push(@$types, 1);
	push(@$precisions, $len);

    } elsif ($type eq "F" || $type eq "float") {
        $$param{"parameter_format"} = "float4".$prec;
	push(@$types, 7);
	push(@$precisions, 4);   #long strings tam
    } elsif ($type eq "D" || $type eq "double") {
        $$param{"parameter_format"} = "float8".$prec;
	push(@$types, 6);
	push(@$precisions, 8);  #long strings tam
    } else {
        $$param{"parameter_format"} = "int4";
	push(@$types, 4);
	push(@$precisions, 4);  #long strings tam
    } 

    #--- change the column name for the offset to match heasarc tables 
    #my $currentOffsetVar = $self->getOffsetVar();
    #if ($key =~ /^$currentOffsetVar$/) {
      #$key = "_Search_Offset"; 
      #$self->setOffsetVar($key);
    #}

    push(@$names, $key);
    $$paraminfo{$key} = $param;
}


1;
