# $Id: HCS.pm,v 1.5 2001/11/08 20:16:44 cwh Exp $
# $Log: HCS.pm,v $
# Revision 1.5  2001/11/08 20:16:44  cwh
# Added Browse:: to sub calls.
#
# Revision 1.4  1999/11/10 21:29:50  saima
# Minor modifications to help track errors.
#
# Revision 1.3  1999/10/08 20:36:08  saima
# Initial Revision.
#
#   DBD::HCS- A base class for implementing DBI talking to
#             HEASARC client-server databases
#
#   This DBD driver makes non-persistent connections to
#   the database.  Only selects are supported.  This
#   probably crashes in any number of hideous ways
#   if you try to push it too hard.

require 5.004;
use strict;
use DateTime;

require DynaLoader;

require DBI;
require Browse::CSIO;

my $haveFileSpec = eval { require File::Spec };

package DBD::HCS;

use vars qw(@ISA $VERSION $drh $err $errstr $sqlstate);

@ISA = qw(DynaLoader);

$VERSION = '0.01';

$err = 0;		# holds error code   for DBI::err
$errstr = "";		# holds error string for DBI::errstr
$sqlstate = "";         # holds error state  for DBI::state
$drh = undef;		# holds driver handle once initialised

sub driver ($;$) {

    my($class, $attr) = @_;
    my $drh = eval '$' . $class . "::drh";
    if (!$drh) {
    
	if (!$attr) { $attr = {} };
	if (!exists($attr->{Attribution})) {
	    $attr->{Attribution} = "$class by Thomas McGlynn";
	}
	if (!exists($attr->{Version})) {
	    $attr->{Version} = eval '$' . $class . '::VERSION';
        }
        if (!exists($attr->{Err})) {
	    $attr->{Err} = eval '\$' . $class . '::err';
        }
        if (!exists($attr->{Errstr})) {
	    $attr->{Errstr} = eval '\$' . $class . '::errstr';
        }
        if (!exists($attr->{State})) {
	    $attr->{State} = eval '\$' . $class . '::state';
        }
        if (!exists($attr->{Name})) {
	    my $c = $class;
	    $c =~ s/^DBD\:\://;
	    $attr->{Name} = $c;
        }

        $drh = DBI::_new_drh($class . "::dr", $attr);
	
    }
    $DBD::HCS::drh = $drh;
    $drh;
}


package DBD::HCS::dr; # ====== DRIVER ======

$DBD::HCS::dr::imp_data_size = 0;
$DBD::HCS::dr::data_sources_attr = undef;

sub connect ($$;$$$) {
    my($drh, $dbname, $user, $auth, $attr)= @_;

    # create a 'blank' dbh
    my $this = DBI::_new_dbh($drh, {
	'Name' => $dbname,
	'USER' => $user, 
	'CURRENT_USER' => $user,
    });

    # Just return, we won't even try to connect here.
    

    $this;
}

sub data_sources ($;$) {
    return undef;
}

sub disconnect_all {
    undef $DBD::HCS::drh;
}

sub DESTROY {
    undef;
}


package DBD::HCS::db; # ====== DATABASE ======

$DBD::HCS::db::imp_data_size = 0;


sub prepare ($$;@) {

    my($dbh, $statement, @attribs)= @_;

    # create a 'blank' dbh
    my $sth = DBI::_new_sth($dbh, {'Statement' => $statement});
    if ($sth) {
	$@ = '';
	my $class = $sth->FETCH('ImplementorClass');
	$class =~ s/::st$/::Statement/;
	my($stmt) = eval { $class->new($statement) };
	if ($@) {
	    DBI::set_err($dbh, 1, $@);
	    undef $sth;
	} else {
	    $sth->STORE('f_stmt', $stmt);
	    $sth->STORE('f_params', []);
	    $sth->STORE('NUM_OF_PARAMS', undef);
	}
    }

    $sth;
}

sub disconnect ($) {
    1;
}

sub FETCH ($$) {
    my ($dbh, $attrib) = @_;
    if ($attrib eq 'AutoCommit') {
	return 1;
    } elsif ($attrib eq (lc $attrib)) {
	# Driver private attributes are lower cased
	return $dbh->{$attrib};
    }
    # else pass up to DBI to handle
    return $dbh->DBD::_::db::FETCH($attrib);
}

sub STORE ($$$) {
    my ($dbh, $attrib, $value) = @_;
    if ($attrib eq 'AutoCommit') {
	return 1 if $value; # is already set
	croak("Can't disable AutoCommit");
    } elsif ($attrib eq (lc $attrib)) {
	# Driver private attributes are lower cased
	$dbh->{$attrib} = $value;
	return 1;
    }
    return $dbh->DBD::_::db::STORE($attrib, $value);
}

sub DESTROY ($) {
    undef;
}

sub type_info_all ($) {
    [
    ]
}


sub table_info ($) {
    undef;
}

sub commit ($) {
    my($dbh) = shift;
    warn("Commit meaningless", -1);
    1;
}

sub rollback ($) {
    my($dbh) = shift;
    warn("Rollback meaningless", -1);
    0;
}


package DBD::HCS::st; # ====== STATEMENT ======

$DBD::HCS::st::imp_data_size = 0;

sub bind_param ($$$;$) {
    my($sth, $pNum, $val, $attr) = @_;
    $sth->{f_params}->[$pNum-1] = $val;
    1;
}

sub execute {

    my $sth = shift;
    my $db = $sth->{Database};
    my $name = $db->{Name};
    my $sql = $sth->{Statement};
    my $stmt = $sth->{f_stmt};
    
    
    my ($host, $port) = split(":", $name);
    
    Browse::CSIO::client($host, $port);
    
    Browse::CSIO::output("sql=$sql", 0);

    # Get initial Sync.

    my ($status, $msg) = Browse::CSIO::input(0);
    
    if ($status != $Browse::CSIO::okStatus || defined($msg)) {
        exit(-1);
    }

    # Now read until undefined message.

    ($status, $msg) = Browse::CSIO::input(0);

    my $result;
    
    while ($status == $Browse::CSIO::okStatus && $msg) {
 
        chop($msg);
	$result .= "$msg\n";
        ($status, $msg) = Browse::CSIO::input(0);    
    }
    
    my @lines = split("\n", $result);
    if ($lines[0] =~ /^\+[\+\-]*\+$/) {
    
        $result = [];
	shift(@lines);
	my $head = shift(@lines);
	my $fields;
	
	# Specify -1 to ensure that blanks at end don't get
	# lost
	@$fields = split(' *\| *', $head, -1);
	shift(@$fields);
	pop(@$fields);
	
	my $maybeTypes = shift(@lines);
	pop(@lines);
	pop(@lines);
	my $line;
	foreach $line(@lines) {
	    my $flds;
	    # Specify -1 to ensure blanks at end don't get lost.
	    @$flds = split(' *\| *', $line, -1);
	    shift(@$flds);
	    pop(@$flds);
	    push(@$result, $flds);
	}
	$stmt->{data} = $result;
	$sth->{"NAME"} =  $fields;
	my @types;
	if ($maybeTypes !~ /^[\+\|\-]*$/) {
	    @types = split('\| *', $maybeTypes);
	    shift(@types);
	    pop(@types);
	} else {
	    @types= @$fields;
	    foreach (@types) {
	        $_ = 0;
	    }
	}

	$sth->{"TYPE"} = \@types;
	$sth->{"PRECISION"} = \@types;
	
	return 1;
    } else {
        DBI::set_err($db,1,"Connection error at: ".DateTime::asctim()."\n".$result);
        return undef;
    }
    
    return $result;
}


sub fetch ($) {

    my $sth = shift;
    my $data = $sth->{f_stmt}->{data};
    if (!$data  ||  ref($data) ne 'ARRAY') {
	DBI::set_err($sth, 1,
		     "Attempt to fetch row from a Non-SELECT statement");
	return undef;
    }
    my $dav = shift @$data;
    if (!$dav) {
	return undef;
    } else {
        return $dav;
    }
}
*fetchrow_arrayref = \&fetch;

sub FETCH ($$) {
    my ($sth, $attrib) = @_;
    return undef if ($attrib eq 'TYPE'); # Workaround for a bug in DBI 0.93
    return $sth->FETCH('f_stmt')->{'NAME'} if ($attrib eq 'NAME');
    if ($attrib eq 'NULLABLE') {
	my($meta) = $sth->FETCH('f_stmt')->{'NAME'}; # Intentional !
	if (!$meta) {
	    return undef;
	}
	my($names) = [];
	my($col);
	foreach $col (@$meta) {
	    push(@$names, 1);
	}
	return $names;
    }
    if ($attrib eq (lc $attrib)) {
	# Private driver attributes are lower cased
	return $sth->{$attrib};
    }
    # else pass up to DBI to handle
    return $sth->DBD::_::st::FETCH($attrib);
}

sub STORE ($$$) {
    my ($sth, $attrib, $value) = @_;
    if ($attrib eq (lc $attrib)) {
	# Private driver attributes are lower cased
	$sth->{$attrib} = $value;
	return 1;
    }
    return $sth->DBD::_::st::STORE($attrib, $value);
}

sub DESTROY ($) {
    undef;
}

sub rows ($) { shift->{'f_stmt'}->{'NUM_OF_ROWS'} };

sub finish ($) { 1; }


package DBD::HCS::Statement;

sub new {

    my $obj = {};
    return bless $obj, $_[0];
    
}
    

package DBD::HCS::Table;

sub drop ($) {
    return 1;
}

1;
