#!/usr1/local/bin/perl5 -T 
#
# $Id: uploadQuery.pl,v 1.6 2006/07/03 13:53:18 lmm Exp $
#
# ------------ based on cgilib.pl script 
# ------------ Copyright (c) 1996 Steven E. Brenner
# Read a previously stored Browse query from user a file 
# and run it


require 5.001;
use strict;
use lib './lib';

use Browse::CGILIB;
use Browse::Config;
use Browse::Decor;
use Browse::Parameter;
use Browse::Utils;
use FreezeThaw qw(freeze thaw cmpStr safeFreeze cmpStrHard);

Browse::Config::AlwaysRun();
#$::debug = 1;

print Browse::CGI::Header(),"uploadQuery: DEBUG MODE<br>\n" if $::debug;

  my (%cgi_data,  # The form data
      %cgi_cfn,   # The uploaded file(s) client-provided name(s)
      %cgi_ct,    # The uploaded file(s) content-type(s).  These are
                  #   set by the user's browser and may be unreliable
      %cgi_sfn,   # The uploaded file(s) name(s) on the server (this machine)
      $ret,       # Return value of the ReadParse call.       
      $buf        # Buffer for data read from disk.
     );

  # When writing files, several options can be set..
  # Spool the files to the /tmp directory
  #$cgi_lib::writefiles = "/tmp";   
 
  # Limit upload size to avoid using too much memory
  #$cgi_lib::maxdata = 50000; 

  # Start off by reading and parsing the data.  Save the return value.
  # Pass references to retreive the data, the filenames, and the content-type
  $ret = Browse::CGILIB::LibReadParse(\%cgi_data,\%cgi_cfn,\%cgi_ct,\%cgi_sfn);

  # A bit of error checking never hurt anyone
  if (!defined $ret) {
    Browse::Utils::Error("Error in reading and parsing of CGI input");
  } elsif (!$ret) {
    Browse::Utils::Error("Missing parameters<br>\n");
  } elsif (!defined $cgi_data{'upfile'} ) {
    Browse::Utils::Error("Data missing<br>\n");
	    
  }

  my($frozenQuery) = "";   # holds encoded and serialized query from file

  # Read the contents of the uploaded file
  open (UPFILE, $cgi_sfn{'upfile'}) or 
    &CgiError("Error: Unable to open file $cgi_sfn{'upfile'}: $!\n");

  $buf = "";    # avoid annoying warning message
  while (read (UPFILE, $buf, 8192)) {
    # Munge the uploaded text so that it doesn't contain HTML elements
    # This munging isn't complete -- lots of illegal characters are left
    # However, it takes care of the most common culprits.  
    $buf =~ s/</&lt;/g;
    $buf =~ s/>/&gt;/g;
    $frozenQuery .= $buf;;
  }

  close (UPFILE);

 
  if ($frozenQuery eq "") {
     my ($msg) = "Please enter a file containing a previously saved query";
     Browse::Utils::Error($msg);
  }

  my($decoded) = Browse::CGI::Decode($frozenQuery);
  my ($tables, $positions, $qualifiers, $in, $batchflag,$resmsg); 

  #-- Fatal error occurs if incorrect file is uploaded
  my $wrongFileMsg ="Error reading query file. Please check that " .
	" the correct previously saved query file has been selected.";

  eval { ($tables, $positions, $qualifiers, $in, $batchflag,$resmsg) = 
      FreezeThaw::thaw $decoded;}; 


   if ($::debug) {
      foreach my $key (%$in) {
         print Browse::CGI::Decode($key) . " ===>" .  
	 	Browse::CGI::Decode($$in{$key}) . "<br>";
      }
   }

  if ($@) {Browse::Utils::Error($wrongFileMsg);}

  my $parameterMode;
  if ($in->{displaymode} =~ /Tabbed/) {
       $parameterMode='TabbedParameter';
  } else { 
       $parameterMode='Parameter';
  }

  my $parameter = Browse::Parameter->new($parameterMode);

  # Set display mode
  my($textdisplay) = ($in->{'TextDisplay'} =~ /on/i) ? 1 : 0;
  my $mode = $$in{"displaymode"};
  if (!defined($mode)) {
     $mode = "TextDisplay" if $textdisplay;
  }
  $parameter->setDisplayMode($mode);

  #-- send query to Browse
  $parameter->Search($tables, $positions, $qualifiers, $in, 
	$batchflag,$resmsg); 
    
  # cleanup - delete the uploaded file
  # Note that when using spooling of files to disk, the uploaded file's
  # name on the server machine is in both %cgi_data and %cgi_sfn
  # (that is, the first and fourth parameters to ReadParse).  However,
  # for technical reasons, the data in %cgi_data are tainted.  The data in
  # %cgi_sfn are not tainted, but the keys can contain only a limited
  # set of characters ([-\w] in cgi-lib 2.8)

  unlink ($cgi_sfn{'upfile'}) or
   &CgiError("Error: Unable to delete file",
             "Error: Unable to delete file $cgi_sfn{'upfile'}: $!\n");



# $Log: uploadQuery.pl,v $
# Revision 1.6  2006/07/03 13:53:18  lmm
# Made changes for new Parameter class. A Parameter object is created and
#   the display object is set.
#
# Revision 1.5  2005/03/21 19:32:26  lmm
# Added evaluation of thaw() call due to fatal error if user attempts to
#   upload a non-valid file.
#
# Revision 1.4  2005/02/15 16:01:55  lmm
# Moved FreezeThaw.pm again to more appropriate library directory
#
# Revision 1.3  2005/02/01 16:12:28  lmm
# FreezeThaw.pm has been moved out of the Browse directory to the
#   lib directory since it is an imported package
#
# Revision 1.2  2004/02/23 16:09:06  lmm
# Replaced calls to non-existent script (CgiDie) with Utils::Error
#
# Revision 1.1  2003/12/02 22:09:14  lmm
# *** empty log message ***
#
