#!/usr1/local/bin/perl5 -T
# $Id: w3cross.pl,v 1.34 2005/12/13 16:35:42 lmm Exp $
# Revision 1.18  2001/05/03 19:13:14  cwh
# Put call to w3config.pl in a BEGIN statement, pulled all 'use lib's out.
#
# Revision 1.17  2001/01/22 20:30:25  cwh
# Added navigation trail.
#
# Revision 1.16  2000/04/25 18:46:57  saima
# Made minor changes for debug mode.
#
# Previous comments deleted from in 1.19 since most functionality
# has been moved to the Cross.pm and CrossRequest.pm modules.
#
#=========================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name & address intact.
#=========================================================================#
# Module Name:    w3cross.pl
# Module Author:  Edward J. Sabol, Hughes STX
# Date:           February 1997
# Project:        Browse : HEASARC
#=========================================================================#
# Module Description:
#
#
#
#=========================================================================#

use strict;
use lib './lib';

use Browse::Config;
use Browse::CrossRequest;
use Browse::Display;
use Browse::Log;

Browse::Config::AlwaysRun();

$0 = 'w3cross.pl'; # Required for MailMessage - do not remove.
#--------------------------------------------------------------------------
#
#$::debug = 1;
$| = 1;

if ($::debug) {
    print "Content-type: text/html\n\n";
}

my $in = Browse::CGI::ReadParse();

my $navtrail=$in->{'navtrail'};
$navtrail = "<a class='navpast' href='w3fullcross.pl'>Choose Cross-Correlation Tables</a>" unless $navtrail;

my $table1 = $in->{Table1};
my $table2 = $in->{Table2};

$in->{'Coordinates'} = "Equatorial" if (! $in->{'Coordinates'});
$in->{'Equinox'}     = 2000         if (! $in->{'Equinox'});

# Get options.
my ($pcross)=      ($in->{'CrossRadius'}     =~ /on/i) ? 1 : 0;
my ($tcross)=      ($in->{'CrossTime'}       =~ /on/i) ? 1 : 0;

my ($textdisplay)= ($in->{'DisplayText'}     =~ /on/i) ? 1 : 0;
my ($anticorr)   = ($in->{'Anticorrelation'} =~ /on/i) ? 1 : 0;
my ($multiline)  = ($in->{'Multiline'}       =~ /on/i) ? 1 : 0;
my ($secondary)  = ($in->{'ShowSecond'}      =~ /on/i) ? 1 : 0;
my ($minimal)    = ($in->{'Minimal'}         =~ /on/i) ? 1 : 0;

if ($anticorr) {
    $secondary = 0;
}

#my ($xradius)    =  Browse::CGI::Decode($in->{'XRadius'});
my ($xradius)    =  $in->{'XRadius'};
my ($xtime)      =  $in->{'XTime'};

  # allow old unit specification style
  my ($nradius,$unit) = $xradius =~ /([\d.]+)(.*)$/;
  $in->{'XRadius_unit'} = $unit if $unit;
  $xradius = $nradius if $nradius;


  my($degsymbol) = "\xb0";

  #convert to arcminutes
  if ($in->{'XRadius_unit'} =~ /sec|\"/) {                    # arcseconds
      $xradius = $xradius / 60;
  } elsif ($in->{'XRadius_unit'} =~ /degree|$degsymbol/) {       # degrees
      $xradius = $xradius * 60;
  }


my ($maxlimit)   = defined($in->{'ResultMax'}) ? $in->{'ResultMax'} : '0';
my (%tinfo1) = Browse::HDB::TableInfo($table1) if($table1);
my (%tinfo2) = Browse::HDB::TableInfo($table2) if($table2);

ErrorChecks($table1, $table2,
             \%tinfo1,\%tinfo2,
	     $pcross, $tcross, 
	     $xradius,$xtime);
	     
	     

# Change defaults to actual values.  Use the bigger of the two defaults.
if ($xradius =~ /default/i) {
    $xradius = 
     ($tinfo1{"default_search_radius"} > $tinfo2{"default_search_radius"} ? 
      $tinfo1{"default_search_radius"} : $tinfo2{"default_search_radius"});
}

if ($xtime =~ /default/i) {
    $xtime = 1;   		# Units are days.
}

my $setup = {
    "xradius"         => $xradius,
    "xtime"           => $xtime,
    "displaymode"     => $$in{"displaymode"},
    "anticorrelation" => $anticorr,
    "secondary"       => $secondary,
    "minimal"         => $minimal,
    "multiline"       => $multiline,
    "pcross"          => $pcross,
    "tcross"          => $tcross,
    "maxlimit"        => $maxlimit,
    "navtrail"        => $navtrail,
    "fields"          => $$in{"Fields"},
    "coordinates"     => $$in{"Coordinates"},
    "equinox"         => $$in{"Equinox"},
    "popupFrom"        => $$in{"popupFrom"}   #lmm
    };
    

my $reqinfo1 = Browse::CGI::Decode($$in{Info1});
my $reqinfo2 = Browse::CGI::Decode($$in{Info2});

processRequest($table1, $table2, $reqinfo1, $reqinfo2, $setup);



sub processRequest {

    my ($table1, $table2, $info1, $info2, $setup) = @_;
    
    my $req1 = Browse::QueryRequest::regenerate($table1, $info1);
    my $req2 = Browse::QueryRequest::regenerate($table2, $info2);
    $req1->setFields($$setup{"fields"});
    $req2->setFields($$setup{"fields"});

    my $request   = Browse::CrossRequest->new($req1, $req2, $setup);
    my $mode;

    $mode = $$setup{"displaymode"} if $$setup{"displaymode"} =~ /\w/;
    my $displayer = Browse::Display->new($mode);

    $displayer->setCoordSys($$setup{"coordinates"}, $$setup{"equinox"});

    $displayer->setPopupRef($$setup{"popupFrom"}); #lmm
    $displayer->setHeaderColor("xcortable"); #lmm

    $displayer->pageHeader($$setup{"navtrail"});
#    my($sorts, $dirs) = OrderClause($in);

    my $block;

    $request->setMaxRows($$setup{"maxlimit"});
    $displayer->MaxRows($$setup{"maxlimit"});
    $displayer->YourSearchWas($req1->getPositions(),$req1->getQuals(),
        [undef],[undef],"",[undef]);

    my $nrow = 0;
    $request->storeState();
    while ($block = $request->getResult()) {
        if ($$block{"type"} eq "meta") {
            $displayer->processMeta($block);
        } else {
            if ($nrow == 0) {
	        $displayer->tableHeader($block);
	        $displayer->legend($block);
	    }
            $displayer->processData($block);
	    $nrow += 1;
        }
    }

    if ($nrow == 0) {
        print "<p><strong> No matching rows found</strong></p>\n";
    }

    $displayer->closeTable();
    $displayer->tableFooter();
    $displayer->pageFooter();
   my $resultsflag = 1 if $info1 or $info2;
    Browse::Log::Query($0,$in,"Cross Search",[($table1,$table2)],$resultsflag);
}

#--------------------------------------------------------------------------
sub ErrorChecks{

  my($table,$xtable,
     $table_info,$xtable_info,
     $pcross,$tcross, $xradius, $xtime) = @_;

  Browse::Utils::Error("Error: You asked for a positional cross-correlation, but no".
           " radius was specified.  ")
           if ($pcross && !$xradius);
	   
  Browse::Utils::Error("Error: You asked for a time cross-correlation, but no".
           " time delta was specified.  ")
           if ($tcross && !$xtime);
	   
  Browse::Utils::Error("Error: Both a primary and secondary correlation table are required.")
        unless ($table && $xtable);

  Browse::Utils::Error("Error: Please select either radius and/or time option to " .
        "cross-correlate results.") 
	unless ($pcross || $tcross);


  Browse::Utils::Error("Error: Insufficient metadata to correlate $table postions")
      if ((!defined($table_info->{'right_ascension'})
          || !defined($table_info->{'declination'})
          || !defined($table_info->{'equinox'})) && $pcross);

  Browse::Utils::Error("Error: You cannot cross-correlate the $xtable " .
           "table since it does not have right ascension and " .
           "declination parameter fields defined.")
      if ((!defined($xtable_info->{'right_ascension'})
          || !defined($xtable_info->{'declination'})
          || !defined($xtable_info->{'equinox'})) && $pcross);

  Browse::Utils::Error("Error: You cannot cross-correlate the selected tables ".
            "since they are not in the same equinox.")
       if (($table_info->{'equinox'} ne $xtable_info->{'equinox'}) 
            && $pcross);

  Browse::Utils::Error("Error: You cannot cross-correlate on time with the ".
           "$table table.") 
      if ((!defined($table_info->{'start_time'})) && $tcross);

  Browse::Utils::Error("Error: You cannot cross-correlate on time with the ".
           "$xtable table.") 
      if ((!defined($xtable_info->{'start_time'})) && $tcross);

  if ($tcross) {
    $_ = $table_info->{'start_time'};
    Browse::Utils::Error("Error: You cannot cross-correlate on time with the ".
             "$table table.") if (!/[A-Za-z_].\w+/); 

    $_ = $xtable_info->{'start_time'};
    Browse::Utils::Error("Error: You cannot cross-correlate on time with the ".
             "$xtable table.") if (!/[A-Za-z_].\w+/); 
  }

} #eosub--ErrorChecks
#--------------------------------------------------------------------------

#########
#  EOF  #
#########
