#!/usr1/local/bin/perl5 -T
#
# $Id: w3query.pl,v 1.80 2006/11/29 15:46:37 lmm Exp $
# $Log: w3query.pl,v $
# Revision 1.80  2006/11/29 15:46:37  lmm
# Changes to accommodate replacement of w3query.pl with Query.pm
#
# Revision 1.79  2006/10/24 16:58:24  lmm
# Removed second declaration of $in variable
#
# Revision 1.78  2006/08/24 14:33:45  lmm
# Fixed bug that prevented class parameter from being selected as
#   default parameter.
#
# Revision 1.77  2006/06/27 18:31:55  lmm
# Added check for batch request before setting display mode (needed
#   for older version of browse_extract.pl)
#
# Revision 1.76  2006/06/23 13:52:11  lmm
# Changes for new Tabbed Display (including new Parameter class)
#
# Revision 1.75  2006/01/25 19:19:09  lmm
# Class list on Parameter page now sorted by class name not number
#
# Revision 1.74  2005/11/21 18:33:27  lmm
# minor debug changes
#
# Revision 1.73  2004/01/15 18:35:23  lmm
# Converted script to subroutine to be used to replace "Location:"
#   (redirection) code in other scripts.
#
# Revision 1.72  2003/10/07 14:37:05  lmm
# Changed " to ' around a link variable to fix an intermittent html
#    problem.  Problem seen when starting query from "Index of all Tables"
#    page.
#
# Revision 1.71  2003/08/28 20:33:27  lmm
# Defined text to use for the next page header
#
# Revision 1.70  2003/07/31 14:28:02  cwh
# Added javascript action to link 'all' buttons on form.
#
# Revision 1.69  2003/06/12 22:09:03  cwh
# Added some explanation to the parameter form interface.
# Now using tickTablestrict to turn on parameter checkboxes.
#
# Revision 1.68  2003/06/09 21:31:58  cwh
# Added a little javascript so that if a user adds a constraint on a field
# on the parameter form, the display checkbox is checked so that the field
# is returned in the results.
#
# Revision 1.67  2003/05/20 18:26:08  cwh
# Added links to help for query terms box.
#
# Revision 1.66  2003/05/20 17:45:58  cwh
# Fixed batch error message.
# Changed instructions on parameter form.
# Changed the parameter form to new format (one entry box).
# Changed the style of the parameter form to the new 'othertable' style.
#
# Revision 1.65  2002/12/16 03:37:03  cwh
# No longer decoding missions, tabletypes.
# Removed extra table and center tags.
# Added missing close bold tags.
# Added labels, ids, and scopes for section 508 compliance.
# Added quotes where needed.
# No longer encoding $stuff
#
# Revision 1.64  2002/09/13 18:53:08  cwh
# Added check for too many processes.
# Isolated form name in variable, passing it to javascript functions.
# Fixed some html.
# Passing along joint good param list.
#
# Revision 1.63  2002/04/11 18:42:57  cwh
# Now uses Parse::set_defaults() and Parse::get_table_list().
#
# Revision 1.62  2002/03/22 23:05:41  cwh
# Changed default # rows returned from 50 to 1000.
# No longer choosing tables to query based on truncating by priority.
#
# Revision 1.61  2002/03/15 23:13:51  cwh
# Added handling for lists of tables specified with '||'s.
# Fixed priority assignment (was assigning 5 if pri=0).
# Added $$in{msg} about the number of tables queried, with a link to query
#   all matching tables, to be printed later.
# Now passing $tablehead to Options::DisplayTables so that the default tables
#   will be pre-selected on the table list form.
#
# Revision 1.60  2002/03/06 22:50:45  cwh
# Fixed bug in quoting url.
# Passing table names to ValidateInput::checkParameter. (Would like to check
#    Entry on parameter form submit for tables that always require position,
#    like Vizier's large tables.  This isn't working yet.)
#
# Revision 1.59  2002/02/12 23:45:53  cwh
# HTML clean-up: Changed all instances of the <font> element to <span>.
#      Added type="text/javascript" to all <script> elements.
# Changed back to PrintParameters2 to stop reordering the catalog columns.  I
#     don't remember why I needed to switch back to PrintParameters().
# Changed '1' to 'undef' in call to Options::DisplayTables().  Caused error.
# Added a default setting for navpast to the main interface page in cases where
#     the $ENV{HTTP_REFERER} doesn't exist.
# Fixed problem of repeating tables when tables are assigned to more than one
#     mission.  Keep track of what has already been seen in a %seen array.
#     Then I have to resort on priority to get them in the right order again.
#
# Revision 1.58  2001/12/17 18:51:33  cwh
# Only suppress '0.0000' for Vizier tables in param form.
#
# Revision 1.57  2001/12/17 18:27:49  cwh
# Now using parameter_format for displaying min/max in parameter form for
# floats and ints.
#
# Revision 1.56  2001/12/05 23:39:54  cwh
# If a batch user asks for a list of all tables, call Display_table_list().
#
# Revision 1.55  2001/11/08 20:15:54  cwh
# Changed 'do w3config.pl' to 'use Config' and AlwaysRun().
# Added Browse:: to all sub calls.
# Added link to wildcard, spaces, and nulls section in help doc.
# Added javascript reset button to each param in param query form.
#
# Revision 1.54  2001/10/25 20:26:54  cwh
# Eliminated WriteHiddenForm, in favor of calling Results::Heraform($input)
#  from Display.  This is because it should not be printed for the FITS output.
#
# Revision 1.53  2001/10/16 20:04:50  cwh
# Allow individual tables to be specified with 'TABLENAME%3DNAME=whatever',
# which is how the old archive search on the left menu bar worked.  Still have
# these floating around (esp on cossc pages), and it's nice to let them work.
#
# Revision 1.52  2001/10/04 22:08:34  cwh
# Fixed small bug in javascript on/off All fields box in parameter form.
# Cleaned up the ParameterHeading() subroutine code.
#
# Revision 1.51  2001/09/24 17:50:34  cwh
# The stuff passed in tablehead was being wiped out and replaced with the
# results of a TableInfo query.  Thus the archive flag was unset, when it
# should have been Y or N as passed in from the w3table.pl form.
#
# Revision 1.50  2001/09/15 02:57:47  cwh
# Modified the form to Cornerstone style.
# Allowing the settings for parameter query to be specified in the query string.
#
# Revision 1.49  2001/08/08 20:48:48  cwh
# Handling Coordinates=[JB]\d+ now.
# Changed 'range' to 'between' and added 'outside', meaning '!range'.
#
# Revision 1.48  2001/07/31 15:17:09  cwh
# Generalized reading of 'Observatory*' variables.
# Got rid of remaining stuff dealing with automatically adding object tables.
#
# Revision 1.47  2001/07/17 18:52:38  cwh
# Changed 'next if /class/' to 'next if /^class$/' - parameters with class in
#   their name were being suppressed, i.e. 'iue_class'.  Meant to only
#   suppress 'class'.
#
# Revision 1.46  2001/07/01 02:11:02  cwh
# *** empty log message ***
#
# Revision 1.45  2001/07/01 01:11:23  cwh
# Change underscores in units to spaces.
# Change 'micro' in units to &micro;.
# When deciding not to display degree units, only withhold them if we're
# printing $table_info{right_ascension} or $table_info{declination}.
#
# Revision 1.44  2001/06/20 22:57:24  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.43  2001/06/18 19:04:28  cwh
# Added a default max rows if one is not set. (=50)
# Removed special handling for 'all vizier tables'.
# Added check so that if 'all vizier tables' is set, don't add more viz tables
#    for each mission if the viz checkbox is on - don't want duplicates.
#
# Revision 1.42  2001/06/14 22:53:59  cwh
# Added support for vizier generic query in Basic searches.
#
# Revision 1.41  2001/06/14 21:33:00  tam
# Changes to support proper handling of VizieR tables in multiple
# querys.
# Kludge to handle tables for which parameter queries are not permitted.
#
# Revision 1.40  2001/06/13 20:40:54  cwh
# oops - left a dumphash in there.
#
# Revision 1.39  2001/06/13 16:22:30  tam
# Fixed bug where tableinfo was being
# called for all tables in a coordinate search
# before beginning the search -- code was
# only there to handle table display page.
#
# Revision 1.38  2001/06/12 16:08:25  tam
# Don't report units for MJD and Degree fields (which are displayed
# differently than they are stored).
#
# Revision 1.37  2001/06/04 21:53:44  cwh
# Added units to param form page.
# Won't display 0, 0 as min and max on Vizier (or any other) param form.
#
# Revision 1.36  2001/06/04 21:24:30  cwh
# Commented out the code which added the multiwavelength (object) catalogs
# which need the object table type, since Object is now checked by default.
#
# Revision 1.35  2001/05/24 16:40:45  cwh
# This mod_perl stuff is killing me.  Got rid of all the BEGIN blocks (those aren't
# called if they've been already done once).  Added the 'use lib "./lib";'s back in.
#
# Revision 1.34  2001/05/21 15:45:36  cwh
# Switched back to original PrintParameters sub.
#
# Revision 1.33  2001/05/17 02:17:07  tam
# Changed test for default parameters.
#
# Revision 1.32  2001/05/03 19:18:15  cwh
# Put call to w3config.pl in a BEGIN statement, pulled all 'use lib's out.
# Now uses global vars defined in w3config.pl wherever possible.
# Added All checkbox to View column on the Parameter Search form, which is only
#     displayed if the user has Javascript enabled.
# Added $joint_param_info->{'#all#'} = [ @good_list ]; for multiple tables
#
# Revision 1.31  2001/04/25 19:31:03  cwh
# - removed sort stuff
# - removed cone query stuff
# - trying PrintParameter2, which doesn't tamper with the order of columns in
#   the parameter search form.
# - various VizieR changes.
#
# Revision 1.30  2001/04/11 01:41:11  cwh
# Brought back handling of compound tables via w3basic.data, since some resources,
# like NED, are using them.
#
# Revision 1.29  2001/04/09 15:37:50  cwh
# - Changed 'Search by Parameter' to 'Parameter Search'.
#
# Revision 1.28  2001/04/03 15:56:36  cwh
# - created action flag to simplify code
# - added max priority truncation
# - added initialization of arrays for mod_perl
# - add on tables from the pseudo-missions that are all object tables
# - get mission metadata for non-mission tables
# - added description of table types to error message when no tables found
# - added tickTable function
# - added sorting of tables so that added tables appear in correct order
# - changed the args given to TableHeadings
#
# Revision 1.27  2001/03/12 16:11:50  cwh
# Modifications for combined interface:
#   W3query now has to read missions as well as tables.
#
#   First it reads in the table types that are selected.  Then it reads in
#     the missions that are selected.  It calls HDB::GetQueryTables to find
#     out which tables in those missions have the selected table types.  Then
#     it adds the individual tables that have been selected, taking care not
#     to add catalogs that were already included in missions earlier.
#
# Revision 1.26  2001/01/22 20:28:23  cwh
# Added a provision for not having a navigation trail as input.
#
# Revision 1.25  2001/01/02 23:37:56  cwh
# Added tabletypes as an input.
#
# Pass $selected flag to Options::Header to flag that only selected table types
#   are being returned.
#
# Added css span info to navtrail.
#
# Pass tabletypes to Options::TableHeadings.
#
# Repeat SubmitButtons() at bottom of form.
#
# Added target=helpwin for all help links.
#
# Revision 1.24  2000/12/04 19:37:53  cwh
# Added navtrail variable which is passed from script to script to give users
# some navigation.
#
# Revision 1.23  2000/03/08 20:04:34  saima
# W3Browse to Browse name change.
#
# Revision 1.22  2000/01/12 18:59:10  saima
# Removed 'UTC' from min and max range of dates.
#
# Revision 1.21  1999/12/20 17:38:43  saima
# Modified PrintHiddenParams() so that the code does not print hidden
#   information about a table.
#
# Revision 1.20  1999/11/04 18:45:42  saima
# Minor fix for tables with no 'heasarc_' prefix.
#
# Revision 1.19  1999/11/01 21:42:01  saima
# Fixed a bug where min/max which had spaces were getting truncated.
#
# Revision 1.18  1999/10/28 17:32:48  saima
# %HDBclass::name variable is used to get class names from class numbers.
#
# Revision 1.17  1999/09/13 18:21:12  saima
# Added help links from parameter options.
#
# Revision 1.16  1999/08/25 21:37:16  saima
# Added &WriteHiddenForm which helps save the form state by the cookie
#   state manager.
#
# Revision 1.15  1999/05/27 15:44:04  tam
# Modified for Apache mod_perl module.
#
# Revision 1.14  1999/03/18 20:37:53  saima
# Added logging capability.
#
# Revision 1.13  1999/02/25 17:30:49  saima
# Fix code so that the sort button stuff is in "" quotes instead of ''.
#
# Revision 1.12  1999/02/24 16:18:34  saima
# Added extra checks for 'text' type parameters. These parameters can
#   not be queried using 'range' and can not be sorted.
#
# Revision 1.11  1999/02/16 22:40:52  saima
# Modified so that JavaScript1.2 is used for 'write' statements.
#
# Revision 1.10  1999/02/08 19:59:20  saima
# Modified SQL query formation so that user can now search for multiple
#   classes.
#
# Revision 1.9  1999/02/03 20:16:33  saima
# Added the functionality for multiple table parameter search whereby,
#   if the tables share a common metabase parameter but they don't have
#   the same table parameter name (for eg start_time can be called time,
#   or start), the metabase parameter still shows up in the list
#   of parameters to set.
#
# Revision 1.8  1999/01/28 22:21:53  saima
# Minor tweaks to HTML code.
#
# Revision 1.7  1999/01/28 21:51:07  saima
# Fixed the html for 'class' settings.
#
# Revision 1.6  1999/01/28 19:52:10  saima
# Added a checkbox button to show/remove min-max ranges.
#
# Revision 1.5  1999/01/26 23:19:57  saima
# Fixed bug so that for multiple tables JavaScript does not look for
#   'varon'.
#
# Revision 1.4  1999/01/26 21:31:28  saima
# Added JavaScript form checking to page. The max values fill with *'s
#   if user selects operator '=' or '!='.
#
# Revision 1.3  1999/01/07 20:41:27  saima
# Modified so that by default, Cone search is called instead of
#   Build Parameter Search page.
#
# Revision 1.2  1998/12/23 20:09:07  saima
# Fixed call to Options::TableHeadings().
#
# Revision 1.1  1998/12/22 20:02:25  saima
# This script decides on the type of query and then calls either Cone
#   search, Parameter search or displays the page where user can
#   set parameter criteria.
#
#=========================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA/Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=========================================================================#
# Module Name:    w3query.pl
# Module Author:  Saima Zobair, Raytheon ITSS
# Date:           October, 98
# Project:        Browse : HEASARC
#=========================================================================#
# Module Description:
#=========================================================================#
$0 = 'w3query.pl'; # Required for MailMessage - do not remove.

use strict;
use lib './lib';

use Browse::Config;
use Browse::CGI;
use Browse::Query;

#------------------------------------------------------------------------
$| = 1; # Make STDOUT unbuffered.
#$::debug=1;
print Browse::CGI::Header,"w3query.pl: DEBUG<br>\n" if $::debug;

my($in,$input) = Browse::CGI::ReadParse(1);

Browse::Query::process($in,$input);
1;
#--------------------------------------------------------------------------
#####
#EOF#
##
