#!/usr1/local/bin/perl5 -T
#
# $Id: w3table.pl,v 1.54 2008/02/20 16:01:50 lmm Exp $
# $Log: w3table.pl,v $
# Revision 1.54  2008/02/20 16:01:50  lmm
# Added missing parameter separator '&' to query string building process.
#
# Revision 1.53  2006/11/29 15:37:10  lmm
# Changes to accommodate replacement of w3query.pl with Query.pm
#
# Revision 1.52  2005/12/13 16:43:10  lmm
# Parameter page is now immediately displayed when "Detailed
#   Mission/Catalog Search" button is selected and there is only one
#   mission table
#
# Revision 1.51  2005/03/30 13:59:41  lmm
# Query string added for GET environment.  This seems to be required
#   when db-perl, not cgi-bin, is in the referring URL.
#
# Revision 1.50  2005/02/23 18:46:56  lmm
# Master table query section update for Public release and some cosmetic
#   changes
#
# Revision 1.49  2004/09/16 17:55:34  lmm
# Added temporary input variable to check if HEASARC user wants to
#   see the alternate DisplayTableHits format
#
# Revision 1.48  2004/09/14 18:54:25  lmm
# Changes for addition of Master Table Query display
# - Moved creation of CGI query string out of POST only block
# - If an object is specified but not a mission/catalog the DisplayTableHits
#   display routine is called
# Included check for new Parameter Search Button name to determine search
#   type
# Made some cosmetic changes (removed old commented-out code)
# Removed parenthesis () from MissionTables subroutine declaration due to
#   error messages
#
# Revision 1.47  2004/05/27 17:58:17  lmm
# Fixed bug introduced when POST queries were added that resulted in
#   an empty http query string.
#
# Revision 1.46  2004/01/15 18:20:31  lmm
# Replaced "Location:" redirection with call to new w3query subroutine
#   due to problems handling the upload of large object files.
#
# Revision 1.45  2003/12/02 22:16:37  lmm
# Changes to allow CI POST method
#
# Revision 1.44  2003/08/28 20:39:44  lmm
# Defined header text for next page and removed HEASARC footer
#
# Revision 1.43  2003/04/28 02:38:49  cwh
# Added a vertical blank space.
#
# Revision 1.42  2002/12/16 03:30:58  cwh
# Added quotes where needed.
# No longer decoding tablehead.
# Added labels and ids for section 508 compliance.
# No longer encodeing mission or table_type in hidden params.
#
# Revision 1.41  2002/09/13 19:04:08  cwh
# Added redirects to cgi-bin if ExcelDisplay is selected (Excel display
# doesn't yet work with mod_perl).
# Isolated form name in var, passing it to javascript functions.
#
# Revision 1.40  2002/05/14 18:14:20  cwh
# Cleaned up some references. (Was doing refs to deref'd refs!!)
#
# Revision 1.39  2002/04/11 18:46:01  cwh
# Now uses Parse::set_defaults() and get_table_list().
#
# Revision 1.38  2002/03/22 23:07:45  cwh
# No longer getting default tables by priority.
#
# Revision 1.37  2002/03/07 23:21:59  cwh
# Removed duplicate code.
# Fixed bug where archive status was not correctly being reported for tables
#   chosen individually.
#
# Revision 1.36  2002/02/14 17:40:38  cwh
# Eliminated duplicates caused when a table belongs to more than one mission.
# HTML cleanup: changed all <font> elements to <span> elements.
#   Fixed link to Observation Date Query section of the help doc.
# Get default tables based on priority and pass them along to
#   Options::DisplayTables to that the default tables are pre-checked in the
#   table list form.
#
# Revision 1.35  2001/11/08 20:11:48  cwh
# Changed 'do w3config.pl' to 'use Config' and AlwaysRun().
# Added Browse:: to all sub calls.
#
# Revision 1.34  2001/10/25 22:18:15  cwh
# Added a legend line for the sort arrows.
#
# Revision 1.33  2001/10/25 20:28:35  cwh
# Made the question numbering dynamic, since if there is only one table, the
#   user doesn't need to choose one.
#
# Revision 1.32  2001/10/16 20:07:27  cwh
# Changed the label for main search form in the navtrail from
# 'Choose Missions' to 'Main Search Form', since the main form is no
# longer just a mission selection form.
#
# Revision 1.31  2001/09/15 02:49:22  cwh
# Modified form to Cornerstone style.
#
# Revision 1.30  2001/08/08 20:50:04  cwh
# Handling Coordinates=[JB]\d+ now.
#
# Revision 1.29  2001/07/31 15:16:09  cwh
# Changed the logic of the redirect to to w3query.pl.
# Generalized the reading of the 'Observatory*' variables.
# Sending $in to OptionBar to preserve state.
# Got rid of remaining stuff dealing with automatically adding object tables.
#
# Revision 1.28  2001/06/20 22:57:24  cwh
# Got rid of $::writeHeader in calls to ::Error.
#
# Revision 1.27  2001/06/18 19:05:25  cwh
# Added exclusions to lowercase and adding $::local_origin to tables names -
#   don't do it for Vizier tables.
#
# Revision 1.26  2001/06/04 21:23:22  cwh
# Commented out the adding of multiwavelength (object) catalogs that require
# the 'Object' table type, since Object is now checked by default.
#
# Revision 1.25  2001/06/04 15:05:26  cwh
# - if there is a file called intro.html in the mission help dir, then the
# contents of that file are printed at the top of a MissionHelp page.
# - commented out the code that removed any instances of <pre> in the overview
# sections of the table help files.
#
# Revision 1.24  2001/05/24 16:40:45  cwh
# This mod_perl stuff is killing me.  Got rid of all the BEGIN blocks (those aren't
# called if they've been already done once).  Added the 'use lib "./lib";'s back in.
#
# Revision 1.23  2001/05/23 15:27:58  cwh
# Added require of w3lib.pl
# Fixed bug where the string "Search+by+Parameter" should have been changed to
#     "Parameter+Search" - this was preventing the forwarding to w3query when one
#     single table and Adv Search were selected.
# Trim $::local_origin off list of tables with datasets, add  $::local_origin back
# to single tables later.  This was preventing single tables from showing up at all
# in the table listing.
#
# Revision 1.22  2001/05/17 15:35:05  cwh
# Don't add $::local_origin
#
# Revision 1.21  2001/05/03 19:19:23  cwh
# Put call to w3config.pl in a BEGIN statement, pulled all 'use lib's out.
# Changed heasarc_ to $::local_origin.
#
# Revision 1.20  2001/04/25 19:33:06  cwh
# aded VizieR stuff
#
# Revision 1.19  2001/04/09 15:32:19  cwh
# - Changed the labels on the buttons: 'Submit Coordinate Search' -> 'Coordinate
# Search', 'Search by Parameter' -> 'Parameter Search'
# - Pulled the buttons and the entry box in one table cell like on main interface.
# - Moved Reset button in between the two submit buttons like on main interface.
#
# Revision 1.18  2001/04/03 16:12:22  cwh
# Added Basic=on to query string when a basic search is detected
#
# Revision 1.17  2001/04/03 15:49:22  cwh
# - added no Action field as a condition which redirects to w3query (this is if
#   the user just presses 'Enter' after typing an object name/coords)
# - added max priority support (currently not used)
# - fixed error: Observatory_objects -> Observatory_object
# - added elimination of duplicate missions
# - added initializations of arrays for mod_perl
# - add on tables from the pseudo-missions that are all object tables
# - pulled all the transforming table meta info to mission meta info into a
#   subroutine: HDB::get_table_meta($_)
# - changed the error message when no tables found to describe the table types.
# - send list of object missions to MissionTables for use in sorting
# - moved sortTables out of this script and into the Sort.pm module.
# - in MissionTables, added objmissions and maxpriority to sort spec.
#
# Revision 1.16  2001/03/12 16:27:04  cwh
# Modifications for combined interface:
#  - Redirects to w3query if Basic Search button is clicked.
#  - Missions now have several different checkbox names that need to all be read
#  - Add any catalogs that have been selected to the tables to be displayed.
#  - Redirect to w3query.pl for Parameter query if only one table has been
#    selected.
#  - Get metadata for catalogs so they can be displayed in the table with others.
#  - Added Tom's new sort routine so we can sort tables without querying the
#    database.
#  - Changed how Options::TableHeadings is called so we can do this sorting.
#
# Revision 1.15  2001/01/22 20:20:04  cwh
# Added support for MissionHelp=tables%3Dtable1||table2 - this is needed to
# allow help when w3basic.pl is changed to a format like w3browse.pl.
#
# Revision 1.14  2001/01/02 23:13:21  cwh
# Added table types to input vars to get.
#
# Pass flag to Options::Header to let user know that a subset of tables is being
#   retrieved, based on table types selected.
#
# Pass table types to Options::TableHeadings and to HDB::GetQueryTables.
#
# Added css span info to navtrail.
#
# Repeat SubmitButtons() at bottom of table.
#
# Put table types selected in hidden info.
#
# Revision 1.13  2000/12/04 19:37:53  cwh
# Added navtrail variable which is passed from script to script to give users
# some navigation.
#
# Revision 1.12  2000/04/25 18:46:01  saima
# Added a parameter to be passed into the tickTable() JavaScript routine.
#
# Revision 1.11  2000/03/08 20:06:32  saima
# W3Browse to Browse name change.
#
# Revision 1.10  1999/09/13 18:21:59  saima
# Added more links to the w3browse help pages.
#
# Revision 1.9  1999/08/26 17:17:22  saima
# Minor tweaks.
#
# Revision 1.8  1999/08/25 21:38:26  saima
# Removed minor comments.
#
# Revision 1.7  1999/05/27 15:46:54  tam
# Modified for Apache mod_perl module.
#
# Revision 1.6  1999/03/18 20:38:34  saima
# Added logging capability.
#
# Revision 1.5  1999/02/22 18:52:55  saima
# The number of mission tables is passed to checkTablePage() to make
#   sure that JavaScript checks are not added if only one table is available.
#
# Revision 1.4  1999/02/16 22:41:07  saima
# Modified so that JavaScript1.2 is used for 'write' statements.
#
# Revision 1.3  1999/02/03 20:17:49  saima
# Added JavaScript to the "Search on all tables" checkbox.
#
# Revision 1.2  1999/01/26 21:33:59  saima
# Added JavaScript form checking to page. Makes sure that a value for
#   name/coordinate is entered if user clicks on
#   'Submit Coordinate Search' button.
#
# Revision 1.1  1998/12/22 20:03:38  saima
# Initial Revision.
#
#=========================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA/Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=========================================================================#
# Module Name:    w3table.pl
# Module Author:  Saima Zobair, Raytheon ITSS
# Date:           October, 98
# Project:        Browse : HEASARC
#=========================================================================#
# Module Description:
#  Creates form which allows users to view tables from selected 
#  missions. This script also provides sorting capability for table 
#  options.
# 
#=========================================================================#
$0 = 'w3table.pl'; # Required for MailMessage - do not remove.

use strict;
use lib './lib'; 
#use lib qw(. ./lib);

use Browse::Config;
use Browse::Log;
use Browse::Options;
use Browse::Sort;
use Browse::ValidateInput;
use Browse::DisplayTableHits;
use Browse::Parse;
use Browse::Query;
#--------------------------------------------------------------------------
Browse::Config::AlwaysRun();
#$::debug = 1;

print Browse::CGI::Header,"w3table: DEBUG<br>\n" if $::debug;
print "<pre>QueryString in raw: $ENV{'QUERY_STRING'}\n\n</pre>\n" if $::debug;

 #-- commented out for now because POSTS are now allowed
 #Browse::Utils::Error("Error: Invalid usage. No mission specified. " .
        #"Please go back to the mission-selection page to start a new query.")
   #unless (($ENV{'REQUEST_METHOD'} eq 'GET') && $ENV{'QUERY_STRING'});
 
 #--- Now accepting POST for file upload  so just check empty string for GETs
 #Browse::Utils::Error("Error: Invalid usage. No mission specified. " .
        #"Please go back to the mission-selection page to start a New query.")
        #if (($ENV{'REQUEST_METHOD'} eq 'GET') && !$ENV{'QUERY_STRING'});
 
 


my($in) = Browse::CGI::ReadParse();  # Read and parse the query string

$in = Browse::Parse::set_defaults($in);

if ($in->{'MissionHelp'}) { # Mission link to help!
  MissionHelp($in->{'MissionHelp'});
  Browse::Log::TableQuery($0,$in,'Help Mission'); 
  exit;
}

#--- Check if any missions are selected
#--- Most queries will be posts so go ahead and try to create query string
my($querystring)="";
my ($key);   
my $missionFound=0;
foreach $key (keys %$in) {
   if ($key=~ /^Observatory/i || $key=~ /^tablehead/i) { $missionFound = 1;}
   if ($querystring eq "") {
         $querystring .= "$key=" . Browse::CGI::Encode($$in{$key});
   } else {
         $querystring .= "&$key=" . Browse::CGI::Encode($$in{$key});
   }
}
print "QSTRING=$querystring<br>\n" if $::debug;

#--------------------------------------------------------------
#--- Master table query
#-- If only  coordinates/object have been entered branch
#-- off to Table/Hits Display  
if (!$missionFound && ($in->{'Entry'} || $in->{'Time'}) && 
    $in->{'Radius'} && $in->{'Coordinates'}) {
  
  my $mode;

  $mode="DisplayTableHits";
  my $disp= new Browse::DisplayTableHits($mode);


  $disp->display($in);
  exit;
}

#----------------------------------------------------------------
# redirect to w3query if Basic search
   print "req " .$ENV{'REQUEST_METHOD'} . "<br>" if $::debug;

if ($in->{'Action'}!~ /Advanced Search|Detailed|More Options/i || !$in->{'Action'}) {
  print "w3table::Normal Search....<br>" if $::debug;

  # Define header title for new page
  my($popupRef)="Main Query Results";


   # Build query string if POST. Redirect if no additional options needed
   if ($ENV{'REQUEST_METHOD'} eq "POST") {

      $querystring .= "&Basic=on";  
      $querystring .= "&popupFrom=$popupRef" 
	unless ($querystring =~ /ExcelDisplay/); #lmm
      
      $ENV{REQUEST_METHOD} = "GET";
      $ENV{QUERY_STRING}  = $querystring;
      $ENV{CONTENT_TYPE}  = "";

      print "***calling w3query with $querystring\n" if $::debug;

      #require "./w3query.pl";
      #w3query();
      Browse::Query::preprocess($querystring);


   } elsif ($ENV{'REQUEST_METHOD'} eq "GET") {

      $ENV{REQUEST_METHOD} = "GET";
      $ENV{CONTENT_TYPE}  = "";
      print "***calling w3query\n" if $::debug;
      #require "./w3query.pl";
      #w3query();
      Browse::Query::preprocess($querystring);

  }

  print "***exiting w3table<br>\n" if $::debug;
  exit;
}

#--- non-Basic search 
print "*** w3table: this is a non-basic search<br>\n" if $::debug;

(my $tablehead, my $missions, my $tabletypes, $in, my $default_tables) = 
	Browse::Parse::get_table_list($in);

print "w3table::number of missions=" .scalar(@{$missions}) . " number of tables=" .scalar(@{$tablehead}) . 
	" query from environment=" . $ENV{'QUERY_STRING'} ."<br>" if $::debug;

if (scalar(@{$missions}) <=1 and scalar(@{$tablehead}) ==1) {

   $querystring  .= "&Action=Parameter+Search\n\n";

   $ENV{REQUEST_METHOD} = "GET";
   #--- Update query input.  Be sure to add parameter separator
   if ($ENV{QUERY_STRING} =~ /\w/) { $ENV{QUERY_STRING} .=  '&';} 
   $ENV{QUERY_STRING}   .=  $querystring;
   $ENV{CONTENT_TYPE} = "";
   print "no missions OR 1 table<br>" if $::debug;
   Browse::Query::preprocess($querystring);

}

my $selected = "selected" if $#{$tabletypes} < 3;
my $other = "and object " if scalar(@{$tablehead});


Browse::Options::Header( $missions, $selected, $other);
my $formname="w3query";
print "\n<form method=POST name=\"$formname\" " .
	"enctype=\"multipart/form-data\" action=\"w3query.pl\">\n";

my $navtrail=$in->{'navtrail'};
$navtrail = "<a class=navpast href=w3browse.pl>Main Search Form</a>" 
	unless $navtrail;
print "$navtrail &gt; <span class=navcurrent> Search Form</span> ";
print "&gt; <span class=navfuture> Search Results &gt; ";
print "Choose Data Products</span><br>\n";

Browse::ValidateInput::checkTablePage($formname,scalar @{$tablehead});
my $num = 0;

if (scalar @{$tablehead} >1) {
    print "<p><b>",++$num,". Please select one or more of the tables below.</b>\n" ;
} else {
    print "<p><b>One table selected.</b>\n";
}

print "<br><span class=\"small\">" .
	"<img src = \"$::images/down.gif\" height=15 width=15 alt=\"Sort\">" .
	" Sort by a column in order: 1,2,3</span> <span class=\"small\">" .
	"<img src = \"$::images/up.gif\" height=15 width=15 " .
	"alt=\"Reverse Sort\"> Sort by column in reverse order: 3,2,1</span>";

MissionTables($tablehead, $missions, $in, $tabletypes, $$in{"vizier_query"},$default_tables);
HiddenInfo($missions, $tabletypes);
$navtrail .= " &gt; <a class=navpast href=".
	$ENV{SCRIPT_NAME}.'?'.$ENV{QUERY_STRING}.">Search Form</a>";
$navtrail =~ s/\</&lt;/g;
$navtrail =~ s/\>/&gt;/g;
print "<input type='hidden' name='navtrail' value=\"$navtrail\">";

my($popupRef) = "Query Results";  #lmm
print "<input type='hidden' name='popupFrom' value=\"$popupRef\">";  #lmm


print "<p><b>",++$num,". Do you want to change any of your current query selections?</b><br>\n";
  Browse::Options::OptionBar($in);
  NewSubmitButtons(++$num);
print "</table></form><br>";

Browse::Decor::EmailFooter();
Browse::Decor::EndTags();

Browse::Log::TableQuery($0,$in,'Display Tables', $missions); 

#--------------------------------------------------------------------------

#sub MissionTables() {
sub MissionTables {
   
  my($tableheader,$missions,$in, $tabletypes, $vizier,$default_tables) = @_;

  print "tableheader -- @$tableheader[0] <br>" if $::debug;

  my $spec;
  my $mission;
  my $amp = "";
  foreach $mission (@$missions) {
      my $p = $mission;
      $p =~ tr/ /+/;
      $spec .= $amp."Observatory=$p";
      $amp = "&";

  }
  my $type;
  foreach $type(@$tabletypes) {
      $spec .= $amp . "table_type=$type";
      $amp = "&";
  }
  if (defined($vizier) ) {
      $spec .= $amp . "vizier_query=$vizier";
      $amp = "&";
  }
  
  my $table;
  #foreach $table (split("\0", Browse::CGI::Decode($in->{'tablehead'}))) {
  foreach $table (split("\0", $in->{'tablehead'})) {
      $spec .= $amp . "tablehead=$table";
      $amp = "&";
  }

  $spec .= "&"."maxpriority=$in->{'maxpriority'}";

  Browse::Options::TableHeadings($spec, $tableheader, 1);
  Browse::Options::DisplayTables($tableheader,$default_tables);
  print "</table></center><p>\n";

  my $compTable;
  my $xtab;
  foreach $xtab(@$tableheader) {
      if ($$xtab{"name"} =~ /^Z\//) {
          $compTable = 1;
          last;
      }
  }

  print "</table></table></center>\n";

  if ($compTable) {
      $$in{"ExpVizier"} = 1;
      my $str = Browse::CGI::Enform($in);
      my $url = "w3table.pl?$str";
      print "<a href='$url'> Display with explicit VizieR journal tables </a><br>\n";
  }

} # eosub--MissionTables
#--------------------------------------------------------------------------
sub SubmitSection {
my ($entry,$time) = @_;

print "<td valign=TOP>\n <table >\n";
SubmitButtons();
  #--- HTML-encode  non-words 
  $entry =~ s/\W/"&#".ord($&).";"/ge;
  print <<QUIT;
<br><label for="Entry"><a target="helpwin" href="$::w3bhelp#Coordinates_Name_Search"><b>Object Name or Coordinates</b></a></label><br>
<input Name="Entry" id="Entry" value="$entry" size="30">
<br><label for="Time"><b><a target="helpwin" href="$::w3bhelp#date_query">Dates:</a></b></label>
<input Name="Time" id="Time" value="$time" size=30>
</td>
</tr>

QUIT

} # eosub--SubmitSection
#--------------------------------------------------------------------------
sub SubmitButtons {

  print <<QUIT2;
<tr>
<td colspan=7 align=center><b>Select one or more catalogs.  Then either enter an object name or coordinates and select 'Coordinate Search', or select 'Parameter Search' to continue to a form where you can query by any table parameter.</b><br>
   <span class="large"> 
   <input type="submit" name="Action" value="Coordinate Search" onClick="return(pageCheck('on'))">&nbsp;<input type="reset" value="Reset">&nbsp;<input type="submit" name="Action" value="Parameter Search" onClick="return(pageCheck('off'))">
   </span> 

QUIT2
} #eosub--SubmitButtons

sub NewSubmitButtons {
my ($num) = @_;
  print <<QUIT2;
   <p><b>$num. <b>
   <input type="submit" name="Action" value="Submit Search" onClick="return(pageCheck('on'))">&nbsp;<input type="reset" value="Reset">&nbsp;<input type="submit" name="Action" value="Specify Additional Parameters" onClick="return(pageCheck('off'))">

QUIT2
} #eosub--SubmitButtons
#--------------------------------------------------------------------------

sub MissionHelp {
  my($obs)   = @_;
  my($ok) = 0;
  local(*FILE);
  my ($dir_obs,@some,$label)=0;
  
  if ($obs =~ /^tables=/i) {  # tables were specified

     $obs =~ s/^tables=//i;
     (@some) = split(/\|\|/,$obs);
     foreach (@some) {$_ = $::local_origin . lc($_);}
     $dir_obs = "all";
     $label="selected";
  
  } else {                    # missions were specified

    # replace _ with spaces for observatory names in database
    my $db_obs = $obs;
    $db_obs     =~ s/\_/ /g;

    # replace _ with - for references in directory structure
    $dir_obs = $obs;
    $dir_obs     =~ s/\_/\-/g;
    $label=$dir_obs;
  
    # get tables for observatory and if the tablename exists in the database
    # and in help files then display the overview information
    @some = Browse::HDB::TableObs($db_obs);

  } # end if missions or tables specified

  my($dir) = "$::helpdir/$dir_obs";
  if(! opendir HELPDIR,"$dir"){
     warn "Browse problem: Unable to open help directory $dir.\n";
     Browse::Utils::Error("Error unable to open help file directory!");
  }
  my(@files) = grep !/^\.\.?$/, readdir HELPDIR; #remove .. and .

  Browse::Decor::Header("List of \U$label\E Catalogs");

 if (open (INTRO, "$::helpdir/$dir_obs/intro.html")) {
    while (<INTRO>) {print};
    close(INTRO);
 }


  print "tables=@some\n<br>\n" if $::debug;

  foreach(@some){
    if (/^$::local_origin/){
      my($file) = $';
      my($htmlfile) = $' . ".html";
      print "w3table: file = $file  $htmlfile<br>" if $::debug;
      if(Browse::Utils::memq($htmlfile, @files)){
        my($fullfile) = "$::helpdir/$dir_obs/" . $htmlfile;
        print "<p><strong> " .
	 "<a href=\"$::w3bdocs/$dir_obs/$htmlfile\">\U$file\E Catalog</a>" .
	 "</strong><p>";
        open FILE, "<$fullfile" or Browse::Utils::Error("File not opened!");
        while (<FILE>){
          #$_ =~ s/\<pre\>//g;
          #$_ =~ s/\<\\pre\>//g;
          if (/Overview/i){ $ok=1; next;}
          if(/\<hr\>/i && $ok){ $ok=0; last;};
          print "$_" if ($ok);
        }
      }
      print "<p>";
    }
  }
  #Browse::Decor::Footer();
  Browse::Decor::EmailFooter();
  Browse::Decor::EndTags();

} #eosub--MissionHelp
#------------------------------------------------------------------------
sub HiddenInfo {
  my($missions,$tabletypes) = @_;

  foreach (@ {$missions} ) {
    #print "<input type='hidden' name='mission' value='".Browse::CGI::Encode($_)."'>\n";
    print "<input type='hidden' name='mission' value='$_'>\n";
  }
  foreach (@ {$tabletypes} ) {
    #print "<input type='hidden' name='table_type' value='".Browse::CGI::Encode($_)."'>\n";
    print "<input type='hidden' name='table_type' value='$_'>\n";
  }

} #eosub--HiddenInfo
#------------------------------------------------------------------------
#########
#  EOF  #
#########
