#!/usr1/local/bin/perl5 -T 
# $Id: xteTar.pl,v 1.7 2007/10/03 15:13:03 lmm Exp $
# See the end of this file for cvs logs.

use strict;
use lib './lib';
use Browse::Utils;
use CGI::Carp qw(fatalsToBrowser);

#
# Create a TAR file included selected elements
# of a specified XTE observation or observations.
#

use Browse::Config;
use DateTime;
use File::Find;

Browse::Config::AlwaysRun();
$| = 1;

#$::debug=1;
#-- This seems to prevent an "bad header" error.
print "Content-type: text/plain\n\n"; 

# These are the files that belong in the intermediate products.
my @interBases = (
    "FMI",
    "FIST*",
    "FIPC*",
    "FIHX*", 
    "FIAC*", 
    "FIOE*",
    "acs/*", 
    "hexte/FS5[28]*",
    "hexte/FH*",
    "orbit/*",
    "pca/FS4[6a]*",
    "pca/FH*",
    "stdprod/*.xfl.gz"
);

# These are the files that belong in the full products
#  (i.e., the entire directory!)
my @fullBases = (
    ""
);

# These are the files that belong in the standard products
my @stdBases  = (
    "stdprod/"
);
 
		 
$::tarSize = 0;
$::block;

my($input) = Browse::CGI::ReadParse();  # Read and parse the query string

#if ($::debug) {
#print "DUMP      <br>" if $::debug;
   #Browse::Utils::DumpRef($input);
#}

if (!(defined($$input{prnb}) && defined($$input{tar_no})) &&
     !defined($$input{obsid})) {
    #print "exiting due to insufficient input!\n" if $::debug;
    exit;
}

my $listonly = $$input{listonly};
my $tarFile  = $$input{tar};
my $filelist = $$input{flist};
my @pathlist;
my %tarhash;
if ($filelist) {
   %tarhash={};
   @pathlist = split(",", $filelist);
    for (@pathlist) {
       $_ =~ s/\/$//;
       $tarhash{$_} = 1;
    }
}

print "list length = ".scalar(@pathlist) ."<br>\n" if $::debug;

if (defined($tarFile)) {
  $tarFile =~ s/\.\.\///g;
  ($tarFile) = $tarFile =~ /([\w\.\-\/]*)/; # untaint
    print "tarFile=$tarFile<br>\n" if $::debug;
}
    

my $prnb   = uc($$input{prnb}) if defined($$input{prnb});
my $tar_no = uc($$input{tar_no}) if defined($$input{tar_no});
my $gain;
if (defined($$input{gain})) {
    $gain = int($$input{gain});
}
my $obsid  = uc($$input{obsid}) if defined($$input{obsid});

# Find which product is to be used.
# Default to the intermediate products.
my $bases = \@interBases;
if (defined($$input{set})) {
    my $set = $$input{set};
    if ($set eq "full") {
        $bases = \@fullBases;
    } elsif ($set eq "stdprods") {
        $bases = \@stdBases;
    }
}

# Find the time interval corresponding to the given gain.
my ($minTime, $maxTime) = timeOf($gain);


my $sql;
if (defined($obsid)) {

    my $op = '=';
    if ($obsid =~ /[\*\%]/) {
        $obsid =~ s/\*/\%/g;
	$op = 'like';
    }
        
    $sql = "select prnb, target_number, cycle, obsid from heasarc_xtepublic ".
           "   where obsid $op '$obsid'";

} else {	

    $sql = "select prnb, target_number, cycle, obsid from heasarc_xtepublic ".
           "   where prnb=$prnb and target_number=$tar_no and ".
	   "   start_time >= $minTime and start_time < $maxTime order by obsid";
	  
}

my ($obsids) = Browse::DBQ::query($sql);

my $row;

# If the user does not define the TAR file that we are writing
# to we want to write the output to standard out.  First we
# write a MIME prefix to indicate the file type and name.

if (!defined($tarFile) and !defined($listonly)) {

    my $file;
    if (defined($obsid)) {
    
        $file = $obsid;
	$file =~ s/\-*[\%\*].*//;
	
    } else {
    
        $file = "$prnb-$tar_no";
    }
     
    # Need to specify header for standard output.
    print<<EOT;
Content-type: application/x-tar
Content-disposition: attachment; filename="xte-$file.tar"

EOT
}

if (defined($listonly)) {
   print "Content-type: text/plain\n\n";
}

# Process each obsid individually

print " before file processing<br>\n" if $::debug;
foreach $row(@$obsids) {

    processFiles($tarFile, $row, $bases);
}
print "after file processing<br>\n" if $::debug;


# A TAR file ends with a lot of nulls which indicate the end.
# There are at least 1024 nulls, and then as many more as are
# needed to make the file a multiple of 10240 bytes.

if (!defined($tarFile) and !defined($listonly)) {

    # print out some nulls to indicate the end of the TAR file
    print chr(0) x 1024;
    $::tarSize += 1024;
    my $mod = $::tarSize % 10240;
    if ($mod > 0) {
        print chr(0) x (10240-$mod);
    }
    
    $::tarSize += (10240-$mod);
}
        
exit;

# Process the data from a single obsid.

sub processFiles {

    my ($tarFile, $row, $bases) = @_;
    
    my ($prnb, $tar_no, $cycle, $obsid) = @$row;
    $prnb = sprintf("P%05d",$prnb);
    print "I see $prnb<br>\n" if $::debug;
    
    my $dir = sprintf("/FTP/xte/data/archive/AO%d/", $cycle);
    

    # First move to the appropriate directory to minimize the
    # need to write directory paths
    chdir ($dir);

    my $cmd;
    if (defined($tarFile)) {
        $cmd = "$::tar -rhf $tarFile ";
    } elsif (defined($listonly)) {
        my %files = ();
        foreach my $mdir (@$bases) {
          $mdir =~ s/\s//g;  # get rid of any extra whitespace
          $mdir =~ s/([^\\])\./$1\\\./g; # change '.' to '\./
          $mdir =~ s/([^.])\*/$1.*/;     # change '*' to '.*'
          File::Find::find(
             {wanted  => sub{print "$dir/$File::Find::name\n" 
                              if $File::Find::name =~ /$mdir/ and ! -d },
             untaint => 1,
             follow => 1,
             },"$prnb/$obsid/");
        }
       return; 

    } else {
        $cmd = "$::tar -chf - ";
    }
    
    
    # Add in each potential file to the tar command
    # If filelist has been supplied exclude files already
    # in list 
    my $file;
    foreach $file (@$bases) {
        #print "checking $dir/$prnb/$obsid/$file<br>\n" if $::debug;
       if (!scalar(@pathlist) || 
	   !Browse::Utils::isDuplicate("$dir/$prnb/$obsid/$file",\%tarhash)) {
        print "adding $dir/$prnb/$obsid/$file<br>\n" if $::debug;
        $cmd .= " $prnb/$obsid/$file";
       } 
    }
    
    # Just run the tar command directly if writing to a file.
    # Otherwise we need to manage the tar writing a little more
    # carefully.
    
    if (defined($tarFile)) {
        print "running $cmd<br>\n" if $::debug;
        `$cmd`;
    } else {
        pushTar($cmd);
    }
}		 

# Push the TAR files contents for a given command onto STDOUT
sub pushTar {

    my ($cmd) = @_;
    
    $cmd .= "|";
    
    open(TAR, $cmd);
    
    # Just loop until we don't get a file.
    
    while (getFile()){};
    
    close(TAR);
}

# Push a single file onto STDOUT
sub getFile {


    # First read the header.  The tar file
    # should end with a couple of null blocks,
    # so if there is no file name, then return 0.
    # Otherwise print the header and copy the data
    # to standard out (if there is any data).
    
    my $size = readBlock(512);
    
    if ($size == 0) {
        return 0;
    } elsif ($size != 512) {
        die("Incomplete header block");
    }
    
    if (substr($::block, 0,1) eq "\0") {
        return 0;
    }
    
    my $sizeStr = substr($::block, 124, 12);

    # The file size is stored as an octal number
    my $fileSize = oct($sizeStr);
    
    # Each file is padded to the next 512 byte boundary.
    if ($fileSize % 512 != 0) {
        $fileSize += (512 - $fileSize%512)
    }
    
    # Print the header block
    print $::block;
    $::tarSize += 512;
    
    my $need = $fileSize;
    
    # Now read the data in chunks.
    while ($need > 0) {
        my $chunk = $need;
	if ($chunk > 32768) {
	    $chunk = 32768;
	}
	my $got = readBlock($chunk);
	if ($got != $chunk) {
	    die("Truncated file");
	}
	
	print $::block;
	$need      -= $got;
	$::tarSize += $got;
    }
    return 1;
    
}

sub readBlock {

    my ($need) = @_;
    
    my $have = 0;
    
    while ($need > 0) {
        my $got = sysread(TAR, $::block, $need, $have);
	if ($got <= 0) {
	    last;
	}
        $have += $got;
	$need -= $got;
    }
    return $have;
}
    
    
    
sub timeOf {

    my ($gain) = @_;
    my ($minTime, $maxTime);
    
    if (!defined($gain)) {
        return (0,1.e10);
    }
    
    
    # Dates at which gain has changed.
    my @dates = ("1996-03-21 18:33:39",
                 "1996-04-15 23:05:59",
                 "1999-03-22 17:38:03",
	         "2000-05-13 00:00:00");
    my @mjds;
    for (my $i=0; $i<= $#dates; $i += 1) {
        $mjds[$i] = DateTime::iso2mjd($dates[$i]);
    }
    
		 
    if ($gain <= 1) {
        $minTime = 0;
	$maxTime = $mjds[0];
    } elsif ($gain > $#mjds+1) {
        $minTime = $mjds[$#mjds];
        $maxTime = 1.e10;
    } else {
        $minTime = $mjds[$gain-2];
	$maxTime = $mjds[$gain-1];
    }
    
    return ($minTime, $maxTime);
}


__END__

# $Log: xteTar.pl,v $
# Revision 1.7  2007/10/03 15:13:03  lmm
# Changed call to updated isDuplicate().
#
# Revision 1.6  2005/05/16 15:31:40  lmm
# Local files are not included in tar file if they are located in a
#   directory included in the data products file list.  This does not
#   apply to the listonly option.
#
# Revision 1.5  2003/03/07 18:48:20  cwh
# Added list-only mode, so that if a list is requested, just returns a list of
# the files that would have been tarred - does not make the tarfile.
#
# Revision 1.4  2002/12/10 21:42:06  cwh
# Only uppercase an input if it is already defined.
#
# Revision 1.3  2002/11/19 20:37:52  cwh
# Now uppercasing obsid/proposal number/target number.  There were some entries
# in zzdp where the obsid was set to #####-##-##-##c, with a lowercase char at
# the end instead of an uppercase, which caused the query to fail.
#
# Revision 1.2  2002/10/23 16:43:09  cwh
# Needed to untaint the tarfile name.
#
# Revision 1.1  2002/10/21 19:14:08  cwh
# Initial commit for xteTar.pl.
#
