###############################################################################
#
# MODULE: $Id: CGI.pm,v 1.10 2001/05/11 21:44:43 sabol Exp $
#
#  AUTHOR: Edward J. Sabol <sabol@alderaan.gsfc.nasa.gov>
# CREATED: 1996/09/20
#
# DESCRIPTION:
#  Constructs buttons and actions for use in CGI scripting.
#  Also, other useful functions for CGI scripting.
#
# HISTORY:
# $Log: CGI.pm,v $
# Revision 1.10  2001/05/11 21:44:43  sabol
# Imported into HEASARC CVS repository.
#
# Revision 1.9  1998/04/14 14:38:04  sabol
# Modified output of Error() when the CGI HTML header has already been
#   printed.
#
# Revision 1.8  1998/04/09 18:12:56  sabol
# Added Error() and ContentType() routines and the $header_has_been_printed
#   package-wide variable. Fixed typo in CGI::BrowserSupport::Tables().
#
# Revision 1.7  1997/07/30 15:33:24  sabol
# Eliminated dependency on smartbrowser.pl by implementing similar
#   functionality in the CGI::BrowserSupport sub-package.
#
# Revision 1.5  1997/03/21 23:29:02  sabol
# Added SimpleSubmit() and PrintVariables() functions. The latter is based on
#   a function from Steven Brenner's cgi-lib.pl.
#
# Revision 1.4  1996/12/19 22:25:04  sabol
# Changed $max_columns to 4 (previously was 3). Changed HTML output slightly
#   for better style.
#
# Revision 1.3  1996/11/22 20:38:43  sabol
# Added ReadParse(), Encode(), and Decode() routines.
#
# Revision 1.2  1996/09/20 22:44:58  sabol
# If there are only one or two actions and the browser doesn't support tables,
#   then it doesn't use a select menu for displaying the actions.
#
# Revision 1.1  1996/09/20 21:01:46  sabol
# Created CGI.pm.
#
#=============================================================================#
# High Energy Astrophysics Science Archive Research Center (HEASARC),
# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, USA.
#
# You are free to use this code as you wish, but unless you change
# it significantly, please leave the above name and address intact.
#=============================================================================#

package CGI;

use strict;

#------------------------------------------------------------------------------
# Package-wide variable(s)
use vars qw($header_has_been_printed);

$header_has_been_printed = 0;

#------------------------------------------------------------------------------
sub Actions {
    # Function  input: array of CGI actions
    # Function output: HTML string that implements the actions

    my(@actions) = @_;

    my($smart) = &CGI::BrowserSupport::Tables();
    my($output_string);
    my($column,$action,$default,$reset_name);
    my($max_columns) = 4;
    my($size) = (scalar(@_) > $max_columns) ? $max_columns : scalar(@_);

    $actions[0] = 'Submit' if (!$size);

    (print "CGI::Actions: \@actions = ",join(',',@actions),"<br>",
           "CGI::Actions: smart = $smart<br>") if $::debug;

    if ($smart) {
	# If the browser supports HTML tables, create a table of action
	# buttons with names with at most $max_columns columns.
	$output_string =
	  "<center><table border>\n<tr><th colspan=$size>Actions</th></tr>\n";
	print "<!-- size = $size -->\n";
	$column = 0;
	while ($action = shift(@actions)) {
	    $output_string .= "<tr>" if (!$column);
	    print "\n<!-- column = $column --><!-- action = \"$action\" -->"
	      if $::debug;
	    if ($action =~ /reset/i) {
		$output_string .=
		  "<td align=center><font size=\"+1\"><input type=reset name=\"$action\"></font></td>";
	    } else {
		$output_string .=
		  "<td align=center><font size=\"+1\"><input type=submit name=Action value=\"$action\"></font></td>";
	    }
	    $column++;
	    if ($column == $size) {
		$output_string .= "</tr>\n";
		$column = 0;
	    }
	}
	$output_string .= "</tr>" if ($column);
	$output_string .= "</table></center>";
    } else {
	# Presumably, if it doesn't support HTML tables, the browser is most
	# likely Mosaic, and to bypass a bug in Mosaic that prevents usage of
	# more than one submit-type button, we use a pop-up menu that the user
	# can select the action they want to submit instead.
	if (scalar(@actions) == 1) {
	    $action = shift(@actions);
	    $output_string = "<font size=\"+1\">" .
	      "<input type=submit name=Action value=\"$action\"></font>" .
		"<input type=reset name=\"$action\"></font>";
	} elsif ((scalar(@actions) == 2) && ($actions[0] =~ /reset/i
					     || $actions[1] =~ /reset/i)) {
	    $output_string = "<font size=\"+1\">";
	    foreach $action (@actions) {
		if ($action =~ /reset/i) {
		    $output_string .= "<input type=reset name=\"$action\">";
		} else {
		    $output_string .= "<input type=submit name=Action value=\"$action\"></font>";
		}
	    }
	} else {
	    $output_string = "Select Action: <select name=Action>";
	    $default = "selected";
	    $reset_name = "Reset Form";
	    foreach $action (@actions) {
		if ($action =~ /reset/i) {
		    $reset_name = $action;
		} else {
		    $output_string .= "<option $default>$action";
		    $default = '';
		}
	    }
	    $output_string .= "</select><input type=submit>" .
	      "<input type=reset name=\"$reset_name\">";
	}
    }

    return($output_string);

} #eosub--Action
#------------------------------------------------------------------------------
sub SimpleSubmit {
    # Function  input: name of the Submit button
    # Function output: HTML string that implements the submit and reset buttons

    my($submit_name) = shift(@_);

    my($smart) = &CGI::BrowserSupport::Tables();
    my($output_string) = '';
    $submit_name = 'Submit' if (! $submit_name);

    (print "CGI::SimpleSubmit: submit_name = $submit_name<br>",
           "CGI::SimpleSubmit: smart = $smart<br>") if $::debug;

    $output_string .= "<table border><tr><th>" if $smart; 
    $output_string .= "<font size=\"+1\">Actions:&nbsp;</font>";
    $output_string .= "</th><td align=center>" if $smart;
    $output_string .= "<font size=\"+1\">&nbsp;&nbsp;<input type=submit value=\"$submit_name\">&nbsp;&nbsp;</font>";
    $output_string .= "</td><td align=center>" if $smart;
    $output_string .= "<font size=\"+1\">&nbsp;&nbsp;<input type=reset value=\"Reset\">&nbsp;&nbsp;</font>";
    $output_string .= "</td></tr></table>" if $smart;

    return($output_string);

} #eosub--SimpleSubmit
#------------------------------------------------------------------------------
sub ReadParse {
    # Based on Steven E. Brenner's cgi-lib Perl routines. This version has
    # been jazzed up to be more Perl 5-savvy. It should also be faster.
    #
    # Reads in GET or POST data, converts it to unescaped text, and returns a
    # reference to a key/value hash of the data, using '\0' to separate multiple
    # values with the same key.
    #
    # Function  input: (optional) reference to a hash to be used by this routine
    # Function output: reference to a key/value hash of the GET or POST CGI data

    my($in);
    if (scalar(@_)) {
	$in = shift(@_);
    } else {
	$in = {};
    }

    my($entry,$input,$key,$val);

    # Read in text.
    if ($ENV{'REQUEST_METHOD'} eq "GET") {
	$input = $ENV{'QUERY_STRING'};
    } elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
	read(STDIN,$input,$ENV{'CONTENT_LENGTH'});
    }

    foreach $entry (split(/&/,$input)) {

	# Convert %XX from hex numbers to alphanumeric and '+' to ' '.
	$entry = &CGI::Decode($entry);

	# Split into key and value.
	($key,$val) = split(/=/,$entry,2);

	# Associate key and value.
	$in->{$key} .= "\0" if (defined($in->{$key})); # \0 is the separator
	$in->{$key} .= $val;

    }

    return($in);

} #eosub--ReadParse
#------------------------------------------------------------------------------
sub Decode {

    my($string) = shift(@_);

    $string =~ tr/\+/ /;
    $string =~ s/%([0-9a-fA-F][0-9a-fA-F])/chr(hex($1))/ge;

    return($string);

} #eosub--Decode
#------------------------------------------------------------------------------
sub Encode {

    my($string) = shift(@_);

    $string =~ s/([^ 0-9A-Za-z])/sprintf("%%%02X",ord($1))/ge;
    $string =~ tr/ /+/;

    return($string);

} #eosub--Encode
#------------------------------------------------------------------------------
sub PrintVariables {

    my($in) = shift(@_);

    my($key,$value,$output);
    $output = '<dl compact>';
    foreach $key (sort keys(%{ $in })) {
	foreach $value (split("\0",$in->{$key})) {
	    $value =~ s/\n/<br>/mg;
	    $output .= "<dt><b>$key</b><dd><i>$value</i><br>";
	}
    }
    $output .= '</dl>';

    return($output);

} #eosub--PrintVariables
#------------------------------------------------------------------------------
sub ContentType {

    return('') if $header_has_been_printed;

    my($type,$format) = @_;

    if (!defined($type)) { $type = 'text'; }
    if (!defined($format)) { $format = 'html'; }

    $header_has_been_printed = 1;

    return("Content-Type: $type/$format\n\n");

} #eosub--ContentType
#------------------------------------------------------------------------------
sub Error { # Prints error message.

    my($msg) = join(' ',@_);

    my($header) = &ContentType();
    if ($header eq '') {
	print "</pre><br>\n";
    } else {
	print $header,
	      "<html>\n<head><title>Error Notice</title></head>\n",
	      "<body>\n";
    }
    print "<h1>$msg</h1>\n</body>\n</html>\n";
    exit(1);

} #eosub--Error
#------------------------------------------------------------------------------

package CGI::BrowserSupport;

use strict;

#------------------------------------------------------------------------------
# Package-wide variable(s)
use vars qw($Excludes);

$Excludes = {
    'tables' =>
      "Lynx|CERN-LineMode|Emacs-W3|Mosaic|Mozilla\/0.|MOMspider|Microsoft",
    'image_maps' => "Lynx|CERN-LineMode|MOMspider"
};

#------------------------------------------------------------------------------
sub Tables {

    my($browser);
    if (scalar(@_)) {
	$browser = shift(@_);
    } else {
	$browser = $ENV{'HTTP_USER_AGENT'};
    }
    return ($browser =~ /$Excludes->{'tables'}/i) ? 0 : 1;

} #eosub--Tables
#------------------------------------------------------------------------------
sub ImageMaps {

    my($browser);
    if (scalar(@_)) {
	$browser = shift(@_);
    } else {
	$browser = $ENV{'HTTP_USER_AGENT'};
    }
    return ($browser =~ /$Excludes->{'image_maps'}/i)
      ? 0 : 1;
} #eosub--ImageMaps
#------------------------------------------------------------------------------
#########
#  EOF  #
#########
1;
