# This -*- perl -*- script checks for necessary Perl
# packages and installs Browse files.
# $Id$

# *** need to allow to skip steps - maybe also cache responses in file?
require 5.004;
use strict;
use Config;

$| = 1;
my $done=0;
my $separator= '-'x70 . "\n";
my $sect_separator= "\n" . '*'x70 . "\n";
my $ans="y";
my (@errors,$docDir,$scriptDir);
$::configChanges="";
$::cacheDir;

print $separator;
print "     HEASARC BROWSE INSTALLATION SCRIPT:\n";
print "     -----------------------------------\n";
print "\nThis script will do the following:\n";
print "1. Test compiled versions of name resolver routines after prompting for their location\n";
print "2. Ensure Browse scripts contain correct Perl path\n";
print "3. Install Browse files in designated directories\n";
print "4. Create an initial Config.pm according to answers to prompts\n" .
      "   Config.pm is a configuration file that is run from within Browse\n\n";

print "Prompts will include some directories that need to exist
and be writable by cgi scripts. You will be prompted for\n";
print "- location of directory for storing temporary files: \n";
print "- location of directory where a cache file will be created \n" .
	"  for storing coordinates returned from name resolver routines.\n" .
	"  This cache is used to make queries more efficient and is optional\n"; 
print "- location of directory for log files \n";
print "\n\nIf resolver programs are not" .
	" found or not working Browse files can still be \ninstalled.  Rerun" .
	" this script after problems are resolved.\n\n";
print "A separate script checkmod.pl checks for Perl Modules\n" .
	" necessary for Browse to run.  Be sure to run that script as well\n";
print "\n** Continue? (y/n) [y] ";
chomp($ans=<STDIN>);  
$ans="y" unless $ans;
exit unless $ans =~ /y/i;
#**********************************************************************
#---- USER: Add addition perl libraries to path HERE
#---- There is also a prompt for this information below
#@::browselibs = ("/www/htdocs/cgi-bin/lib/",
                   #"/www/htdocs/cgi-bin/lib/heasarc/",
                   #"./lib");
#unshift(@INC,@::browselibs);
# /www/bin/perldir/lib/site_perl/5.6.0/i386-linux/,/www/htdocs/cgi-bin/lib/,/www/htdocs/cgi-bin/lib/heasarc/,./lib
#/home1/lmm/nameres/nameres_leg  
#/www/htdocs/cgi-bin/W3Browse/bin
#---------------------------------------------------------------------


#print $separator;

#checkPerl();

#print $separator;

#required_mods();
#---------------------------------------------------------------------

print $sect_separator;

#**********************************************************************
#--- Check for resolvers and tar command
check_resolvers();

#print $::configChanges;
#---------------------------------------------------------------------

print $sect_separator;
#**********************************************************************
#--- Adjust path in all perl scripts to match user machine path
shebang();

#---------------------------------------------------------------------

print $sect_separator;

#**********************************************************************
#--- Prompt for installation directory and save current scripts (just in case..)
#print "** Proceed with Browse file installation? (y/n) [y] ";
#chomp($ans=<STDIN>);  
#$ans="y" unless $ans;
#if ($ans =~ /y/i ) {
  $ans="y";
  get_install_vars() until $ans !~ /y/i;
  
  print $sect_separator;
  print "** Going to copy files to their final destinations. Proceed? (y/n) [y] ";
  chomp($ans=<STDIN>);  
  $ans="y" unless $ans;
  #quit() if $ans !~ /y/i;
  move_files() unless $ans !~ /y/i;
#}
#---------------------------------------------------------------------

print $sect_separator;

my $perllib=$ENV{PERL5LIB};

createConfigFile();

$done=1;
# Make sure we print a list of all errors.
END {
   my $err;
   exit unless scalar(@errors);
   #print "\n"."="x75 ."\nSummary of all errors:\n\n";
   print "\n"."\nSummary of all errors:\n\n";
   foreach $err (@errors) {
      print "* " . $err;
   }
   print "\nThe README file may have information necessary to fix" .
   	"\nthese errors.  Rerun this script after errors are fixed.\n";
   print "runme.pl terminated abnormally! $!\n" unless $done;
   print "\n"."="x75 ."\n";
}
#-----------------------------------------------------------------------------
#--- Create a premilinary Config.pm file using answers to promts and
#--- other setup
sub createConfigFile {

   open(CONFIG, ">Config.pm") || die "Cannot open Config.pm";;



print CONFIG "#============================================================" .
		"=========#\n";
print CONFIG "# Project:        Browse : HEASARC\n";
print CONFIG "#============================================================" .
		"=========#\n";
print CONFIG "# Module Description:\n";
print CONFIG "# Define W3Browse program variables.\n";
print CONFIG "#  This module should be 'use'd by all W3Browse perl scripts.\n";
print CONFIG "#  The $::variables are in the main package.\n";
print CONFIG "#\n";
print CONFIG "#============================================================" .
		"=========#\n";
print CONFIG "# High Energy Astrophysics Science Archive Research Center " .
		"(HEASARC),\n";
print CONFIG "# NASA's Goddard Space Flight Center, Greenbelt, MD 20771, " .
		"USA.\n";
print CONFIG "#\n";
print CONFIG "# You are free to use this code as you wish, but unless you " .
		"change\n";
print CONFIG "#  it significantly, please leave the above name & address " .
		"intact.\n";
print CONFIG "#============================================================" .
		"=========#\n";

print CONFIG "package main;\n\n";

print CONFIG '# These are things you won\'t need to change'. "\n";
print CONFIG "use CGI::Carp qw(fatalsToBrowser);\n";
print CONFIG '$::version       = $VERSION = "8.0.0";' . "\n"; 
print CONFIG '$::batch_version = "2.0";' . "\n";
print CONFIG '$::heasarc       = "http://heasarc.gsfc.nasa.gov";' . "\n";
print CONFIG "\n# To see dubug messages the string 'debug' can be set in the query string.\n";
print CONFIG '$::debug = 1 if $ENV{\'QUERY_STRING\'} =~ /debug/;' . "\n\n";

print CONFIG "# Set Perl path for required libraries... \n";
print CONFIG "# The paths where your XML, DBI, and other nonstandard \n";
print CONFIG "# libraries live should be included.  The directory './lib'\n" .
	     "# should always be listed, since that is where the \n" .
	     "# Browse:: package lives.\n\n";
print CONFIG "BEGIN {\n";
print CONFIG "# root is the path to just above cgi-bin.  \n";
print CONFIG '$::root    ="' . $docDir . '";' ."\n";
print CONFIG '$::cgi     ="' . $scriptDir . 'W3Browse";' ."\n\n";

print CONFIG '    @::browselibs = ("$::cgi/lib/", "$::cgi/../lib",' . "\n";
print CONFIG '       "./lib");' ."\n";
print CONFIG 'unshift(@INC,@::browselibs);' . "\n";
print CONFIG "}\n\n";

#-- This should be name resolver paths
print CONFIG "# Locations of compiled name resolvers\n";
print CONFIG "$::configChanges\n\n";

#--- prompt for Name and email of site maintainer
my ($Mname, $Memail);
print "\n\n** Please enter the name and then email address of the site maintainer\n" .
      "** (This information  will be added at the bottom of the web page):\n\n";
print "\n\nEnter NAME: ";
chomp($Mname=<STDIN>);  

print "Enter EMAIL: ";
chomp($Memail=<STDIN>);  

my ($Mid,$Maddress)=split(/\@/,$Memail);

print CONFIG "# Maintainer name and email is printed at the bottom of each " .
	"page.  Change it\n";
print CONFIG "# to whoever is maintaining this installation of Browse\n";
print CONFIG '$::maintainer_name  ="' . $Mname .'";' . "\n";
print CONFIG '$::maintainer_email = "' . $Mid . '\@' .$Maddress .'";' . "\n\n"; 

#--- email address for MailMessage
print CONFIG "# This is the emergency address mailed to by " .
	"the MailMessage() package.\n";
print CONFIG '$::addr = "' . $Mid . '\@' . $Maddress .'";' . "\n\n";


#--- prompt for SYBASE directory
print "\n\n** If SYBASE is used please enter the directory that \n";
print "** contains the Sybase lib directory \n";
print "Sybase Directory: ";
chomp($ans=<STDIN>);  
if ($ans) {
   print CONFIG "# Sybase/installation specific stuff \n";
   print CONFIG '$ENV{\'SYBASE\'} = "' . $ans . '";' . "\n";
   print CONFIG '$ENV{\'LD_LIBRARY_PATH\'} = ' .
	'"$ENV{\'SYBASE\'}/$ENV{\'SYBASE_OCS\'}/lib";' .
		"\n\n";
} else {
   print "No Sybase directory specified\n\n";
}

#---- prompt for fully qualified server name for java
my $server = $ENV{'HOST'};

print CONFIG "# The java plotting package needs the fully qualified name\n" .
	"# of the web server.\n";
print "\n\n** The java plotting package needs the fully qualified server name.\n";
print "** Your server name appears to be: " . $server. "\n";
print "** If this is incorrect or blank please enter the name of the server.\n";
print "Hit return if correct or enter fully qualified name: ";
chomp($ans=<STDIN>);  
if ($ans) {$server = $ans;} 

print CONFIG '$::host      ="' .  "$server" . '"' .";\n";
print CONFIG '$::serverURL   = "http://$::host/$::cgiurl/w3server.pl";' .
	"\n\n";

#--- Jave distribution path
print CONFIG "# Please change " . '$::javaroot' . " value to your current " .
	"java distribution path.\n#  The rest can probably stay as is.\n";
print CONFIG '$::javaroot     = "/usr/java/j2sdk1.4.0_01";' . "\n";
print CONFIG '$::jbin         = "$::javaroot/bin/java";' . "\n";
print CONFIG '$::rtjar        = "$::javaroot/jre/lib/rt.jar";' . "\n";
print CONFIG '$::jsystemfonts = "$::javaroot/jre/lib/fonts";' . "\n\n";

#-- prompt for log file area
print CONFIG "#-- Log Files ---\n";
print CONFIG "# We write logs for certain Browse actions.  The " . '$::logDir' .
	 " variable should \n";
print CONFIG "# to a diectory writeable by cgi-scripts;\n";
print CONFIG '$::logDir   = "/Apache.Logs";' . "\n";
print CONFIG '$::queryLog = "$::logDir/w3browse_query.log";' . "\n";
print CONFIG '$::tarLog   = "$::logDir/w3browse_tar.log";' . "\n";
print CONFIG '$::sqlLog   = "$::logDir/w3browse_sql.log";' . "\n";
print CONFIG '$::VizLog   = "$::logDir/w3browse_vizierquery.log";' . "\n\n";

#--- data prooducts creation area
my $urlbase ="/FTP";
my $ftpbase = "/retrieve/w3browse";
print "\n\n** If data products are made available an FTP directory will \n";
print "** need to exist.  This directory will need to  be\n";
print "** writable by the cgi-script that generates data products. \n";
print "** Please enter the ftp base directory and the url base where \n";
print "** files are to deposited \n";
print "** example: if  data products are accessed via urls \n";
print "**      http://www.system.com/FTP/retrieve/w3browse/ and \n";
print "**      ftp://www.system.com/retrieve/w3browse/ \n";
print "** then the FTP base is /FTP and the URL base is /retrieve/w3browse\n\n";
print "Enter the FTP base or RETURN to ignore: \n";
chomp($ans=<STDIN>);  
if ($ans) {$urlbase = $ans;} 
print "Enter the URL base or RETURN to ignore: \n";
chomp($ans=<STDIN>);  
if ($ans) {$ftpbase = $ans;} 
print CONFIG "#-- Data Products Files ---\n";
print CONFIG "# If no data products are available there is no need to " .
	"change this section\n";
print CONFIG "# At HEASARC, our tarfiles are deposited in " .
	"/FTP/retrieve/w3browse/ and are \n";
print CONFIG "# available at the url " .
	"ftp://legacy.gsfc.nasa.gov/retrieve/w3browse/\n";
print CONFIG "# and at the url " .
	"http://legacy.gsfc.nasa.gov/FTP/retrieve/w3browse\n";
print CONFIG "# The tar size limit is the maximum estimated tarfile " .
	"size you want to \n";
print CONFIG "# allow users to request.  The limit is set in bytes.\n";
print CONFIG '$::tarurlbase = "' . $ftpbase . '";' ."\n";
print CONFIG '$::tarftpbase = "' . $urlbase . '";' ."\n";
print CONFIG '$::tarsizelimit = 3*1024*1024*1024;  # 3 GB' ."\n\n";

#--- Vizier mirror sites
print CONFIG "# These are the VizieR sources - uncomment the one you " .
	"want to use, or add\n";
print CONFIG "# more if you want to use others.\n";
print CONFIG '$::vizier      = "http://vizier.u-strasbg.fr"; ' .
	"# CDS in Strasbourg, France\n";
print CONFIG '#$::vizier      = "http://vizier.cfa.harvard.edu"; ' .
	"#CFA at Harvard\n";
print CONFIG '#$::vizier      = "http://adc.gsfc.nasa.gov"; # ADC: ' .
	"slow, maybe inactive?\n";
print CONFIG '#$::vizier      = "http://vizier.hia.nrc.ca";  ' .
	"# CADC in Canada\n";
print CONFIG '#$::vizier      = "http://archive.ast.cam.ac.uk/";  ' .
	"# Cambridge\n";
print CONFIG '#$::vizier      = "http://data.bao.ac.cn/"; # Bejing ' .
	"Obs. in China, slow\n";
print CONFIG '#$::vizier      = "http://vizier.nao.ac.jp/"; # ADAC ' .
	"in Tokyo, Japan\n";
print CONFIG '#$::vizier      = "http://urania.iucaa.ernet.in/"; ' .
	"IUCAA in Pune, India\n";
print CONFIG '#$::vizier      = "http://www.inasan.rssi.ru/"; INASAN ' .
	"in Russia\n\n";


print CONFIG "# Change the following variable to your main web page or \n" .
	"# leave as is (links to Browse Help)\n";
print CONFIG '$::mainbrowse = "/W3Browse/w3browse-help.html";' . "\n\n";

print CONFIG "# This variable can stay as is unless you want to use it \n";
print CONFIG "# in your CGI srcipts as a global reference to the base \n";
print CONFIG "# directory containing mission documentation\n"; 
print CONFIG '$::helpdir     = "" ; # mission help pages' .
	"\n\n";


print CONFIG "# temp directory (used for writing temporary files) \n" .
	"# This directory must be writable by cgi-scripts and should have\n" .
        "# 1-2 MB of free space\n";
print CONFIG '$::tempdir = "/tmp";' . "\n\n";

#-- local table prefix
print CONFIG "# The custom is to prefix your local tables with this " .
	"origin label:\n"; 
print CONFIG "#   e.g.: heasarc_rospublic, heasarc_xray\n";
print CONFIG "# This variable is used on display pages to distinguish" .
	" local tables\n" .
	"# from other types\n";
print CONFIG '$::local_origin= "heasarc_";' . "\n\n";

#-- Runtime definitions
print CONFIG "# How do you want the 'tar' binary called? (this is called " .
	"in Products.pm)\n";
print CONFIG "# nice will lower the system process priority and will " .
	"interfere less with\n"; 
print CONFIG "# other processes\n";
print CONFIG '$::tar = "nice tar";' . "\n\n";

#--- df binary
print CONFIG "# What is your df binary?  This is used to test disk space.\n";
print CONFIG "# Pls leave the '-k', or whatever you need to get\n";
print CONFIG "# the disk space in kilobytes\n"; 
print CONFIG '$::df = "df -k";' . "\n\n";

#--- max images for data products
print CONFIG "# Maximum number of preview images you want displayed on the " .
	"data products list page\n" . 
	"# Ignore if no data products are available\n";;
print CONFIG '$::MaxGIFs = 25;' . "\n\n";

print CONFIG "#--- Miscellaneous settings ---\n";
print CONFIG "# We check to see if the server is swamped with " .
	"requests and print error\n";
print CONFIG '$::maxprocesses = 15; # if there are more than this, ' .
	"print error \n";
print CONFIG '$::ps =' . " 'ps -A';" . "  # your ps command - see " .
	"Utils.pm if more changes needed\n\n";

print CONFIG "# This is the maximum angular separation that two resolved " .
	"positions can\n";
print CONFIG "# differ by before we report a warning.  \n";
print CONFIG '$::radius_tolerance = 0.167; ' ." # arcminutes = 10 arcsec\n\n";


print CONFIG "# These are strings to use as the start of saved filed names.\n";
print CONFIG '$::savedQueryFile="BrowseQuery";' . "\n";
print CONFIG '$::savedTargetsFile="BrowseTargets";' . "\n\n";


print CONFIG "# Map of table name patterns to data products modules:\n";
print CONFIG "#   e.g.:  table names matching \"heasarc_\" will cause " .
	"data product query objects\n";
print CONFIG "#          to be created with type HDPQuery.\n";
print CONFIG "# If you have modules to process data products just replace\n";
print CONFIG "# the HProdDislay and HProdGen modules names with yours.\n";
print CONFIG '%::dpq = (' ."\n";
print CONFIG '    "heasarc_" => "HDPQuery",' ."\n";
print CONFIG '    "\/"       => "HDPQuery",' ."\n";
print CONFIG ');' ."\n";
print CONFIG '%::prods = (' ."\n";
print CONFIG '    "view" => "HProdDisplay",' ."\n";
print CONFIG '    "write" => "HProdGen",' ."\n";
print CONFIG ');' ."\n";


#-- Some definitions based on previously defined variables
print CONFIG "# w3bdocs is the directory under " . '$::root' . 
	" where the Browse html pages live.\n";
print CONFIG "# images is the URL where your Browse images live.\n";
print CONFIG "# this only needs to be changed if you moved files after " .
	"running the \n";
print CONFIG "# installation script\n";
print CONFIG '$::w3bdocs = "/W3Browse";' . "\n";
print CONFIG '$::images  = "/Images/w3b";' . "\n";
#--- Java  ---- 
print CONFIG "#--- Java component of Browse (plotting package)---\n";
print CONFIG '$::java = "$::root/W3Browse/classesnew";' . "\n\n";

#print CONFIG '   $::cgi =  $scriptDir;' . "\n";
print CONFIG '$::cgiurl = $::cgi;' . "\n";

print CONFIG '$::writeHeader = 1;' . "\n";
print CONFIG '$::lib         = "$::cgi/lib";' ."\n";
print CONFIG '$::bin         = "$::cgi/bin";' ."\n";
print CONFIG '#$::basicData   = "$::root/$::cgi/w3basic.data";' ."\n";

print CONFIG "# Set path for binary executables...\n";
print CONFIG '$ENV{\'PATH\'} = "/bin";' . "\n\n";


print CONFIG "# You may not have to change anything in this group, " .
	"since these build on \n";
print CONFIG "#   prior definitions.\n";
print CONFIG '$::w3bhelp     = "$::w3bdocs/w3browse-help.html"; ' .
	"# Browse help doc\n";
print CONFIG '$::vizgif      = "$::images/vizier_logo.gif";' . "\n\n";


#--- definitions needed for older versions of Filesys package
print CONFIG "# Unless you are using an old version of Perl packages this variable\n";
print CONFIG "# is nost likely irrelevant and should stay as is\n";
print CONFIG "# dflib is the directory where you will find " .
	"sys/statvfs.ph and sys/syscall.ph.\n";
print CONFIG "# This may be needed for calling Filesys::Diskspace " .
	"in Products.pm.\n";
print CONFIG '$::dflib = "/usr1/local/lib/perl5";' ."\n\n";

#----- not need but may break if variable does not exist ----
print CONFIG "# This setup is for an older version of Browse. No need to " .
	"change this.  \n";
print CONFIG "# map old tables that are no longer used to their " .
	"new equivalents\n";
print CONFIG '%::maptables = (' . "\n";
print CONFIG '    "BATCHRETRIEVALCATALOG_2.0 heasarc_usnoa2" => ' . "\n";
print CONFIG '                        "BATCHRETRIEVALCATALOG_2.0 I/252",' . 
	"\n";
print CONFIG '    "usnoa2" => "I/252",' . "\n";
print CONFIG '    "heasarc_usnoa2" => "I/252",' . "\n";
print CONFIG ');' . "\n\n";


#print CONFIG '   # Set Perl path for required libraries...' . "\n";
#print CONFIG '   # keep these as unshifts, not uses, because of mod_perl' .
	#"\n";
#print CONFIG '   unshift(@INC,@::browselibs);' . "\n";

print CONFIG '# Get Remote Host' . "\n";
print CONFIG 'if ($ENV{\'HTTP_X_FORWARDED_FOR\'}) {' . "\n";
print CONFIG '      $::REMOTE_HOST = $ENV{\'HTTP_X_FORWARDED_FOR\'};' . "\n";
print CONFIG '} elsif ($ENV{\'REMOTE_HOST\'}) {' . "\n";
print CONFIG '     $::REMOTE_HOST = $ENV{\'REMOTE_HOST\'};' . "\n";
print CONFIG '} elsif ($ENV{\'REMOTE_ADDR\'}) {' . "\n";
print CONFIG '     $::REMOTE_HOST = $ENV{\'REMOTE_ADDR\'};' . "\n";
print CONFIG '} else {' . "\n";
print CONFIG '     $::REMOTE_HOST = "Unknown";' . "\n";
print CONFIG '}' . "\n\n";

#-- Footer and header 
print CONFIG "# " . 'THESE FILES ARE CURRENTLY NOT DISPLAYING PROPERLY' ."\n";
print CONFIG "# " . '$::footer' . " should reference a file containing text ".
             " to be displayed at the bottom  of each page, " .
	'$::menubar file is displayed at the top.' ."\n";

print CONFIG "# Bare bones versions of these files are included in the \n" .
	"# distribution in the htdocs/W3Browse directory.  Make changes to \n" .
	"# these files or change the paths below to the paths of your own \n" .
	"# footer/header files. Note: These files are currently not \n" .
	"# diplaying properly\n";
print CONFIG '$::footer  = "/W3Browse/footer.html";' . "\n";
print CONFIG '$::menubar = "/W3Browse/header.html";' . "\n\n";


print CONFIG "# commonly-used modules.  No need to change these.\n";
print CONFIG "use Browse::Utils;\n";
print CONFIG "use Browse::MailMessage;\n";
print CONFIG "use Browse::HDB;\n";
print CONFIG "use Browse::DPQuery;\n";
print CONFIG "use Browse::DBQ;\n";
print CONFIG "use Browse::Decor;\n";
print CONFIG "use Browse::CGI;\n";
print CONFIG "use Sys::Hostname;\n\n";


#print CONFIG "# ** There are more settings to change in the next " .
	#"subroutine.... **\n";

print CONFIG "#-------------------------------------------------" .
	"-----------------------\n\n";

print CONFIG "package Browse::Config;\n";

print CONFIG "sub AlwaysRun() {\n";
#print CONFIG "   # this code should always be run for each request, even " . 
	#"under mod_perl\n";
#--- initialize database
print CONFIG "   # counts the number of times this process has been called\n";
print CONFIG '   $::pcount += 1;' . "\n\n";
print CONFIG '   # Initialize database tables.' . "\n";
print CONFIG '   Browse::DPQuery::clear();' . "\n";
print CONFIG '   Browse::DBQ::Registry::init("databases.dat");' . "\n";



print CONFIG '}' . "\n";

close(CONFIG);

   if ($perllib) {
      if ($::configChanges =~ /browselibs/) {
         print "\n- You may need to add the following directories (currently in" .
         "\n- the PERL5LIB environment variable) to the " . '@::browselibs list' .
         "\n- or separately as in the following example code:\n"
      } else {
         print "\n- You may need to include the following directories \n" .
         "- (currently in the PERL5LIB environment variable) to Config.pm." .
         "\n- The following example code should work;\n";
      }
         my @libs=split(/:/,$perllib);
         my $liblist="";
         for (my $i=0; $i <=$#libs; ++$i ) {
            $liblist .= "\"$libs[$i]\"";
            if ($i <$#libs) {
               $liblist .= ",";
            }
         }
      
         print "\n" .'@perllibs' . 
   	   "= ($liblist)\n"  .
	   "unshift(". '@INC' . "," . '@perllibs' .")\n";
      }
   }
print "-"x75 ."\n";
print "Finished with installation!\n\n\n";
print "*"x75 ."\n";
print "** The configuration file Config.pm has been created.\n";
print "** Please look at the file and make any necessary changes \n" .
      "** especially if there are errors listed below. \n" .
      "** THE FILE Config.pm MUST THEN BE COPIED TO \n" .
      "** $scriptDir/W3Browse/lib/Browse\n\n\n";
print "** Also be sure to edit the databases.dat in the directory\n" .
	"$scriptDir/W3Browse. \n" .
	"See the README file for details\n\n\n";


#}

#-----------------------------------------------------------------------------
#--- Prompt for location of snr and NED_name.  Test the programs to see if
#--- they have been compiled correctly.  Also check if tar command is 
#--- accessible
sub check_resolvers {

   my $ans="y"; 
   print "\nThe Browse distribution includes source code for SIMBAD \n";
   print "and NED_name name resolvers.  The code needs to be compiled \n";
   print "to create the programs snr and NED_name.  Name resolver \n" .
     	"compilation help can be found in the README file.\n";
   print "\n** Are the name resolvers compiled? (y/n) [y] ";
   my $snrDir;
   my $nedDir;
   chomp($ans=<STDIN>);  
   $ans="y" unless $ans;
   
   if ($ans =~ /y/i) {
      #--------------------------------------------------------------------
      #--- Test snr
      print "\n** In which directory is snr program located? ";
      chomp($snrDir=<STDIN>);  
      #print "SNR dir=$snrDir\n";
      if ($snrDir) {
         my $coord =`$snrDir/snr mkn421`;
         #print "snr *$coord*\n";
         if (!length($coord) || $coord =~ /error/i) {
            Error("There is a problem with snr.  Error message is:\n  $!");
         } else {
            print "\n>>Program snr looks fine. \n" ;
            $::configChanges .=  
		"\n" . '$Astro::Object::Resolver::SIMBAD::SNR =' . 
     		"\"$snrDir/snr\"\;\n";
         }
      
      } else {
         print "\n>>No directory path was entered. \n" ;
         Error("The snr name resolver program needs to be installed")
      }

      #--------------------------------------------------------------------
      #--- Test NED_name
      print "\n** In which directory is NED_name program located? ";
      chomp($nedDir=<STDIN>);  
      if ($nedDir) {
         my @coord =`$nedDir/NED_name mkn421`;
         chop($coord[0]);
         if (!length($coord[0]) || $coord[0] =~ /error/i ) {
            Error("There is a problem with NED_name.");
         }  else {
            print "\n>>Program NED_name looks fine. \n" ;
            $::configChanges .= "\n" . 
  		'$Astro::Object::Resolver::NED::NED = ' .
     		"\"$nedDir/NED_name\"\;\n";
         }
      
      } else {
         print "\n>>No directory path was entered. \n" ;
         Error("The NED_name name resolver program needs to be installed")
      }
      print $separator;
      #--------------------------------------------------------------------
      #--- Create name resolver cache
      print "\nThe name resolvers use a cache to store coordinates " .
	"locally for future queries.\n" .
	"Typically the cache file is in a subdirectory of /tmp\n";
      print "\n** In which directory should the name resolver " .
	"cache be created?\n";
      print "** A subdirectory W3Browse/dbm will be created there and" .
	" a link to \n" .
	"** it will be created in the Browse installation area:\n" ;
      $::cacheDir="./dbmtest";
      chomp($::cacheDir=<STDIN>);  
      
      if ($::cacheDir) {   # Was a directory entered?
        if (! -x "$::cacheDir/W3Browse/dbm") {
         if (-x $::cacheDir) {    # Does directory already exist
            my $w3b=`mkdir $::cacheDir/W3Browse` unless (-x "$::cacheDir/W3Browse");

            if (-x "$::cacheDir/W3Browse") {   # Was the first subdir created?
               my $dbm=`mkdir $::cacheDir/W3Browse/dbm`;

               if (! -x "$::cacheDir/W3Browse/dbm") {# Was subdir dbm created?
                  print "\n>>$::cacheDir/W3Browse/dbm could not be created\n";
                  print ">>The Name resolver cache is a time saving feature \n";
                  print ">>but it not required.\n"
               }else {
                 print "\n>>Directory created.  Link will be created later.\n";
               }

            } else {
                  print "\n>>$::cacheDir/W3Browse could not be created\n";
                  print ">>The Name resolver cache is a time saving feature \n";
                  print ">>but it is not required.\n";
                  Error("The name resolver cache could not be created");
            }

         } else {  
            print "\n>>$::cacheDir does not exist.\n";
            Error("The name resolver cache was not created");
         }
        } else {
            print "\n>>$::cacheDir/W3Browse/dbm exists.  No need to recreate.\n";
         }
      } else {
         print "\n>>No cache directory was entered.\n";
      }
   } else {
         Error("The name resolvers need to be compiled.  Check the README\n" .
		"file for more information");
   }
   
  # print $separator;
   print "\nLooking for tar command... \n";
   my $tar=`which tar`;
   if (!length($tar) || $tar =~ /not found/i) {
       print "\n>>Tar command could not be found\n"
   } else {
      print "\n>>Tar command found.\n";
   }
}
#-----------------------------------------------------------------------------
sub required_mods {
# Check for non-standard modules that are used by Browse

  my %check = (
     "LWP.pm,REQUIRED" => "The LWP module is needed for querying VizieR tables, and for startfv.pl, which slurps in remote FITS files for viewing.",
     "DBI.pm,REQUIRED" => "The DBI module is REQUIRED for querying the databases. ",
     "GDBM_File.pm,REQUIRED" => "The GMDB_File module is required for resolving object names.",
     "CGI.pm,REQUIRED" => "The CGI module is required for Error messages.",
     "XML/Parser.pm,REQUIRED" => "The XML::Parser module is required for querying VizieR tables.",
     "RPC/PlClient.pm,required ONLY if connecting to Heasarc database" => "The RPC module is required only if you are connecting to HEASARC database via a DBI proxy.",
     "Filesys/DiskSpace.pm,optional but requires code alteration to leave out" => "The Filesys::DiskSpace module is used in Products.pm for determining how much disk space is available before creating a tar file.  You can replace this code with a call to 'du' (du is not installed in our chrooted web tree, for security.).",
  );

  my $failed=0;
  foreach my $key (keys %check) {
     my ($mod, $status) = split(',',$key);
     print "Checking for $mod...........";
     eval { require $mod; };
     if ($@) {
        if ($status =~ /REQUIRED/) {$failed=1;}
        print " failed\n";
        Error("$mod module is $status!");
        print "$@\n$check{$key}\n\n";
        sleep(2);
        $@='';
     } else {
        print " ok\n";
        print "   Available DBI drivers: ",
               join(', ',DBI->available_drivers),"\n" if $mod =~ /DBI/;
     }
  }
  if ($failed) {
print $separator;
   my $err;
   print "\nSummary of Perl Module Test\n\n";  
   foreach $err (@errors) {
      print "* " . $err;
   }
   print $separator;
   print "\nRequired Perl modules are missing. ";  
   print "Perl Module installation help\n" .
      "can be found in the README files.\n" .
	"** Do you want to exit to install these modules? [y] ";
      chomp($ans=<STDIN>);  
      $ans="y" unless $ans;
      exit if $ans =~ /y/i;
   
  }
}
#-----------------------------------------------------------------------------

sub shebang {
   # change all #! lines for .pl files to the correct perl binary.

   my $dir="htdocs/cgi-bin/W3Browse";
   opendir(DIR,$dir) || die "Can't read this directory to find .pl files!: $!\n";
   my @plfiles = grep(/\.pl$/,readdir(DIR));
   closedir(DIR);
   die "Something is wrong! Can't find any *.pl files! $!\n" unless scalar (@plfiles);

   print "#! lines in Browse Perl scripts may need to be changed to work \n" .
	"on this computer. It looks like the perl binary to use is:\n" .
	"   $Config{startperl} \n";
   print "** [Hit Enter to accept, or type another location] ";

   chomp(my $startperl = <STDIN>);
   $startperl = $Config{startperl} unless ($startperl =~ /\S/i); 
   $startperl =~ s/^#!//;
   unless (-x $startperl) {
       warn("$startperl doesn't exist or isn't executable!. Please try again.\n");
       shebang();
   }
   $startperl = "#!".$startperl unless $startperl =~ /^#!/;

   my $errflag=0;
   my $file ='';
   foreach $file (@plfiles) {
       my $err = "Error: I can't alter $file - you'll need to change the #! line yourself.";
       open(IN,"<$dir/$file") || (Error($err) and $errflag=1);
       open(OUT,">$dir/$file.tmp") || (Error($err) and $errflag=1);
       my $line='';
       while (my $line=<IN>) {
            $line =~ s/^\#\!.*perl\S*/$startperl/;
            print OUT $line;
       }
       close(IN);
       close(OUT);
       rename("$dir/$file.tmp","$dir/$file") || (Error($err) and $errflag=1);
   }

   
   #print " ok.\nOriginal files are kept as *.pl.orig just in case.\n\n" unless $errflag;
   print " done.\nThere were some errors - please investigate.\n\n" if $errflag;

} # end of sub to change shebang

#-----------------------------------------------------------------------------
sub Error {
# print out each error as it comes in, and append it to a list of all errors.
   print "\n"."-"x75 . "\n@_\n" ."-"x75 ."\n";
   push @errors,"@_\n";
}

#-----------------------------------------------------------------------------
sub get_install_vars {

  $docDir    = get_docDir();
  $scriptDir = get_scriptDir();

  print $separator;
  print "\nVERIFICATION OF INPUT:\n     ----------------------\n";
  print "You told me that:\n";
  print "  html dir          = $docDir\n";
  print "  cgi dir           = $scriptDir\n";
  print "** Do you want to go back and change any answers? (y/n) [n] ";
  chomp($ans=<STDIN>);
}

#-----------------------------------------------------------------------------
sub get_docDir {

  print <<EOT;
** What is the full path where you want the Browse html directory? 
** (example: if you respond "/www/htdocs/", the Browse 
** html files will be installed in "/www/htdocs/W3Browse/".) You
** will be prompted again before actual installation.
EOT

  $docDir = get_var($docDir,"get_docDir");
  $docDir .= "/" unless $docDir =~ /\/$/;
   unless (-d $docDir) {print "\n $docDir doesn\'t seem to exist!\nPlease try again.\n"; get_docDir();}
  #return $docDir."W3Browse";
  return $docDir;
}

#-----------------------------------------------------------------------------
sub get_scriptDir {
  print $separator;

  print <<EOT;
** What is the full path to the directory where you want the Browse 
** CGI directory? Example: if you respond  "/www/cgi-bin/", the Browse 
** cgi directory will be installed in "/www/cgi-bin/W3Browse/". You
** will be prompted again before actual installation.
EOT
  $scriptDir = get_var($scriptDir,"get_scriptDir");
  $scriptDir .= "/" unless $scriptDir =~ /\/$/;
  my $path;
  unless (-d $scriptDir) {print "\n $path doesn\'t seem to exist!\nPlease try again.\n"; get_scriptDir();}
  #return $scriptDir."W3Browse";
  return $scriptDir;
}

#-----------------------------------------------------------------------------
sub get_var {
  my ($var,$subname) = @_;
  print " (return if ok) [$var] " if $var;
  my $save_var = $var;
  chomp($var=<STDIN>);
  $var = $save_var unless $var;
  unless ($var) {
    print "\n This value is required. Please try again.\n";
    $var = &$subname;
  }
  return $var; 
}

sub move_files {
  print "\nChecking for existing installation...\n";

  foreach my $file ($scriptDir."lib/Browse/Config.pm",
                    $scriptDir."databases.dat", 
                    $scriptDir."w3browse-form-tables.html", 
                    $scriptDir."inc_coordstime.html", 
                    $scriptDir."inc_missions2.html", 
                    $scriptDir."inc_tabletypes.html", 
                    $scriptDir."inc_results.html", 
                    $scriptDir."inc_submit.html", 
                    $scriptDir."inc_otherlinks.html", 
                    $docDir."w3blaunch.html", 
                    $docDir."w3browse.css", 
                    $docDir."w3browseprint.css", 
                    $docDir."w3browsetable.css", 
                    $docDir."w3header.html", 
                    $docDir."w3footer.html", 
                   ) { 
     if (-s $file or -d $file) {
         print "Found existing $file.  \n  Renaming to $file.keep.\n";
         print "You should diff $file and $file.keep to see if we've " .
		"made changes that you need to incorporate into your $file.\n";
         rename("$file","$file.keep") or 
         Error("Warning! Can't rename $file to $file.keep! : $!\n");
     }
  }

  print "\nCopying files...\n";
  my $sDirTest=`mkdir $scriptDir/W3Browse` 
	unless (-x "$scriptDir/W3Browse");
  my $dDirTest=`mkdir $docDir/W3Browse` 
	unless (-x "$docDir/W3Browse");
  print "Copying htdocs/cgi-bin/W3Browse/ to $scriptDir/W3Browse...\n";
  system("cp -R htdocs/cgi-bin/W3Browse/ $scriptDir/W3Browse") && 
       Error("Can't cp -R cgi-bin/W3Browse $scriptDir/W3Browse ! $@\n");
  print "Changing permissions...\n";
  system("chmod 775 $scriptDir/W3Browse/*.pl");
  $@='';
  print "Copying htdocs/W3Browse/ to $docDir...\n";
  system("cp -R htdocs/W3Browse/ $docDir") && 
	Error("Can't cp -R htdocs/W3Browse $docDir ! $@\n");
  $@='';
  print "Copying Astro library to $scriptDir/lib...\n";
  system("cp -R htdocs/cgi-bin/lib/heasarc/ $scriptDir/lib") && 
	Error("Can't cp -R htdocs/cgi-bin/lib/heasarc/ $scriptDir/lib ! $@\n");
  $@='';
  if (-x "$::cacheDir/W3Browse/dbm $scriptDir/W3Browse/dbm" ) {
     unlink "$::cacheDir/W3Browse/dbm $scriptDir/W3Browse/dbm"; 
  }
  print "Linking  name resolver cache to $::cacheDir...\n";
  system("ln -s $::cacheDir/W3Browse/dbm $scriptDir/W3Browse/dbm") && 
	Error("Can't ln -s $::cacheDir/W3Browse/dbm $scriptDir/W3Browse/dbm ! $@\n");
  
  $@='';
  print "Copying htdocs/Images/ to $docDir...\n";
  system("cp -R htdocs/Images/ $docDir") && 
	Error("Can't cp -R htdocs/Images $docDir ! $@\n");
}

#-----------------------------------------------------------------------------
sub quit {
   print "\nOkay, but installation is not complete. Rerun \"$^X $0\" to try again.\n\n";
   exit;
}
sub checkPerl {
#**********************************************************************
#--- Check for required Perl libraries
#
print "\nRequired Perl modules will be searched for in the following " .
"directories:\n@INC\n";

print "\n** Do you want to add any directories to this list? (y/n) [n] ";
chomp($ans=<STDIN>);  
$ans="n" unless $ans;
my $extraDirs;
my $extraDirsConfig=$_;
if ($ans =~ /y/i) {
   print "\n** Enter additional directories separated by a comma ";
   chomp($extraDirs=<STDIN>);  
   if ($extraDirs) {
      $_=$extraDirs;
      s/,/","/;
      s/\s//;
      $extraDirsConfig=$_;
   
      print "Adding $extraDirs\n";
      @::browselibs=split(/,/,$extraDirs);
   
      unshift (@INC,@::browselibs);

      print "\nPerl path is now: @INC\n\n";
      $::configChanges .= "\n" .'@::browselibs' . 
   	"= (\"$extraDirsConfig\")\n"  .
	"unshift(". '@INC' . "," . '@::browselibs' .")\n";


        print "\n\nconfig=$::configChanges";
   } else {
      print "\n>>No directories added to current Perl library path\n";
   }
} 
}
